//  crm114.c  - Controllable Regex Mutilator base declarations, version X0.1
//  Copyright 2001  William S. Yerazunis, all rights reserved.
//  
//  This software is licensed to the public under the Free Software
//  Foundation's GNU GPL, version 2.1.  You may obtain a copy of the
//  GPL by visiting the Free Software Foundations web site at
//  www.fsf.org, and a copy is included in this distribution.  
//
//  Other licenses may be negotiated; contact the 
//  author for details.  
//
//  include some standard files
#include "crm114_sysincludes.h"

//  include any local crm114 configuration file
#include "crm114_config.h"

//  include the crm114 data structures file
#include "crm114_structs.h"

//  and include the routine declarations file
#include "crm114.h"

//    the command line argc, argv
static int prog_argc;
static char **prog_argv;

//    the auxilliary input buffer (for WINDOW input)
static char *newinputbuf;

//    the globals used when we need a big buffer  - allocated once, used 
//    wherever needed.  These are sized to the same size as the data window.
static char *inbuf;
static char *outbuf;
static char *tempbuf;

int main (int argc, char **argv)
{
  
  int i;    //  some random counters, when we need a loop
  int status;
  int openbracket;  //  if there's a command-line program... 
  int openparen = -1;     //  if there's a list of acceptable arguments
  int user_cmd_line_vars = 0;  // did the user specify --vars on cmdline?
  
  //  printf (" args: %d \n", argc);
  //  for (i = 0; i < argc; i++)
  //    fprintf (stderr, " argi: %d, argv: %s \n", i, argv[i]);
  
  //   copy argc and argv into global statics...
  prog_argc = argc;
  prog_argv = argv;
  
  vht_size = DEFAULT_VHT_SIZE;
  cstk_limit = DEFAULT_CSTK_LIMIT;
  max_pgmlines = DEFAULT_MAX_PGMLINES;
  data_window_size = DEFAULT_DATA_WINDOW;
  user_trace = DEFAULT_USER_TRACE_LEVEL;
  internal_trace = DEFAULT_INTERNAL_TRACE_LEVEL;
  sparse_spectrum_file_length = DEFAULT_SPARSE_SPECTRUM_FILE_LENGTH;
  ignore_environment_vars = 0;
  debug_countdown = -1;
  cycle_counter = 0;
  cmdline_break = -1;
  profile_execution = 0;
  q_expansion_mode = 0;
  //    allcate and initialize the initial root csl (control stack
  //    level) cell.  We do this first, before command-line parsing,
  //    because the command line parse fills in a lot of the first level csl.
  
  csl = (CSL_CELL *) malloc (sizeof (CSL_CELL));
  if (!csl)
    untrappableerror ("Couldn't malloc the csl.  Big problem!\n","");
  csl -> filename = NULL;
  csl -> filedes = -1; 
  csl -> rdwr = 0;   //  0 means readonly, 1 means read/write
  csl -> nchars = 0;
  csl -> mct = 0;
  csl -> cstmt = 0;
  csl -> nstmts = 0;
  csl -> preload_window = 1;
  csl -> caller = NULL;
  csl -> aliusstk[0]  = 0;  // this gets initted later. 
  
  openbracket = -1;
  openparen = -1;
  
  //   and allocate the argparse block
  apb = (ARGPARSE_BLOCK *) malloc (sizeof (ARGPARSE_BLOCK));
  if (!apb)
    untrappableerror ("Couldn't malloc apb.  This is very bad.\n","");
  
  //   Parse the input command arguments
  
  //  user_trace = 1;
  //internal_trace = 1;
  
  for (i = 1; i < argc; i++)
    { 
      // fprintf (stderr, "Arg %d = '%s' \n", i, argv[i]);
      //   is this a plea for help?
      if (
	  (strncmp (argv[i], "-?", 2) == 0)
	  || (strncmp (argv[i], "-h", 2) == 0)
	  || (argc == 1) )
	{
	  fprintf (stderr, " CRM114 version %s . \n", VERSION);
	  fprintf (stderr, " Copyright 2001-2003 W.S.Yerazunis.\n");
	  fprintf (stderr, " This software is licensed under the GPL with ABSOLUTELY NO WARRANTY\n");
	  fprintf (stderr, "     For language help, RTFRM. \n");
	  fprintf (stderr, "     Command Line Options:\n");
	  fprintf (stderr, " -{statements}   executes statements\n");
	  fprintf (stderr, " -b nn   sets a breakpoint on stmt nn\n");
	  fprintf (stderr, " -d nn   run nn statements, then drop to debug\n");
	  fprintf (stderr, " -e      ignore environment variables\n");
	  fprintf (stderr, " -h      this help\n");
	  fprintf (stderr, " -p      profile statement times\n");
	  fprintf (stderr, " -P nn   max program lines\n");
	  fprintf (stderr, " -q m    mathmode (0,1 alg/RPN in EVAL, 2,3 alg/RPN everywhere)\n"); 
	  fprintf (stderr, " -s nn   sparse spectra (.css) featureslots \n");
	  fprintf (stderr, " -S nn   round up to 2^N+1 .css featureslots \n");
	  fprintf (stderr, " -t      user trace mode on\n");
	  fprintf (stderr, " -T      implementors trace mode on\n");
	  fprintf (stderr, " -u dir  chdir to directory \n");
	  fprintf (stderr, " -v      print version ID and exit \n");
	  fprintf (stderr, " -w nn   max data window size ( bytes ) \n");
	  fprintf (stderr, " --      end of CRM114 flags; start user args\n");
	  fprintf (stderr, " --foo   creates var :foo: with value 'SET'\n"); 
	  fprintf (stderr, " --x=y   creates var :x: with value 'y'\n");
	  if (openparen > 0)
	    {
	      fprintf (stderr, "\n This program also claims to accept these command line args:" );
	      fprintf (stderr, "\n  %s\n", &argv[openparen][1] );
	    };
	  exit (EXIT_SUCCESS);
	}
      
      //  -- means "end of crm114 flags" - remainder of args goes to 
      //  the program alone.
      if (strncmp (argv[i], "--", 2) == 0  && strlen (argv[i]) == 2)
	{
	  if (user_trace > 0)
	    fprintf (stderr, "system flag processing ended at arg %d .\n", i);
	  i = argc;
	  goto end_command_line_parse_loop;
	}
      if (strncmp (argv[i], "--", 2) == 0 && strlen (argv[i]) > 2)
	{
	  if (user_trace > 0)
	    fprintf (stderr, "Commandline set of user variable at %d '%s'.\n",
		     i, argv[i]);
	  if (user_cmd_line_vars == 0) user_cmd_line_vars = i;
	  goto end_command_line_parse_loop;
	};
      //   set debug levels
      if (strncmp (argv[i], "-t", 2) == 0 && strlen(argv[i]) == 2)
	{
	  user_trace++;
	  if (user_trace > 0)
	    {
	      fprintf (stderr, "Setting usertrace level to %ld\n", user_trace);
	    };
	  goto end_command_line_parse_loop;
	};
      
      if (strncmp (argv[i], "-T", 2) == 0 && strlen(argv[i]) == 2)
	{
	  internal_trace++;
	  if (user_trace > 0 )
	  fprintf (stderr, "Setting internaltrace to %ld\n", internal_trace);
	  goto end_command_line_parse_loop;
	};
      
      if (strncmp (argv[i], "-p", 2) == 0 && strlen(argv[i]) == 2)
	{
	  profile_execution = 1;
	  if (user_trace > 0 )
	    fprintf (stderr, "Setting profile_execution to 1" );
	  goto end_command_line_parse_loop;
	};
      
      //   is this a change to the maximum number of program lines?
      if (strncmp (argv[i], "-P", 2) == 0 && strlen(argv[i]) == 2)
	{
	  i++;    // move to the next arg
	  if (i < argc)
	    sscanf (argv[i], "%ld", &max_pgmlines);
	  if (user_trace > 0)
	    fprintf (stderr, "Setting max prog lines to %ld\n", max_pgmlines);
	  goto end_command_line_parse_loop;
	};
      
      //   is this a change to the math mode (0,1 for alg/RPN but only in EVAL,
      //   2,3 for alg/RPN everywhere.
      if (strncmp (argv[i], "-q", 2) == 0 && strlen(argv[i]) == 2)
	{
	  i++;    // move to the next arg
	  if (i < argc)
	    sscanf (argv[i], "%ld", &q_expansion_mode);
	  if (user_trace > 0)
	    {
	      fprintf (stderr, "Setting math mode to %ld ", q_expansion_mode);
	      if (q_expansion_mode == 0) 
		fprintf (stderr, "(algebraic, only in EVAL\n");
	      if (q_expansion_mode == 1) 
		fprintf (stderr, "(RPN, only in EVAL\n");
	      if (q_expansion_mode == 2) 
		fprintf (stderr, "(algebraic, in all expressions)\n");
	      if (q_expansion_mode == 3) 
		fprintf (stderr, "(RPN, in all expressions)\n");
	    };
	  goto end_command_line_parse_loop;
	};
      
      //   change the size of the maximum data window we'll allow
      if (strncmp (argv[i], "-w", 2) == 0 && strlen(argv[i]) == 2)
	{
	  i++;    // move to the next arg
	  if (i < argc)
	    sscanf (argv[i], "%ld", &data_window_size);
	  if (data_window_size < 8192)
	    {
	      fprintf (stderr, "Sorry, but the min data window is 8192 bytes");
	      data_window_size = 8192;
	    };
	  if (user_trace > 0)
	    fprintf (stderr, "Setting max data window to %ld chars\n", 
		     data_window_size);
	  goto end_command_line_parse_loop;
	};
      
      //   change the size of the sparse spectrum file default.
      if (strncasecmp (argv[i], "-s", 2) == 0 && strlen(argv[i]) == 2)
        {
          i++;    // move to the next arg
	  if (i < argc &&
	      sscanf (argv[i], "%ld", &sparse_spectrum_file_length))
	    {
	      if (strcmp (argv[i-1], "-S") == 0)
		{
		  long k;
		  
		  k=(long) floor(log10(sparse_spectrum_file_length-1)
				 / log10(2));
		  while ( (2<<k)+1 < sparse_spectrum_file_length)
		    {
		      k++;
		    };
		  sparse_spectrum_file_length=(2<<k)+1;
		};
            }
	  else
	    {
	      fprintf (stderr, "On -s flag: Missing or incomprehensible .CSS file length.\n");
	      exit (EXIT_FAILURE);
	    };
	  
	  if (user_trace > 0)
	    fprintf (stderr, "Setting sparse spectrum length to %ld bins\n", 
		     sparse_spectrum_file_length );
	  goto end_command_line_parse_loop;
	};
      
      //   set a break from the command line
      if (strncmp (argv[i], "-b", 2) == 0 && strlen(argv[i]) == 2)
	{
	  i++;    // move to the next arg
	  if (i < argc)
	    sscanf (argv[i], "%ld", &cmdline_break);
	  if (user_trace > 0)
	    fprintf (stderr, "Setting the command-line break to line %ld\n", 
		     cmdline_break);
	  goto end_command_line_parse_loop;
	};
      
      //   set countdown cycles before dropping to debugger
      if (strncmp (argv[i], "-d", 2) == 0 && strlen(argv[i]) == 2)
	{
	  i++;    // move to the next arg
	  debug_countdown = 0;
	  if (i < argc)
	    sscanf (argv[i], "%ld", &debug_countdown);
	  if (user_trace > 0)
	    fprintf (stderr, "Setting debug countdown to %ld statements\n", 
		     debug_countdown);
	  if (debug_countdown == 0)   //  if next arg wasn't numeric, back up
	    i-- ;  
	  goto end_command_line_parse_loop;
	};
      
      //   ignore environment variables?
      if (strncmp (argv[i], "-e", 2) == 0 && strlen(argv[i]) == 2)
	{
	  ignore_environment_vars++;
	  if (user_trace > 0)
	    fprintf (stderr, "Ignoring environment variables\n");
	  goto end_command_line_parse_loop;
	};
      
      // is this to set the cwd? 
      if (strncmp (argv[i], "-u", 2) == 0 && strlen(argv[i]) == 2) 
	{ 
	  i++;    // move to the next arg 
          if (user_trace) 
	    fprintf (stderr, "Setting WD to %s\n",argv[i]); 
          if ( i >= argc )
	    {
	      fprintf (stderr, "The -u working-directory change needs an arg");
	      goto end_command_line_parse_loop;
	    };
	  if ( chdir(argv[i] ))
	    { 
	      fprintf (stderr, "Sorry, couldn't do a chdir to %s \n", argv[i]); 
	    };
	  goto end_command_line_parse_loop;
	};
      
      if (strncmp (argv[i], "-v", 2) == 0 && strlen(argv[i]) == 2)
	{
	  fprintf (stderr, " This is CRM114, version %s\n", VERSION);
	  fprintf (stderr, " Copyright 2001-2003 W.S.Yerazunis.\n");
	  fprintf (stderr, " This software is licensed under the GPL with ABSOLUTELY NO WARRANTY\n");
	  exit( EXIT_SUCCESS );
	};
      
      if (strncmp (argv[i], "-{", 2) == 0)  //  don't care about the "}"
	{
	  if (user_trace)
	    fprintf (stderr, "Command line program at arg %d\n", i);
	  openbracket = i;
	  goto end_command_line_parse_loop;
	};
      
      //
      //      What about -( var var var ) cmdline var restrictions?
      if (strncmp (argv[i], "-(", 2) == 0 )
	{
	  if (user_trace)
	    fprintf (stderr, "Allowed command line arg list at arg %d\n", i);
	  openparen = i;
	  //
	  //      If there's a -- at the end of the arg, lock out system 
	  //      flags as though we hit a '--' flag.
	  //      (i.e. no debugger.  Minimal security. No doubt this is
	  //      circumventable by a sufficiently skilled user, but
	  //      at least it's a start.)
	  if ( strncmp ("--", &argv[i][strlen(argv[i])-2], 2) == 0)
	    {
	      if (user_trace)
		fprintf (stderr, "cmdline arglist also locks out sysflags.\n");
	      i = argc;
	    };
	  goto end_command_line_parse_loop;
	};
      
      //  that's all of the flags.  Anything left must be
      //  the name of the file we want to use as a program
      //  BOGOSITUDE - only the FIRST such thing is the name of the
      //  file we want to use as a program.  The rest of the args 
      //  should just be passed along 
      if (csl->filename == NULL)
	{
          if (strlen(argv[i]) > MAX_FILE_NAME_LEN)
	    untrappableerror ("Invalid filename, ", "filename too long.");
	  csl->filename = argv[i];
	  if (user_trace > 0)
	    fprintf (stderr, "Using program file %s\n", csl->filename);
	};
    end_command_line_parse_loop:
      if (internal_trace)
	fprintf (stderr, "End of pass %d through cmdline parse loop\n",
		 i);
    }
  
  
  //
  //     Did we get a program filename?  If not, look for one.
  //     At this point, accept any arg that doesn't start with a - sign
  //
  if (csl->filename == NULL && openbracket < 1)
    {
      if (internal_trace)
	fprintf (stderr, "Looking for _some_ program to run...\n");
      for (i = 1; i < argc; i++)
	if (argv[i][0] != '-')
	  {
           if (strlen(argv[i]) > MAX_FILE_NAME_LEN)
	     untrappableerror ("Couldn't open the file, ",
			       "filename too long.");
	   csl->filename = argv[i];
	   i = argc;
	  }
      if (user_trace > 0)
	fprintf (stderr, "Using program file %s\n", csl->filename);
    };
  //      If we still don't have a program, we're _pissed_.  Squalk an
  //      error.
  if (csl->filename == NULL && openbracket < 0)
    {
      fprintf (stderr, "\nCan't find a file to run,");
      fprintf (stderr, "or a command-line to execute. \nI give up.");
      fprintf (stderr, "... (exiting)\n");
      exit (EXIT_SUCCESS);
    };
  
  //     open, stat and load the program file
  if (openbracket < 0 )
    {
      {
	if (argc <= 1)
	  {
	    fprintf (stderr, "CRM114 version %s \n", VERSION);
	    fprintf (stderr, "Try 'crm <progname>', or 'crm -h' for help\n");
	    exit (EXIT_SUCCESS);
	  }
	else
	  {
	    if (user_trace)
	      fprintf (stderr, "Loading program from file %s\n",
		       csl->filename);
	    crm_load_csl (csl);
	  };
      };
    }
  else
    {
      //   if we got here, then it's a command-line program, and
      //   we should just assemble the proggie from the argv [openbracket] 
      csl->filename = NULL;
      csl->filetext = (char *) malloc (sizeof (char) * data_window_size);
      if (!csl->filetext)
	untrappableerror ("Couldn't malloc csl->filetext space (where I was going to put your program.\nNo program space, no can run.  Sorry.","");
      csl->filetext[0] = '\000';
      strcat (csl->filetext, "\n");
      //     the [1] below gets rid of the leading - sign
      strcat (csl->filetext, &(argv[openbracket][1]));
      strcat (csl->filetext, "\n");
      strcat (csl->filetext, "\n");
      csl->nchars = strlen (csl->filetext);
    };
  
  //  We get another csl-like data structure,
  //  which we'll call the cdw, which has all the fields we need, and
  //  simply allocate the data window of "adequate size" and read
  //  stuff in on stdin.
  
  cdw = malloc (sizeof (CSL_CELL));
  if (!cdw)
    untrappableerror ("Couldn't malloc cdw.\nThis is very bad.","");
  cdw->filename = NULL;
  cdw->rdwr = 1;
  cdw->filedes = -1;
  cdw->filetext = malloc (sizeof (char) * data_window_size);
  if (!cdw->filetext)
    untrappableerror ("Couldn't malloc cdw->filetext.\nWithout this space, you have no place for data.  Thus, we cannot run.","");
  //      also allocate storage for the windowed data input
  newinputbuf = malloc (sizeof (char) * data_window_size);
  
  //      and our three big work buffers - these are used ONLY inside
  //      of a single statement's execution and do NOT ever contain state
  //      that has to exist across statements.
  inbuf = malloc (sizeof (char) * data_window_size);
  outbuf = malloc (sizeof (char) * data_window_size);
  tempbuf = malloc (sizeof (char) * data_window_size);
  if (!tempbuf || !outbuf || !inbuf || !newinputbuf)
    untrappableerror (
		      "Couldn't malloc one or more of newinputbuf,inbuf,outbuf,tempbuf.\nThese are all necessary for operation.  We can't run.","");
  //     Initialize the VHT, add in a few predefined variables
  //
  crm_vht_init(argc, argv);
  
  //    Call the pre-processor on the program
  
  status = crm_preprocessor (csl, 0);
  
  //    Now, call the microcompiler on the program file.
  status = crm_microcompiler ( csl, vht);
  //    Great - program file is now mapped via csl->mct
  
  //  If the windowflag == 0, we should preload the data window.  Now,
  //  let's get some data in.  
  
  //    and fill up the data window with stdin until we hit EOF
  i = 0;
  if (csl->preload_window)
    {
      while (!feof (stdin) && i < data_window_size - 1)
	{
	  cdw->filetext[i] = fgetc (stdin);
	  i++;
	};
      i-- ;  //     get rid of the extra ++ on i from the loop; this is the
      //            EOF "character" which prints like an umlauted-Y.
    };
  cdw->filetext[i] = '\000';
  cdw->nchars = i;
  cdw->hash = strnhash (cdw->filetext, cdw->nchars);
  cdw->mct = NULL;
  cdw->nstmts = -1;
  cdw->cstmt = -1;
  cdw->caller = NULL;
  
  // and put the initial data window suck-in contents into the vht 
  //  with the special name :_dw: 
  //
  //   GROT GROT GROT  will have to change this when we get rid of separate
  //   areas for the data window and the temporary area.  In particular, the
  //   "start" will no longer be zero.  Note to self: get rid of this comment
  //   when it gets fixed.  Second note to self - since most of the insert
  //   and delete action happens in :_dw:, for efficiency reasons perhaps
  //   we don't want to merge these areas.
  //
  {
    long dwname;
    long dwlen;
    dwlen = strlen (":_dw:");
    dwname = tdw->nchars;
    strcat (tdw->filetext, ":_dw:"); 
    tdw->nchars = tdw->nchars + dwlen;
    strcat (tdw->filetext, "\n");
    tdw->nchars++;
    crm_setvar ( NULL, 
		 0, 
		 tdw->filetext,
		 dwname,
		 dwlen,
		 cdw->filetext,
		 0,
		 cdw->nchars,
		 -1);
  };
  //    Now we're here, we can actually run!
  //    set up to start at the 0'th statement (the start)
  csl->cstmt = 0;
  
  crm_invoke (csl, cdw);
  
  return (EXIT_SUCCESS);
  
 }


//   Get a file into a memory buffer.  We can either prep to execute
//    it, or use it as read-only data, or as read-write data.
//
int crm_load_csl (CSL_CELL *csl)
{
  
  struct stat statbuf;          //  status buffer - for statting files
  int i;

  //   open it first
  csl->filedes = -1;
  if (csl->rdwr)
    {
      csl->filedes = open (csl->filename, O_RDWR);
    }
  else
    {
      csl->filedes = open (csl->filename, O_RDONLY);
    };

  if (csl->filedes < 0)
    {
      if (csl->filedes == ENAMETOOLONG)
        untrappableerror ("Couldn't open the file, ","filename too long.");
      else
        untrappableerror ("Couldn't open the file: ",
			  csl->filename );
    };
  
  if (internal_trace > 0)
    fprintf (stderr, "file open on file descriptor %ld \n", csl->filedes);
  
  // and stat the file descriptor
  fstat (csl->filedes, &statbuf);
  csl->nchars = statbuf.st_size;
  if (internal_trace > 0)
    fprintf (stderr, "file is %ld bytes\n", csl->nchars);

  //   and read in the source file
  csl->filetext = (char *) malloc ( (csl->nchars + data_window_size) 
				    * sizeof (char));
  if (csl->filetext == NULL) 
    untrappableerror ("malloc of the file text failed","" );
  if (internal_trace > 0)
    fprintf (stderr, "File text malloc'd at %lX \n", (long int) csl->filetext);

  //     read the file in...
  i = read (csl->filedes, csl->filetext, csl->nchars);

  //     and put a cr and then a null at the end.
  csl->filetext[i] = '\n';
  i++;
  csl->filetext[i] = '\n';
  i++;
  csl->filetext[i] = '\000';
  csl->nchars = i;

  csl->hash = strnhash (csl->filetext, csl->nchars);
  if (user_trace)
    fprintf (stderr, "Hash of program: %lX \n", csl->hash);
 
  return 0;
}


//      The CRM114 microcompiler.  It takes in a paritally completed
//      csl struct which is the program, the program length, and
//      returns a completed microcompile table for that particular
//      program.  Side effect: it also sets some variables in the
//      variable hash table (a.k.a. the VHT)

int crm_microcompiler ( CSL_CELL *csl, VHT_CELL ** vht )
{
  //    ***   This is the CRM114 microcompiler.  It's a 5-pass
  //   compiler, but each pass is really simple.
  //   pass 1) count lines and allocate microcompile table
  //   pass 2) run thru file, matching on first word in statement, setting
  //        statement type code.  If it's a label statement, also add 
  //	    an entry to the variables hash table.  If the statement
  //	    assigns a value, we _don't_ put the value itself into the
  //	    variable hash table until we actually execute the statement
  //   pass 3) run thru file, setting bracket nesting level.  If
  //	    bracket level ever goes below 0 or is nonzero at the end
  //	    of the file, issue a warning.
  //   pass 4) run thru file, setting FAIL and LIAF targets

  //  i, j, and k are random beat-upon-me longs
  long i, j, k;   

  //  a counter to use when iterating thru statements
  long stmtnum;

  //  number of statements actually used
  long numstmts;

  //   how many chars in this program
  long pgmlength;

  //   pointer to the chars
  char *pgmtext;

  // how deep a nesting of brackets does this file have?
  long bracketlevel;

  // index of first char of this statement
  long sindex;

  // index of first nonblank character in the line?
  long nbindex;

  // index of first character in the arguments to any particular statement
  long aindex;

  // length of this statement
  long slength;

  //  have we seen an action statement yet?
  long seenaction ;

  if (internal_trace > 0)
    fprintf (stderr, "Starting phase 1 of microcompile.\n");

  seenaction = 0;
  pgmlength = csl->nchars;
  pgmtext = csl->filetext;

  //  ***  Microcompile phase 1 ****
  //  *** Allocation of the microcompile table 
  
  //  get a line count
  j = 0;   //  j will be our line counter here.
  for (i = 0; i < pgmlength; i++)
    {
      if (  pgmtext[i] == '\n' 
	  || pgmtext[i] == ';'
	  || pgmtext[i] == '{'
	  || pgmtext[i] == '}' )
	j++;
    };

  csl->nstmts = j;

  //  now, allocate the microcompile table
  if (user_trace > 0)
    fprintf (stderr, "Program statements: %ld\n", j);
  
  csl->mct = (MCT_CELL **) malloc (sizeof (MCT_CELL * ) * (csl->nstmts + 10) );
  if (!csl->mct)
    untrappableerror("Couldn't malloc MCT table.\nThis is where your compilation results go, so if we can't compile, we can't run.  Sorry.","");
  
  if (internal_trace > 0)
    fprintf (stderr, "microcompile table at %lX\n", (long) csl->mct);

  //  malloc all of the statement cells.
  for (i = 0; i < csl->nstmts + 10; i++)
    {
      csl->mct[i] = (MCT_CELL *) malloc (sizeof (MCT_CELL));
      if (!csl->mct[i])
	untrappableerror(
		"Couldn't malloc MCT cell. This is very bad.\n","");
    };

  // ***  Microcompile phase 2 - set statement types
  
  //   iterate through the statements, setting types.
  //   i is our character counter

  if (internal_trace > 0)
    fprintf (stderr, "Starting phase 2 of microcompile.\n");

  stmtnum = 0;
  sindex = 0;
  bracketlevel = 0;

  while (stmtnum <= csl->nstmts && sindex < pgmlength)
    {
      //      the strcspan below will fail if there's an unescaped
      //      semicolon embedded in a string (or, for that matter, an
      //      explicit newline).  Fortunately, the preprocessor fixes the 
      //      former and the latter is explicitly prohibited by the language.
      //
      slength = strcspn (&pgmtext[sindex], "\n");

      // allocate and fill in the mct table entry for this line
      csl->mct[stmtnum]->hosttxt = pgmtext;
      csl->mct[stmtnum]->start = sindex;
      csl->mct[stmtnum+1]->start = sindex + slength + 1;
      csl->mct[stmtnum]->stmt_utime = 0;
      csl->mct[stmtnum]->stmt_stime = 0;
      csl->mct[stmtnum]->stmt_type = CRM_BOGUS;
      csl->mct[stmtnum]->nest_level = bracketlevel;
      csl->mct[stmtnum]->fail_index = 0;
      csl->mct[stmtnum]->liaf_index = 0;
      csl->mct[stmtnum]->stmt_break = 0;
      csl->cstmt = stmtnum;
      
      //  skip nbindex to the first nonblank  
      //  GROT GROT GROT  here we define nonblank as values > 0x21
      //  GROT GROT GROT  which absolutely _sucks_ in terms of coding
      //  GROT GROT GROT  portability, but it's what we have.
      nbindex = sindex;
      while (pgmtext[nbindex] < 0x021 && pgmtext[nbindex] != '\000') 
	nbindex++;
      
      if (pgmtext[nbindex] == '\000') 
	csl->mct[stmtnum]->stmt_type = CRM_NOOP;
      
      //  save up the first nonblank char:
      csl->mct[stmtnum]->fchar = nbindex;

      // and set up the start of arguments as well, they start at the first
      // nonblank after the first blank after the command...

      aindex = nbindex;
      while (pgmtext[aindex] > 0x021 && pgmtext[aindex] != '\000')
	aindex++;
      while (pgmtext[aindex] < 0x021 && pgmtext[aindex] != '\000')
	aindex++;

      // but don't skip it all the way into the next statement!
      if (csl->mct[stmtnum+1]->start < aindex)
	aindex = csl->mct[stmtnum+1]->start -1;

      csl->mct[stmtnum]->achar = aindex;

      //  now a big bunch of IF-statements, to find the statement type
      
      ///   GROT GROT GROT should be changed to match any blank statement
      if (strncasecmp (&pgmtext[sindex],"\n", 1) == 0)
	{
	  csl->mct [stmtnum] -> stmt_type = CRM_NOOP;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "exit", 4) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_EXIT;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "{", 1) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_OPENBRACKET;
	  bracketlevel++;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "}", 1) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_CLOSEBRACKET;
	  bracketlevel--;
	  // hack - reset the bracketlevel here, as a bracket is "outside"
	  //  the nestlevel, not inside.
	  csl->mct[stmtnum]->nest_level = bracketlevel;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "goto", 4) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_GOTO;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "match", 5) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_MATCH;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "fail", 4) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_FAIL;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "liaf", 4) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_LIAF;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "accept", 6) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_ACCEPT;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "output", 6) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_OUTPUT;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "window", 6) == 0 )
	{
	  if (seenaction == 0)
	    {
	      csl->preload_window = 0;
	      seenaction = 1;
	    };

	  csl->mct[stmtnum]->stmt_type = CRM_WINDOW;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "alter", 5) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_ALTER;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "eval", 4) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_EVAL;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "learn", 5) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_LEARN;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "classify", 8) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_CLASSIFY;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "isolate", 7) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_ISOLATE;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "syscall", 7) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_SYSCALL;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "input", 5) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_INPUT;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "union", 5) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_UNION;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "intersect", 9) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_INTERSECT;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "hash", 4) == 0 )
	{
	  seenaction = 1;
	  csl->mct[stmtnum]->stmt_type = CRM_HASH;
	  goto end_mct_stmttype;
	};
      
      if (strncasecmp (&pgmtext[nbindex], "alius", 5) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_ALIUS;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "trap", 4) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_TRAP;
	  goto end_mct_stmttype;
	};

      if (strncasecmp (&pgmtext[nbindex], "fault", 5) == 0 )
	{
	  csl->mct[stmtnum]->stmt_type = CRM_FAULT;
	  goto end_mct_stmttype;
	};
      
      //  if we got to here, it's either a label, a
      //   comment, or something bogus...
      
      //   is it a comment?
      if (strncasecmp (&pgmtext[nbindex], "#", 1) == 0)
	{
	  csl->mct [stmtnum] -> stmt_type = CRM_NOOP;
	  goto end_mct_stmttype;
	};
      
      //   is it an Insert statement?  These are not executable either.
      if (strncasecmp (&pgmtext[nbindex], "insert", 6) == 0)
	{
	  csl->mct [stmtnum] -> stmt_type = CRM_NOOP;
	  goto end_mct_stmttype;
	};
      
      //   is it a label ?
      if (strncasecmp (&pgmtext[nbindex], ":", 1) == 0)
	{
	  //  it's either a label or an assignment; if a label, it has
	  //  nothing on the rest of the line.
	  csl->mct [stmtnum] -> stmt_type = CRM_LABEL;
	  k = strcspn (&pgmtext[nbindex+1], ":");
	  crm_setvar ( NULL, -1, pgmtext, nbindex, k+2, 
		       NULL, 0, 0,  stmtnum);
	  goto end_mct_stmttype;
	};
      
    end_mct_stmttype: 

      if (0) // (internal_trace)
	{
	  fprintf (stderr, "\nStmt %3ld type %2d ", 
		 stmtnum, csl->mct[stmtnum]->stmt_type);
	  {
	    long ic;
	    for (ic = csl->mct[stmtnum    ]->start;
		 ic < csl->mct[stmtnum + 1]->start-1 ;
		 ic++)
	      fprintf (stderr, "%c", pgmtext[ic]);
	  };
	};		   

      //    check for bracket level underflow....
      if (bracketlevel < 0)
	fatalerror (" Your program seems to achieve a negative nesting",
		       "level, which is quite likely bogus.");
	  
      // move on to the next statement - +1 to get past the \n
      sindex = sindex + slength + 1;

      stmtnum++;
      
    }  
  
  numstmts = stmtnum - 1;

  //  check to be sure that the brackets close!
  
  if (bracketlevel != 0) 
    nonfatalerror ("\nDang!  The brackets don't match up!\n",
      "Check your source code. ");
  
  
  //  Phase 3 of microcompiler- set FAIL and LIAF targets for each line
  //  in the MCT.
  
  {
    long stack[MAX_BRACKETDEPTH];
    long sdx ;
    
    if (internal_trace > 0) 
      fprintf (stderr, "Starting phase 3 of microcompile.\n");
    
    //  set initial stack values
    sdx = 0;
    stack[sdx] = 0;

    //   Work downwards first, assigning LIAF targets
    for (stmtnum = 0; stmtnum < numstmts; stmtnum++)
      {
	switch (csl->mct[stmtnum]-> stmt_type)
	  {
	  case  CRM_OPENBRACKET:
	    {
	      //  if we're open bracket, we're the new LIAF target,
	      //  but we ourselves LIAF to the previous open bracket.
	      csl->mct[stmtnum]->liaf_index = stack[sdx];
	      sdx++;
	      stack[sdx] = stmtnum;
	    };
	    break;
	  case CRM_CLOSEBRACKET:
	    {
	      //  if we're a close bracket, we LIAF not to the current
	      //  open bracket, but to the one before it, so pop the 
	      //  stack and LIAF there.
	      sdx--;
	      csl->mct[stmtnum]->liaf_index = stack [sdx];
	    };
	    break;
	  default:
	    {
	      //   Most statements use the current liaf 
	      csl->mct[stmtnum] -> liaf_index = stack[sdx];
	    }
	    break;
	  }


      }
    
    //   Work upwards next, assigning the fail targets
    sdx = 0;
    stack[sdx] = numstmts+1;
    for (stmtnum = numstmts; stmtnum >= 0; stmtnum--)
      {
	switch (csl->mct[stmtnum]-> stmt_type)
	  {
	  case  CRM_CLOSEBRACKET:
	    {
	      //  if we're close bracket, we're the new FAIL target,
	      //  but we ourselves FAIL to the next close bracket
	      csl->mct[stmtnum]->fail_index = stack[sdx];
	      sdx++;
	      stack[sdx] = stmtnum;
	    };
	    break;
	  case CRM_OPENBRACKET:
	    {
	      //  if we're an open bracket, we FAIL not to the current
	      //  CLOSE bracket, but to the one before it, so pop the 
	      //  stack and FAIL there.
	      sdx--;
	      csl->mct[stmtnum]->fail_index = stack [sdx];
	    };
	    break;
	  default:
	    {
	      //   Most statements use the current liaf 
	      csl->mct[stmtnum] -> fail_index = stack[sdx];
	    }
	    break;
	  }
      }


    //   Work upwards again, assigning the TRAP targets
    sdx = 0;
    stack[sdx] = numstmts+1;
    for (stmtnum = numstmts; stmtnum >= 0; stmtnum--)
      {
	switch (csl->mct[stmtnum]-> stmt_type)
	  {
	  case  CRM_TRAP:
	    {
	      //  if we're the TRAP statement, we change the TRAP target,
	      //  but we ourselves TRAP to the next TRAP statement
	      csl->mct[stmtnum]->trap_index = stack[sdx];
	      stack[sdx] = stmtnum;
	    };
	    break;
	  case CRM_OPENBRACKET:
	    {
	      //  if we're an open bracket, we trap not to the current
	      //  level's TRAP statement, but to the one before it, so pop the 
	      //  stack and aim TRAP there.
	      sdx--;
	      csl->mct[stmtnum]->trap_index = stack [sdx];
	    };
	    break;
	  case CRM_CLOSEBRACKET:
	    {
	      //  if we're a close bracket, we keep our current trap target
	      //  but move down one level in the stack
	      stack[sdx + 1] = stack [sdx];
	      sdx++;
	      csl->mct[stmtnum]->trap_index = stack [sdx];
	    };
	    break;
	  default:
	    {
	      //   Most statements use the current TRAP level
	      csl->mct[stmtnum] -> trap_index = stack[sdx];
	    }
	    break;
	  }
      }
    
    //  print out statement info if desired
    if ( user_trace )
      {
	for (stmtnum = 0; stmtnum <= numstmts; stmtnum++)
	  {
	    fprintf (stderr, "\n");
	    fprintf (stderr, "%4.4ld {%2.2d}",
		     stmtnum, 
		     csl->mct[stmtnum]->nest_level);
	    
	    if (internal_trace > 0)
	      {
		fprintf (stderr, " <<%2.2d>>", 
			 csl->mct[stmtnum]->stmt_type);
		
		fprintf (stderr, " L%4.4d F%4.4d T%4.4d", 
			 csl->mct[stmtnum]->liaf_index,
			 csl->mct[stmtnum]->fail_index,
			 csl->mct[stmtnum]->trap_index);
	      }
	    fprintf (stderr, " :  ");
	    
	    //  space over two spaces per indent
	    for (k = 0; k < csl->mct[stmtnum]->nest_level; k++)
	      fprintf (stderr, "  ");

	    //   print out text of the first statement:
	    fprintf (stderr,"-");
	    k = csl->mct[stmtnum]->fchar; 
	    while (pgmtext[k] > 0x021 && k < csl->mct[stmtnum]->achar)
	      {
		fprintf (stderr, "%c", pgmtext[k]);
		k++;
	      };
	    fprintf (stderr,"-");

	    //  and if there are args, print them out as well.
	    if ( csl->mct[stmtnum]->achar < csl->mct[stmtnum+1]->start-1)
	      {
		fprintf (stderr, "=");
		for (k = csl->mct[stmtnum]->achar;
		     k< csl->mct[stmtnum+1]->start-1; k++)
		  fprintf (stderr, "%c", pgmtext[k]);
		fprintf (stderr, "=");
	      } 
	  }
	fprintf (stderr, "\n");
      }

    //    Finally got to the end.  Fill in the last bits of the CSL
    //  with the new information, and return.

    csl->nstmts = numstmts;

    if (internal_trace > 0)
      fprintf (stderr, "microcompile completed\n");
  }

  return (0);
}

//
//        Here it is, the core of CRM114 - the execution engine toplevel,
//      which, given a CSL and a CDW, executes the CSL against the CDW      
//

int crm_invoke (CSL_CELL *csl, CSL_CELL *cdw)
{
  long i, j, k;
  long status;
  long done;
  long slen;

  //     timer1, timer2, and tstmt are for time profiling.
  //
  TMS_STRUCT timer1, timer2;
  long tstmt;

  tstmt = 0;
  i = j = k = 0;
  
  status = 0;

  //    Sanity check - don't try to execute a file before compilation
  if (csl->mct == NULL)
    {
      untrappableerror ( "Can't execute a file without compiling first.\n",
		   "This means that CRM114 is somehow broken.");
    };

  //    empty out the alius stack (nothing FAILed yet.)
  //
  for (i = 0; i < MAX_BRACKETDEPTH; i++)
    csl->aliusstk[i] = 1;
    
  //    if there was a command-line-specified BREAK, set it.
  //    
  if (cmdline_break > 0)
    {
      if (cmdline_break <= csl->nstmts)
	{
	  csl->mct[cmdline_break]->stmt_break = 1;
	};
    };

  if (user_trace > 0)
    fprintf (stderr, "Starting to execute %s at line %ld\n",
	     csl->filename, csl->cstmt);

 invoke_top:

  //   initialize timers ?
  if (profile_execution)
    {
      tstmt = csl->cstmt;
      times ( (void *) &timer1);
    };

  if (csl->cstmt >= csl->nstmts)
    {
      //  OK, we're at the end of the program.  When this happens,
      //  we know we can exit this invocation of the invoker
      if (user_trace > 0 )
	fprintf (stderr, "Finished the program %s.\n", csl->filename);
      done = 1;
      status = 0;
      goto invoke_done;
    };

  slen = (csl->mct[csl->cstmt+1]->fchar) - (csl->mct[csl->cstmt ]->fchar);

  if (user_trace > 0)
    {
      fprintf (stderr, "\nParsing line %ld :\n", csl->cstmt);
      fprintf (stderr, " -->  ");
      for (i = 0; i < slen; i++)
	fprintf (stderr, "%c", 
		 csl->filetext[csl->mct[csl->cstmt]->fchar+i]);
    };

  //   Invoke the common declensional parser on the statement only if it's
  //   an executable statement.
  //
  switch ( csl->mct[csl->cstmt]->stmt_type )
    { 
      //
      //   Do the processing that all statements need (well, _almost_ all.)
      //
    case CRM_LABEL:
    case CRM_NOOP:
	break;
    default:
      i = crm_statement_parse (  &(csl->filetext[csl->mct[csl->cstmt]->fchar]),
			     slen,
			     apb);
      //    and run the standard flag parser against the flags found (if any)
      //
      {
	char flagz[1024];
	long fl;
	fl = 1024;
	crm_get_pgm_arg (flagz, fl, apb->a1start, apb->a1len);
	//    fprintf (stderr, "flagz --%s-- len %d\n", flagz, strlen(flagz));
	apb->sflags = crm_flagparse (flagz, strlen (flagz));
      };
      break;
    };
  
  //    and maybe drop into the debugger?
  //
  cycle_counter++;
  if (debug_countdown > 0) debug_countdown--;
  if (debug_countdown == 0
      || csl->mct[csl->cstmt]->stmt_break == 1 )
    {
      i = crm_debugger ();
      if (i == -1)
	exit ( EXIT_SUCCESS );
      if (i == 1)
	goto invoke_top;
    };

  if (user_trace > 0)
    {
      fprintf (stderr, "\nExecuting line %ld :\n", csl->cstmt);
    };


  //  so, we're not off the end of the program (yet), which means look
  //  at the statement type and see if it's somethign we know how to
  //  do, otherwise we make a nasty little noise and continue onward.
  //  Dispatch is done on a big SWITCH statement
  
  switch ( csl->mct[csl->cstmt]->stmt_type )
    {
    case CRM_NOOP:
    case CRM_LABEL:
      {
	if (user_trace)
	  fprintf (stderr, "Statement %ld is non-executable, continuing.\n",
		   csl->cstmt);
      }
      break;

    case CRM_OPENBRACKET:
      {
	//  the nest_level+1 is because the statements in front are at +1 depth
	csl->aliusstk [ csl->mct [ csl->cstmt] -> nest_level+1 ] = 1;
	if (user_trace)
	  fprintf (stderr, "Statement %ld is an openbracket. depth now %d.\n",
		   csl->cstmt, 1 + csl->mct [ csl->cstmt]->nest_level);
      }
      break;

    case CRM_CLOSEBRACKET:
      {
	if (user_trace)
	  fprintf (stderr, "Statement %ld is a closebracket. depth now %d.\n",
		   csl->cstmt, csl->mct[ csl->cstmt]->nest_level);
      }
      break;
      
    case CRM_BOGUS:
      {
        char bogusbuffer[1024];
	char bogusstmt [1024];
        sprintf (bogusbuffer, "Statement %ld is bogus!!!  Here's the text: \n",
 		 csl->cstmt);
	memmove (bogusstmt, 
		&csl->filetext[csl->mct[csl->cstmt]->start],
		csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start);
	bogusstmt [csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start]
	  = '\000';
        fatalerror (bogusbuffer, bogusstmt);
      }
      break;

    case CRM_EXIT:
      {
	int retval; long retlen;
	char retstr [MAX_PATTERN];
	crm_get_pgm_arg (retstr, MAX_VARNAME, apb->s1start, apb->s1len);
	retlen = apb->s1len;
	retlen = crm_nexpandvar (retstr, retlen, MAX_VARNAME);
	retval = 0;
	if (retlen > 0)
	  sscanf (retstr, "%d", &retval);
	if (user_trace)
	  fprintf (stderr, "Exiting at statement %ld with value %d\n", 
		   csl->cstmt, retval);
	if (profile_execution)
	  crm_output_profile (csl);
	exit (retval);
      }
      break;

    case CRM_RETURN:
      {
	if (user_trace)
	  fprintf (stderr, "Returning to caller at statement %ld\n", 
		   csl->cstmt);
	return (0);
      }
      break;

    case CRM_GOTO:
      {
	char target[MAX_VARNAME];
	long tarlen;
	//  look up the variable name in the vht.  If it's not there, or 
	//  not in our file, call a fatal error.

	crm_get_pgm_arg ( target, MAX_VARNAME, apb->s1start, apb->s1len);
	tarlen = apb->s1len;
	if (internal_trace) 
	  fprintf (stderr, "\n    untranslated label %s , ", 
		   target);

	//   do indirection if needed.
	tarlen = crm_nexpandvar (target, tarlen, MAX_VARNAME);
	if (internal_trace)
	  fprintf (stderr, " translates to %s .", target);

	k = crm_lookupvarline (vht, target, 0, tarlen);

	if (k > 0)
	  {
	    if (user_trace)
	      fprintf (stderr, "GOTO from line %ld to line %ld\n", 
		       csl->cstmt,  k);
	    csl->cstmt = k;
	    //  and going here didn't fail... 
	    csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = 1;
	  }
	else
	  {
	    //  this is recoverable if we have a trap... so we continue
	    //   execution right to the BREAK.
	    fatalerror (" Can't GOTO the nonexistent label: ", target);
	  };
      }
      break;

    case CRM_FAIL:
      {   //       If we get a FAIL, then we should branch to the statement
	//         pointed to by the fail_index entry for that line.
	if (user_trace)
	  fprintf (stderr, "Executing hard-FAIL at line %ld\n", csl->cstmt);
	csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
	csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = -1;
      };
      break;
	
    case CRM_LIAF:
      {   //       If we get a LIAF, then we should branch to the statement
	//         pointed to by the liaf_index entry for that line.
	if (user_trace)
	  fprintf (stderr, "Executing hard-LIAF at line %ld\n", csl->cstmt);
	csl->cstmt = csl->mct[csl->cstmt]->liaf_index;
	goto invoke_top;
      };
      break;

    case CRM_ALIUS:
      {
	//   ALIUS looks at the finish state of the last bracket - if it
	//   was a FAIL-to, then ALIUS is a no-op.  If it was NOT a fail-to,
	//   then ALIUS itself is a FAIL
	if (user_trace)
	  fprintf (stderr, "Executing ALIUS at line %ld\n", csl->cstmt);
	if (csl->aliusstk [csl->mct[csl->cstmt]->nest_level + 1] == 1)
	  {
	    if (user_trace)
	      fprintf (stderr, "prior group exit OK, ALIUS fails forward. \n");
	    csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;  
	  };
      }
      break;

    case CRM_TRAP:
      {
	//  TRAP is a placeholder statement that holds the regex that
	//  the faulting statement must match.  The background support
	//  code is in crm_trigger_fault that will look at the error string
	//  and see if it matches the regex.  
	//
	//  If we get to a TRAP statement itself, we should treat it as
	//  a skip to end of block (that's a SKIP, not a FAIL)

	if (user_trace) 
	  {
	    fprintf (stderr, "Executing a TRAP statement...");
	    fprintf (stderr, " this is a NOOP unless you have a live FAULT\n");
	  }
	csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
      }
      break;

    case CRM_FAULT:
      {
	char *reason;
	char rbuf [MAX_PATTERN];
	long rlen;
	long fresult;
	
	//  FAULT forces the triggering of the TRAP; it's a super-FAIL
	if (user_trace)
	  fprintf (stderr, "Forcing a FAULT at line %ld\n", csl->cstmt);
	crm_get_pgm_arg ( rbuf, MAX_PATTERN, apb->s1start, apb->s1len );
	rlen = crm_nexpandvar (rbuf, apb->s1len, MAX_PATTERN);

	//   We malloc the reason - better free() it when we take the trap.
	//   in crm_trigger_fault
	//
	reason = malloc (rlen + 5);
        if (!reason)
	  untrappableerror(
	    "Couldn't malloc 'reason' in CRM_FAULT - out of memory.\n", "Don't you just HATE it when the error fixup routine gets an error?!?!");
	strncpy (reason, rbuf, rlen+1);
	fresult = crm_trigger_fault (reason);
	if (fresult != 0)
	  fatalerror("Your program has no TRAP for the user defined fault:",
			reason);
      }
      break;

    case CRM_ACCEPT:
      {
	//   Accept:  take the current window, and output it to
	//   standard output.  It's a joyous occasion !!!  :-)
	if (user_trace)
	  fprintf (stderr, "Executing an ACCEPT \n");
	for (i = 0; i < cdw->nchars ; i++)
	  fprintf (stdout, "%c", cdw->filetext[i]);
	fflush (stdout);
      }
      break;

    case CRM_MATCH:
      {
	crm_expr_match (csl, apb);
      }
      break;

    case CRM_OUTPUT:
      {
	long i, j;

	//    output a string, usually to stdout unless otherwise 
	//    specified in the output statement.
	//    
	//    We do variable substitutions here....
	char outfile[MAX_VARNAME];
	long outfilelen;
	long outtextlen;
	char ofn[MAX_VARNAME];
	FILE *outf;

	crm_get_pgm_arg (outfile, MAX_VARNAME,
			 apb->b1start, apb->b1len);
	outfilelen = apb->b1len;

	if (internal_trace)
	  fprintf (stderr," Executing an OUTPUT statement\n");

	outf = stdout;
	if (outfilelen > 0)
	  {
	    //   do variable substitution on the filename.
	    outfilelen = crm_nexpandvar (outfile, outfilelen, MAX_VARNAME);
	    //    and cut out the first non-blank word.
	    crm_nextword ( outfile, outfilelen, 0, &i, &j);
	    if (j > 0)
	      {
		memmove (ofn, &outfile[i], j);
		ofn[j] = '\000';
		if (user_trace)
		  fprintf (stderr, "Outputting to file %s\n", ofn);
		if (strcmp(ofn,"stderr")==0)
		  {
		    outf = stderr;
		  }
		else if (strcmp(ofn,"stdout")!=0)
		  {
		    if (apb->sflags & CRM_APPEND)
		      { outf = fopen (ofn, "a"); }
		    else
		      { outf = fopen (ofn, "w"); };
		  };
	      };
	  };
	
	crm_get_pgm_arg (outbuf, data_window_size,
			 apb->s1start, apb->s1len);
	outtextlen = apb->s1len;
	if (internal_trace)
	  fprintf (stderr, "  outputting with pattern %s\n", outbuf);

	//      Do variable substitution on outbuf.
	outtextlen = crm_nexpandvar (outbuf, outtextlen, data_window_size);
	
	//   and send it to outf
	fwrite (outbuf, outtextlen, 1, outf);
	fflush (outf);
	if (outf != stdout && outf != stderr ) fclose (outf);
      }
      break;
    case CRM_WINDOW:
      {
	//       a window operation - two steps...first is to discard
	//       everything up till the first regex match, and second
	//       is to add more data until the second regex is
	//       satisfied by the incoming data.  We just add the
	//       incoming data onto the back of the window buffer, and
	//       when we get a read completes.
        //       Yes, there are more efficient, less memory-intensive ways
	//       to do this, but this is simple and unlikely to be broken in 
	//       subtle ways.

	static long newbuflen = 0;
	char pch [MAX_PATTERN];

	long srcidx;
	char inputsrc[MAX_VARNAME];
	long inputsrclen;
	char *savedinputtxt;
	long savedinputtxtlen;
	char wvname [MAX_VARNAME];
	long wvnamelen;
	CSL_CELL* mdw;
	long flen;
	int regexflags;
	regex_t preg;
	int inputmode;
	long vmidx;
	regmatch_t matches [2];  //  we're only interested in the first match.

	inputsrc[0] = '\000';
	inputsrclen = 0;
	
	//	wvname[0] = '\000';
	//wvnamelen = 0;

	srcidx = 0;
	savedinputtxt = NULL;
	savedinputtxtlen = 0;

	if (user_trace) 
	  fprintf (stderr, "Executing a 'window' operation\n");


	//      check for the flags
	//
	//   default is BYCHAR
	inputmode = CRM_BYCHAR;
	if (apb->sflags & CRM_BYCHAR)
	  {
	    if (user_trace)
	      fprintf (stderr, "  window input by character ");
	    inputmode = CRM_BYCHAR;
	  };
	if (apb->sflags & CRM_BYLINE)
	  {
	    if (user_trace)
	      fprintf (stderr, "  window input by lines ");
	    inputmode = CRM_BYLINE;
	  };
	if (apb->sflags & CRM_BYEOF)
	  {
	    if (user_trace)
	      fprintf (stderr, "  window input by EOFs");
	    inputmode = CRM_BYEOF;
	  };
	if (apb->sflags & CRM_EOFENDS)
	  {
	    if (user_trace)
	      fprintf (stderr, "  window input by EOFs with EOFending");
	    inputmode = CRM_EOFENDS;
	  };
	regexflags = REG_EXTENDED;
	if (apb->sflags & CRM_NOCASE)
	  {
	    if (user_trace)
	      fprintf (stderr, "  no case matching turned on ");
	    regexflags = regexflags | REG_ICASE;
	  };

	//    part 1: dispose of old window worth of data.  If no match,
	//    dispose of all of the old window.
	//  
	//     get the disposal pattern	
	//
	crm_get_pgm_arg (pch, MAX_PATTERN, apb->s1start, apb->s1len);

	//     null window check - if no cut or paste patterns, then we
	//     just skip to the end of the WINDOW statement code
	//     which is how a WINDOW statement can be used to have a
	//     program "come out running" before reading stdin.
	if (apb->s1len == 0 && apb->s2len == 0)
	  goto crm_window_no_changes_made;

	//     We have the first pattern in pch.  We ought to look for the
	//     appropriate flags here (common code, anyone?) but for now,
	//     we'll just do a brutally straightforward expansion and then
	//     matching.
	
	if (internal_trace)
	  fprintf (stderr, " window cut pattern ---%s---\n", pch);
	flen = apb->s1len;

	//       expand the match pattern
	flen = crm_nexpandvar (pch, apb->s1len, MAX_PATTERN);
	//
	//       compile the regex
	i = crm_regcomp (&preg, pch, flen, regexflags);
	if ( i > 0)
	  {
	    crm_regerror ( i, &preg, tempbuf, data_window_size);
	    nonfatalerror ("Regular Expression Compilation Problem:", tempbuf);
	    goto invoke_bailout;
	  };


	//       We have to be a little careful here.  We can't just
	//       clobber things; we need to do it as though we were
	//       surgically altering a variable (which, as a matter of
	//       fact, we are)

	crm_get_pgm_arg (wvname, MAX_PATTERN, apb->p1start, apb->p1len);
	wvnamelen = crm_nexpandvar (wvname, apb->p1len, MAX_PATTERN);
	//    if no svname, then we're defaulted to :_dw:
	if (strlen (wvname) == 0)
	  {
	    strcat (wvname, ":_dw:");
	  };

	//
	//     Get the input source, if one is supplied (in parens!)
	crm_get_pgm_arg (inputsrc, MAX_PATTERN, apb->p2start, apb->p2len);
	inputsrclen = apb->p2len;
	  

	vmidx = crm_vht_lookup (vht, wvname, 
				strlen (wvname));
	if (vht[vmidx] == NULL)
	  {
	    nonfatalerror ("We seem to be windowing a nonexistent variable.",
			  "How very bizarre.");
	    goto invoke_bailout;
	  }

	mdw = NULL;
	if (vht[vmidx]->valtxt == cdw->filetext) mdw = cdw;
	if (vht[vmidx]->valtxt == tdw->filetext) mdw = tdw;
	if (mdw == NULL)
	  {
	    nonfatalerror ("We seem to have lost the windowed var buffer",
			  "This is just plain sick.");
	    goto invoke_bailout;
	  }

	//       execute the regex.
	// i = regexec (&preg, 
	//	     &(vht[vmidx]->valtxt[vht[vmidx]->vstart]),
	//	     1, matches, 0);
	i = crm_regexec ( &preg, 
			  &(vht[vmidx]->valtxt[vht[vmidx]->vstart]),
			  vht[vmidx]->vlen, 
			  1, matches, 0, NULL);
	crm_regfree (&preg);

	//       starting offset of the "keep section" is at matches[0].rm.eo
	//       so we use crm_slice_and_splice_window to get rid of it.
	//
	if (i == 0)
	  {
	    //     delete everything up to and including the delimiter
	    crm_slice_and_splice_window (mdw,
					 vht[vmidx]->vstart, 
					 -matches[0].rm_eo);
	  }
	else
	  {
	    //  didn't find the terminator pattern at all, which means we
	    //  flush the input window and go look for more.
	    crm_slice_and_splice_window (mdw,
					 vht[vmidx]->vstart, 
					 -vht[vmidx]->vlen);
	  };

	if (user_trace) 
	  fprintf (stderr, "  new variable length after cut is %ld\n", 
		   vht[vmidx]->vlen);

	//       OK, part one is done- we've windowed off the first
	//       part of the input.  Now we read new input ... which
	//       is done by looping on "does the newinput buffer have
	//       the second delimiter?", "if not sleep for a little
	//       while, then clearerr on stdin and read more
	//       characters, else, copy the new characters (including
	//       the second delimiter) into the input data window."
	//       and null terminate.

	crm_get_pgm_arg (pch, MAX_PATTERN, apb->s2start, apb->s2len);
	flen = apb->s2len;
	if (user_trace)
	  fprintf (stderr, "adding input with terminator of --%s--,", pch);

	//       expand the match pattern
	flen = crm_nexpandvar (pch, flen, MAX_PATTERN);
	
	if (user_trace)
	  fprintf (stderr, " which expands to --%s--", pch);
	
	//
	//       compile the match regex 
	i = crm_regcomp (&preg, pch, flen, regexflags);
	if ( i > 0)
	  {
	    crm_regerror ( i, &preg, tempbuf, data_window_size);
	    nonfatalerror ("Regular Expression Compilation Problem:", tempbuf);
	    goto invoke_bailout;
	  };

	//    Now we decide - do we suck input from stdin, or from
	//    a variable that's already here?



	//       now, the loop, on the delimiter regex...
	if (internal_trace)
	  fprintf (stderr, "   prior newinput buf --%s--\n", newinputbuf);
	
	if (apb->p2start)
	  {
	    //     NonZero input source variable, so we're gonna take our input
	    //     from this input variable.
	    inputmode = 0;
	    if (user_trace)
	      fprintf (stderr, " Using input source from variable %s\n",
		       inputsrc);
	  };


	switch (inputmode)
	  {
	  case 0:
	    {
	      //    we're supposed to grab our input from an input variable.
	      //     so we fake it as though it came from a file.
	      //
	      //    Later on, we have to undo the faking, and also modify
	      //    the length of the input variable (cutting out the stuff 
	      //    that went into the WINDOW.

	      srcidx = crm_vht_lookup (vht, inputsrc, inputsrclen);
	      if (vht[srcidx] == NULL) 
		{
		  nonfatalerror ("Trying to take WINDOW input from nonexistent variables doesn't work, in this case, from :", inputsrc);
		  goto invoke_bailout;
		};
	      //
	      //
	      //    malloc up some temporary space to keep the static input
	      //   buffer's stored text
	      savedinputtxt = (char *) 
		malloc (sizeof (char) * (32 + newbuflen ));
	      if (savedinputtxt == NULL)
		{
		  fatalerror ("Malloc failed.  Aw, crud.", "Can't WINDOW this way");
		  goto invoke_bailout;
		};

	      //
	      //    save our newinputbuf txt
	      strncpy (savedinputtxt,
		       newinputbuf,
		       newbuflen);
	      savedinputtxtlen = newbuflen;
	      //
	      //     and push the contents of the variable into newinputbuf
	      strncpy (newinputbuf, 
		       &vht[srcidx]->valtxt[vht[srcidx]->vstart], 
		       vht[srcidx]->vlen );
	      newinputbuf[vht[srcidx]->vlen] = '\000';
	      newbuflen = vht[srcidx]->vlen;
	      //
	      //    start by seeing how much we want to take off the
	      //    front of the inputxt variable
	      if (crm_regexec (&preg, 
			       newinputbuf,
			       newbuflen,
			       1, matches, 0, NULL) )
		//  0 on regex is success, nonzero is fail)
		{
		  //   No joy, we didn't match.  So we have fake
		  //   things up to use the entire inputtxt string as
		  //   input.
		  if (newbuflen > 0)
		    {
		      matches[0].rm_so = 0 ;
		      matches[0].rm_eo = newbuflen;
		    }
		  else
		    {
		      //  No text left either.  This is a FAIL condition
		      matches[0].rm_so = 0;
		      matches[0].rm_eo = 0;
		      csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
		      csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = -1;
		    };
		}
	      //       That's all, we've set up the moves.
	      break;
	    }
	  case CRM_BYCHAR:
	    {
	      int icount;
	      icount = 0;
	      //       set the input terminal stream to unbuffered  
	      //   GROT GROT GROT this seems to be broken - skip for now
	      //   GROT GROT GROT further information - buffering can only
	      //   be changed BEFORE the first I/O.  So, in the genral
	      //   case, this can't work unless we spawn a new process.
	      //   So, that's what we do.
#ifdef MULTIPROCESS_INPUTS
	      {
		int to_reader[2];
		int from_reader[2];
		pid_t reader;
		int status;
		reader = 0;
		to_reader[0] = 0;
		from_reader[1] = 0;
		status = pipe (to_reader);
		status = pipe (from_reader);
		reader = fork();
		//  we're in the reader process here.  We share files,
		//  but it's a separate process.
		close (to_reader[1]);
		close (from_reader[0]);
		dup2 (from_reader[0], fileno (stdout));
		if (user_trace)
		  fprintf (stderr, "Unbuffered input via secondary process\n");
		setvbuf (stdin, NULL, _IONBF, NULL)
		  while (1)
		    {
		      clearerr (stdin);
		      usleep (INPUT_WINDOW_SLEEP_USEC);
		      if 
			
			}
	      }
#endif
	      //	      setvbuf ( stdin, NULL, _IONBF, BUFSIZ);
	      while ( crm_regexec ( &preg, 
				    newinputbuf, 
				    newbuflen, 
				    1, matches, 0, NULL))
		{

		  icount = read (0, &(newinputbuf[newbuflen]), 1); 
		  if (icount == 0) 
		    {
		      clearerr(stdin);
		      usleep (INPUT_WINDOW_SLEEP_USEC);  
		      if (user_trace)
			fprintf (stderr, "...waiting for char input...");
		    };
		  newbuflen = newbuflen + icount;
		  newinputbuf[newbuflen] = '\000'; // put on the terminator
		};
	    }
	    break;
	  case CRM_BYLINE:
	    {
	      while ( crm_regexec ( &preg, 
				    newinputbuf, 
				    newbuflen, 
				    1, matches, 0, NULL))
		{
		  if (user_trace)
		    fprintf (stderr, "...waiting for line input...");
		  usleep (INPUT_WINDOW_SLEEP_USEC);  
		  clearerr(stdin);
		  while ( (newinputbuf[newbuflen] = fgetc (stdin)) != '\n')
		    {
		      newbuflen++;
		    };
		  newbuflen++;
		  newinputbuf[newbuflen] = '\000'; // put on the terminator
		};
	    }
	    break;
	  case CRM_BYEOF:
	    {
	      while ( crm_regexec ( &preg, 
				    newinputbuf, 
				    newbuflen, 
				    1, matches, 0, NULL))
		
		// was while ( regexec (&preg, newinputbuf, 1, matches, 0))
		{
		  if (user_trace)
		    fprintf (stderr, "...waiting for EOF input...");
		  usleep (INPUT_WINDOW_SLEEP_USEC);  
		  clearerr(stdin);
		  while (!feof (stdin))
		    {
		      //newinputbuf[newbuflen] = fgetc (stdin);
		      //newbuflen++;
		      //
		      //      read as much as we can at one whack...
		      newbuflen =+ read ( 0, 
					  &newinputbuf[newbuflen],
					  (data_window_size- (newbuflen+32)));
		    };
		  newbuflen--;     //  get rid of the ^D on EOF
		  newinputbuf[newbuflen] = '\000'; // put on the terminator
		};
	    }
	    break;
	  case CRM_EOFENDS:
	    {
	      //    EOFENDS is slightly different.  We read as much as we
      	      //    can, either to EOF, or to full newinputbuf.  
	      //    Then, we see if we get enough to satisfy the
	      //    regex.  If so, great.  
	      //    IF not, but there's _something_ left there, we take it,
	      //    all of it, right up to EOF.
	      //    IF there was nothing left there, we FAIL out of the loop.
	      //    If you wanted to fail out whenever the input regex didn't
	      //    satisfy, use another MATCH right after the WINDOW.t 
	      //
	      
	      if (user_trace)
		fprintf (stderr, "...waiting for EOFENDS input...");
	      
	      usleep (INPUT_WINDOW_SLEEP_USEC);  
	      clearerr(stdin);
	      while (!feof (stdin) && newbuflen < data_window_size - 10)
		{
		  newinputbuf[newbuflen] = fgetc (stdin);
		  newbuflen++;
		};
	      newbuflen--;     //  get rid of the ^D on EOF
	      newinputbuf[newbuflen] = '\000'; // put on the terminator
	     
	      if ( crm_regexec ( &preg, 
				 newinputbuf, 
				 newbuflen, 
				 1, matches, 0, NULL))
		//  0 on regexec is success, just fall thru.  Else, we
		//   have work to do...
		{
		  //    We didn't find the regex.  So...
		  //    Did we have anything at all in the newinputbuf?
		  if (newbuflen > 0)
		    {
		      //    Yes, there's text left.  We take everything up
		      //    to EOF and stuff it in.   (well, actually
		      //    _fake_ _up_ match`es[0] so it looks like we
		      //    matched though we didn't.
		      matches[0].rm_so = 0;
		      matches[0].rm_eo = newbuflen;
		    }
		  else
		    {
		      //    No, no text left.  We're done.  Execute a 
		      //    FAIL operation.
		      matches[0].rm_so = 0;
		      matches[0].rm_eo = 0;
		      csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
		      csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = -1;
		    };
		};
	    }
	    break;
	  default:
	    {
	      fatalerror ("Internal error in window input mode selection", "");
	      goto invoke_bailout;
	    };
	  };
	
	crm_regfree (&preg);
	
	if (internal_trace)
	  fprintf (stderr, "   now newinput buf --%s--\n", newinputbuf);
	//     Once we get to here, we have the new input in newinputbuf, and
	//     matches[0].rm_eo is the length.  So, we copy the new data onto
	//     the end of the cdw window, and slide the new input up.
	//     
	//     start by making some space at the end of the input buffer

	crm_slice_and_splice_window (mdw,
				     vht[vmidx]->vstart+vht[vmidx]->vlen,
				     matches[0].rm_eo);
	

	//     copy the first part of newinputbuf into the space.
	memmove (&(vht[vmidx]->valtxt[vht[vmidx]->vstart 
				      + vht[vmidx]->vlen
				      - matches[0].rm_eo]),
		 newinputbuf, 
		 matches[0].rm_eo);
 	
	//     and get rid of the same characters out of newinputbuf
	if (newbuflen > 0 )
	  memmove (newinputbuf, 
		   &(newinputbuf[matches[0].rm_eo]), 
		   newbuflen - matches[0].rm_eo + 1);
	newbuflen = newbuflen - matches[0].rm_eo;
	newinputbuf[newbuflen] = '\000';

	//
	//    Last little bit of cleanup is that IF we fetched from a 
	//    variable (not a file) we have to undo our fakery of stuffing
	//    the var's contents into newinputbuf.
	//   
	//    This cleanup is two parts - restoring the old stdin buffer
	//    contents from savedinputtxt and freeing the temporary
	//    space, and clipping the front off of the source variable.
	if (inputmode == 0)
	  {
	    //      restore the old stdin buffer
	    strncpy (newinputbuf,
		     savedinputtxt,
		     savedinputtxtlen + 1);
	    newbuflen = savedinputtxtlen;
	    //
	    //      free the temporary space
	    free (savedinputtxt);
	    //
	    //      and clip the front off of the source variable.
	    {
	      CSL_CELL* mdw;
	      mdw = NULL;
	      if (tdw->filetext == vht[srcidx]->valtxt)
		mdw = tdw;
	      if (cdw->filetext == vht[srcidx]->valtxt)
		mdw = cdw;
	      crm_slice_and_splice_window (mdw, 
					   vht[srcidx]->vstart,  
					   -matches[0].rm_eo);
	    };
	  };
      }
    crm_window_no_changes_made:
      break;
      
    case CRM_ALTER:
      {
	crm_expr_alter (csl, apb);
      }
      break;
      
    case CRM_EVAL:
      {
	crm_expr_eval (csl, apb);
      }
      break;
      
    case CRM_HASH:
      {
	//      here's where we surgiclly alter a variable to a hash.
	//      We have to watch out in case a variable is not in the
	//      cdw (it might be in tdw; that's legal as well.  syntax
	//      is to replace the contents of the variable in the
	//      varlist with hash of the evaluated string.  

	char varname[MAX_VARNAME];
	long varlen;
	long vns, vnl;
	char newstr [MAX_VARNAME];
	long newstrlen;
	unsigned long hval;         //   hash value

	if (user_trace)
	  fprintf (stderr, "Executing a HASHing\n");

	//     get the variable name
	crm_get_pgm_arg (varname, MAX_VARNAME, apb->p1start, apb->p1len);
	varlen = apb->p1len;
	varlen = crm_nexpandvar (varname, varlen, MAX_VARNAME);
	crm_nextword (varname, varlen, 0, &vns, &vnl);

	//     get the to-be-hashed pattern, and expand it.
	crm_get_pgm_arg (tempbuf, data_window_size, apb->s1start, apb->s1len);
	newstrlen = apb->s1len;
	//
	//                   if no var given, hash the full data window.
	if (newstrlen == 0)
	  {
	    strcpy (tempbuf, ":*:_dw:");
	    newstrlen = strlen (tempbuf);
	  }
	newstrlen = crm_nexpandvar (tempbuf, newstrlen, data_window_size);

	//    The pattern is now expanded, we can hash it to obscure meaning.
	hval = strnhash (tempbuf, newstrlen );
	sprintf (newstr, "%08lX", hval);

	if (internal_trace)
	  {
	    fprintf (stderr, "String: '%s' \n hashed to: %08lX\n", 
		     tempbuf,
		     hval);
	  };

	//    and stuff the new value in.
	crm_destructive_alter_nvariable (&varname[vns], vnl,
					 newstr, strlen (newstr));

      };
      break;


    case CRM_LEARN:
      {
	crm_expr_learn (csl, apb);
      };
      break;
      
      //   we had to split out classify- it was just too big.
    case CRM_CLASSIFY:
      crm_expr_classify ( csl, apb);
      break;

    case CRM_ISOLATE:
      //           Allow creation of a temporary isolated variable;
      //	   this lives in the same big buffer as the environ
      //           args, the arg0 args, and the basic formatting args
      //           (like :_nl:, :_ht:, :_bs:, etc).
      {
	char temp_vars [MAX_VARNAME];
	long tvlen;
	long vn_start_here;
	long vstart;
	long vlen;
	long mc;
	long done;
	long vallen;
	
	if (user_trace)
	  fprintf (stderr, "executing an ISOLATE statement\n");

	//    get the list of variable names
	//
	crm_get_pgm_arg (temp_vars, MAX_VARNAME, apb->p1start, apb->p1len); 
	tvlen = crm_nexpandvar (temp_vars, apb->p1len, MAX_VARNAME);
	if (internal_trace)
	  fprintf (stderr, "  creating isolated vars: ***%s***\n", temp_vars);
	
	done = 0;
	mc = 0;

	//  Now, find the vars (space-delimited, doncha know) and make them
	//  isolated.
	//
	vstart = 0;
	vlen = 0;
	vn_start_here = 0;
	while (!done)
	  {
	    crm_nextword (temp_vars, tvlen, vn_start_here, &vstart, &vlen);
	    vn_start_here = vstart + vlen + 1;
	    if (vlen == 0) 
	      {
		done = 1;
	      }
	    else
	      {
		//        must make a copy of the varname.
		char vname[MAX_VARNAME];
		long vmidx;
		memmove (vname, &(temp_vars[vstart]), vlen);
		vname [vlen] = '\000';
		if (strcmp (vname, ":_dw:") != 0)
		  {
		    vmidx = crm_vht_lookup (vht, vname, vlen);
		    //
		    //        get initial value - that's the slashed value.
		    crm_get_pgm_arg (tempbuf, data_window_size, 
				     apb->s1start, apb->s1len);
		    //
		    //     Now, there's these cases
		    //     not preexisting, no /value/ -  isolate, set to ""
		    //     not preexisting, with /value/ -isolate, set /val/ 
		    //     preexisting, with /value/ - isolate, set to /val/ 
		    //     preexisting _dw, no /value/ - isolate, set to dwval.
		    //     preexisting isolated, no /value/ - noop
		    //     preexisting isolated, with /value/ - alter /value/
		    //
		    //    not preexisting
		    if (vht[vmidx] == NULL)
		      {
			if (!apb->s1start)
			  {
			    // no slash value- set to ""
			    crm_set_temp_var (vname, "");
			  }
			else
			  {
			    //  has a /value/, use it.
			    vallen = crm_nexpandvar(tempbuf, apb->s1len, 
						    data_window_size - tdw->nchars);
			    crm_set_temp_nvar (vname, tempbuf, vallen);
			  };
		      }
		    else
		      //      it IS preexisting- is it isolated?
		      {
			if ( vht[vmidx]->valtxt != tdw->filetext )
			  {
			    //  it's not isolated.  Make it isolated
			    //
			    //   now, do we have a /value/, or do we use the
			    //   preexisting value?
			    if (!apb->s1start)
			      {
				//  nope, no /value/, use old value
				strcpy (tempbuf, ":*");
				strncat (tempbuf, vname, vlen);
				vallen = crm_nexpandvar (tempbuf, vlen+2, 
							 data_window_size - tdw->nchars);
			      }
			    else
			      {
				//  yes, statement has a /value/
				vallen = crm_nexpandvar(tempbuf, apb->s1len, 
							data_window_size - tdw->nchars);
			      };
			    //   now we can clobber the vht slot contents
			    vht[vmidx]->valtxt = tdw->filetext;
			    //        add a separater for good luck.
			    memmove (&(tdw->filetext[tdw->nchars]), "\n", 1);
			    tdw->nchars++;
			    //       
			    //       and put the value in
			    vht[vmidx]->vstart = tdw->nchars;
			    vht[vmidx]->vlen   = vallen;
			    memmove (&tdw->filetext [tdw->nchars], 
				     tempbuf, vallen);
			    tdw->nchars = tdw->nchars + vallen;
			    tdw->filetext[tdw->nchars] = '\000';
			    //   
			    //        and add a separater for good luck.
			    strcat (tdw->filetext, "\n");
			    tdw->nchars++;
			  }
			else
			  {
			    //  the variable is preexisting, and already
			    //  in the tdw.  So, really all we have to do
			    //  is set up the /val/, and do an alter.
			    //  (if there is no /val/, do nothing).
			    if (apb->s1start)
			      {
				//  yes, statement has a /value/
				//    get the /value/
				vallen = crm_nexpandvar(tempbuf, apb->s1len, 
							data_window_size - tdw->nchars);
				//  
				//   and ALTER the value
				//
				crm_destructive_alter_nvariable (vname, 
								 vlen,
								 tempbuf,
								 vallen);
			      };
			  };
		      };
		  }
		else
		  {
		    nonfatalerror ("You can't ISOLATE the :_dw: data window!  ",
				   "We'll just ignore that for now");
		  };
	      };
	    vstart = vstart + vlen;
	    if (temp_vars[vstart] == '\000'
		|| vstart >= tvlen )
	      done = 1;
	  };
      };
      break;
      
    case CRM_INPUT:
      //           Allow input of text from stdin.  
      {
	FILE *fp;
	char temp_vars [MAX_PATTERN];
	long tvlen;
	char filename [MAX_FILE_NAME_LEN];
	long fnlen;
	long vstart;
	long vlen;
	long mc;
	long done;
	long till_eof;

	if (user_trace)
	  fprintf (stderr, "executing an INPUT statement");

	//    set up the flags (if any)
	//
	till_eof = 1;
	if (apb->sflags & CRM_BYLINE)
	  {
	    till_eof = 0;
	    if (user_trace)
	      fprintf (stderr, " reading one line mode ");
	  };

	//    get the list of variable names
	//
	crm_get_pgm_arg (temp_vars, MAX_PATTERN, apb->p1start, apb->p1len);
	tvlen = crm_nexpandvar (temp_vars, apb->p1len,  MAX_PATTERN);
	if (internal_trace)
	  fprintf (stderr, "  inputting to var: ***%s***\n", temp_vars);
	
	//   and what file to get it from...
	//
	crm_get_pgm_arg (filename, MAX_FILE_NAME_LEN, apb->b1start,apb->b1len);
	fnlen = crm_nexpandvar (filename, apb-> b1len, MAX_FILE_NAME_LEN);
	if (internal_trace)
	  fprintf (stderr, "  from filename ***%s***\n", filename);
	fp = stdin;
	if ( fnlen  > 0)
	  {
	    fp = fopen (filename, "r");
	    if (fp == NULL)
	      {
		fatalerror ("I tried, but I couldn't open file ",
			  filename);
		goto invoke_bailout;
	      };
	  };

	done = 0;
	mc = 0;

	//   get the file name...
	crm_nextword( temp_vars, tvlen, 0,  &vstart, &vlen);

	if (vlen == 0) 
	  {
	    done = 1;
	  }
	else
	  {
	    //        must make a copy of the varname.
	    char vname[MAX_VARNAME];
	    long ichar;
	    memmove (vname, &(temp_vars[vstart]), vlen);
	    vname [vlen] = '\000';
	    
	    //        and grab a line from fp
	    //    grab characters in a loop, terminated by EOF or newline
	    ichar = 0;
	    if (feof (fp) ) clearerr (fp);
	    while (!feof (fp) 
		   && ichar < (data_window_size >> SYSCALL_WINDOW_RATIO)
		   && (inbuf [ichar-1] != '\n' || till_eof) )	    
	      {
		inbuf[ichar] = fgetc (fp);
		ichar++;
	      };
	    ichar-- ; //   get rid of the newline
	    inbuf[ichar] = '\000';   // and put a null on the end of it.
	    crm_set_temp_nvar (vname, inbuf, ichar);
	  };
	//     and close the input file if it's not stdin.
	if (fp != stdin) fclose (fp);
      };
      break;

    case CRM_SYSCALL:
      {
	//           Go off and fork a process, sending that process
	//           one pattern evaluated as input, and then accepting
	//           all the returns from that process as the new value
	//           for a variable.
	//           
	//           syntax is:
	//               exec (:to:) (:from:) (:ctl:) /commandline/
	long inlen;
	long outlen;
	char from_var [MAX_VARNAME];
	char sys_cmd [MAX_PATTERN];
	long cmd_len;
	char keep_buf [MAX_PATTERN];
	long keep_len;
	char exp_keep_buf[MAX_PATTERN];
	long exp_keep_len;
	long vstart;
	long vlen;
	long done, charsread;
	int keep_proc;
	int async_mode;
	int to_minion[2];
	int from_minion[2];
	pid_t minion;
	pid_t pusher;
	int status;
	int minion_exit_status;
	long timeout;	
	
	if (user_trace)
	  fprintf (stderr, "executing an SYSCALL statement");

	//  clean up any prior processes
	waitpid ( 0, &minion_exit_status, WNOHANG) ;
	waitpid ( 0, &minion_exit_status, WNOHANG) ;
	waitpid ( 0, &minion_exit_status, WNOHANG) ;
	waitpid ( 0, &minion_exit_status, WNOHANG) ;


	//    get the flags
	//
	keep_proc = 0;
	if (apb->sflags & CRM_KEEP)
	  {
	    if (user_trace)
	      fprintf (stderr, "Keeping the process around if possible\n");
	    keep_proc = 1;
	  };
	async_mode = 0;
	if (apb->sflags & CRM_ASYNC)
	  {
	    if (user_trace)
	      fprintf (stderr, "Doing asynchronous reads from the process\n");
	    async_mode = 1;
	  };
	
	//    get the input variable(s)
	//
	crm_get_pgm_arg (inbuf, data_window_size, apb->p1start, apb->p1len);
	inlen = crm_nexpandvar (inbuf, apb->p1len, data_window_size);
	if (user_trace)
	  fprintf (stderr, "  command's input wil be: ***%s***\n", inbuf);

	//    now get the name of the variable where the return will be 
	//    placed... this is a crock and should be fixed someday.
	//    the output goes only into a single var (the first one)
	//    so we extract that
	//
	crm_get_pgm_arg (from_var, MAX_PATTERN, apb->p2start, apb->p2len);
	outlen = crm_nexpandvar (from_var, apb->p2len, MAX_PATTERN);
	done = 0;
	vstart = 0;
	while (from_var[vstart] < 0x021 && from_var[vstart] > 0x0 )
	  vstart++;
	vlen = 0;
	while (from_var[vstart+vlen] >= 0x021)
	  vlen++;
	memmove (from_var, &from_var[vstart], vlen);
	from_var[vlen] = '\000';
	if (user_trace)
	  fprintf (stderr, "   command output will overwrite var ***%s***\n",
		   from_var);


	//    now get the name of the variable (if it exists) where
	//    the kept-around minion process's pipes and pid are stored.
	crm_get_pgm_arg (keep_buf, MAX_PATTERN, apb->p3start, apb->p3len);
	keep_len = crm_nexpandvar (keep_buf, apb->p3len, MAX_PATTERN);
	if (user_trace)
	  fprintf (stderr, "   command status kept in var ***%s***\n", 
		   keep_buf);

	//      get the command to execute
	//
	crm_get_pgm_arg (sys_cmd, MAX_PATTERN, apb->s1start, apb->s1len);
	cmd_len = crm_nexpandvar (sys_cmd, apb->s1len, MAX_PATTERN);
	if (user_trace)
	  fprintf (stderr, "   command will be ***%s***\n", sys_cmd);

	//     Do we reuse an already-existing process?  Check to see if the 
	//     keeper variable has it... note that we have to :* prefix it
	//     and expand it again.
	minion = 0;
	to_minion[0] = 0;
	from_minion[1] = 0;
	exp_keep_buf [0] = '\000';
	//  this is 8-bit-safe because vars are never wchars. 
	strcat (exp_keep_buf, ":*");
	strncat (exp_keep_buf, keep_buf, keep_len);
	exp_keep_len = crm_nexpandvar (exp_keep_buf, keep_len+2, MAX_PATTERN);
	sscanf (exp_keep_buf, "MINION PROC PID: %d from-pipe: %d to-pipe: %d",
		&minion,
		&from_minion[0],
		&to_minion[1]);


	//      if, no minion already existing, we create
	//      communications pipes and launch the subprocess.  This
	//      code borrows concepts from both liblaunch and from
	//      netcat (thanks, *Hobbit*!)
	//
	if (minion == 0)
	  {
	    if (user_trace)
	      fprintf (stderr, "  Must start a new minion.\n");
	    status = pipe (to_minion);
	    status = pipe (from_minion);
	    minion = fork();
	    if (minion == 0)
	      {
		//    We're in the minion here; close the ends of the
		//    pipes we don't use.  NOTE: if this gets messed
		//    up, you end up with a race condition, because
		//    both master and minion processes can both read
		//    and write both pipes (effectively a process
		//    could write something out, then read it again
		//    right back out of the pipe)!  So, it's // REALLY
		//    REALLY IMPORTANT that you use two pipe
		//    structures, (one for each direction) and you
		//    keep track of which process should write to
		//    which pipe!!!
		int retcode;
		close (to_minion[1]);
		close (from_minion[0]);
		dup2 (to_minion[0], fileno(stdin));
		dup2 (from_minion[1], fileno(stdout));
		if (user_trace)
		  fprintf (stderr, "systemcalling on %s\n", sys_cmd);
		retcode = system (sys_cmd);
		if (retcode == -1)
		  exit ( EXIT_FAILURE );
		exit ( retcode );
	      };
	  }
	else
	  {
	    if (user_trace)
	      fprintf (stderr, "  reusing old minion PID: %d\n", minion);
	  };
	//      Now, we're out of the minion for sure.
	//    so we close the pipe ends we know we won't be using.
	if (to_minion[0] != 0) 
	  {
	    close (to_minion[0]);
	    close (from_minion[1]);
	  };
	//  
	//   launch "pusher" process to send the buffer to the minion
	//    (this hint from Dave Soderberg).  This avoids the deadly
	//   embrace situation where both processes are waiting to read
	//   (or, equally, both processes have written and filled up
	//   their buffers, and are now held up waiting for the other
	//   process to empty some space in the output buffer)
	//
	pusher = fork ();
	if (pusher == 0)
	  {
	    //    we're in the "input pusher" process if we got here.
	    //    shove the input buffer out to the minion
	    if (strlen (inbuf) > 0)
	      {
		write (to_minion[1], inbuf, inlen );
		if (internal_trace) 
		  fprintf (stderr, "pusher: input sent to minion.\n");
		close (to_minion[1]);
		if (internal_trace)
		  fprintf (stderr, "pusher: minion input pipe closed\n");
		if (internal_trace)
		  fprintf (stderr, "pusher: exiting pusher\n");
	    };
	    exit ( EXIT_SUCCESS );
	  };
	//    now we're out of the pusher process.
	//    if we don't want to keep this proc, we close it's input, and
	//    wait for it to exit.
	if (! keep_proc)
	  {
	    close (to_minion[1]);
	    if (internal_trace)
	      fprintf (stderr, "minion input pipe closed\n");
	  }
	timeout = MINION_SLEEP_USEC;
	usleep (timeout);

	//   and see what is in the pipe for us.
	outbuf[0] = '\000';
	done = 0;
	//   grot grot grot this only works if varnames are not widechars
	if (strlen (from_var) > 0)
	  {
	    if (async_mode == 0)
	      {
		//   synchronous read- read till we hit EOF, which is read 
		//   returning a char count of zero.
	      readloop:
		usleep (timeout);
		charsread = 
		  read (from_minion[0], 
			&outbuf[done], 
			(data_window_size >> SYSCALL_WINDOW_RATIO) - done - 2);
		done = done + charsread; 
		if (charsread > 0 &&
		    done + 2 < (data_window_size >> SYSCALL_WINDOW_RATIO)) 
		  goto readloop;
		outbuf [done] = '\000';
		outlen = done ;
	      }
	    else
	      {
		//   we're in nonblocking mode.  Set nonblocking mode, then
		//   read it once; then put it back in regular mode.
		fcntl (from_minion[0], F_SETFL, O_NONBLOCK);
		usleep (timeout);
		charsread = read (from_minion[0], 
				  &outbuf[done], 
				  (data_window_size >> SYSCALL_WINDOW_RATIO));
		done = done + charsread; 
		outbuf [done] = '\000';
		outlen = done ;
		fcntl (from_minion[0], F_SETFL, 0);
	      }
	    //  and set the returned value into from_var.
	    if (user_trace)
	      fprintf (stderr, "SYSCALL output: %ld chars ---%s---.\n ", 
		       done, outbuf);
	    if (internal_trace)
	      fprintf (stderr, "  storing return str in var %s\n", from_var);

	    crm_destructive_alter_nvariable ( from_var, vlen, outbuf, outlen);
	  };

	//  Record useful minion data, if possible.
	if (strlen (keep_buf) > 0)
	  {
	    sprintf (exp_keep_buf, 
		     "MINION PROC PID: %d from-pipe: %d to-pipe: %d",
		     minion,
		     from_minion[0],
		     to_minion[1]);
	    if (internal_trace)
	      fprintf (stderr, "   saving minion state: %s \n", 
		       exp_keep_buf);
	    crm_destructive_alter_nvariable (keep_buf, keep_len,
					     exp_keep_buf, 
					     strlen (exp_keep_buf));
	  };
	//      If we're keeping this minion process around, record the useful
	//      information, like pid, in and out pipes, etc.
	if (keep_proc)
	  {
	  }
	else
	  {
	    if (internal_trace)
	      fprintf (stderr, "Not keeping minion, closing everything.\n");
	    //   no, we're not keeping it around, so close the pipe.
	    close (from_minion [0]);
	    //    and de-zombify any processes.
	    waitpid ( minion, &minion_exit_status, 0) ;
	    if ( crm_vht_lookup (vht, keep_buf, strlen (keep_buf)))
	      {
		char exit_value_string[MAX_VARNAME];
		if (internal_trace) 
		  fprintf (stderr, "and whacking %s\n", keep_buf);
		sprintf (exit_value_string, "DEAD MINION, EXIT CODE: %d", 
			 minion_exit_status); 
		crm_destructive_alter_nvariable (keep_buf, keep_len,
						exit_value_string,
						strlen (exit_value_string));
	      };
	  };
      };
      break;

    case CRM_INTERSECT:
      //           Calculate the intersection of N variables; the result
      //           replaces the captured value of the first variable.
      //           Captured values not in the data window are ignored.
      {
	char temp_vars [MAX_VARNAME];
	long tvlen;
	char out_var [MAX_VARNAME];
	long ovstart;
	long ovlen;
	long vstart;
	long vend;
	long vlen;
	long istart, iend, ilen, i_index;
	long mc;
	long done;
	
	if (user_trace)
	  fprintf (stderr, "executing an INTERSECT statement");

	//    get the output variable (the one we're gonna whack)
	//
	crm_get_pgm_arg (out_var, MAX_VARNAME, apb->p1start, apb->p1len);
	ovstart = 0;
	ovlen = crm_nexpandvar (out_var, apb->p1len, MAX_VARNAME);
	

	//    get the list of variable names
	//
	//     note- since vars never contain wchars, we're OK here.
	crm_get_pgm_arg (temp_vars, MAX_VARNAME, apb->b1start, apb->b1len);
	tvlen = crm_nexpandvar (temp_vars, apb->b1len, MAX_VARNAME);
	if (internal_trace)
	  {
	    fprintf (stderr, "  Intersecting vars: ***%s***\n", temp_vars);
	    fprintf (stderr, "   with result in ***%s***\n", out_var);
	  };
	done = 0;
	mc = 0;
	vstart = 0;
	vend = 0;
	istart = 0;
	iend = cdw->nchars;
	ilen = 0;
	i_index = -1;
	while (!done)
	  {
	    while (temp_vars[vstart] < 0x021
		   && vstart < tvlen )    //  was temp_vars[vstart] != '\000')
	      vstart++;
	    vlen = 0;
	    while (temp_vars[vstart+vlen] >= 0x021
		   && vstart+vlen < tvlen )
	      vlen++;
	    if (vlen == 0) 
	      {
		done = 1;
	      }
	    else
	      {
		long vht_index;
		//
		//        look up the variable
		vht_index = crm_vht_lookup (vht, &temp_vars[vstart], vlen);
		if (vht[vht_index] == NULL )
		  {
		    char varname[MAX_VARNAME];
		    strncpy (varname, &temp_vars[vstart], vlen);
		    varname[vlen] = '\000';
		    nonfatalerror ( "can't intersection a nonexistent variabe.",
				    varname);
		    goto invoke_bailout;
		  }
		else
		  { 
		    //  it was a good var, make sure it's in the data window
		    //
		    if (vht[vht_index] -> valtxt != cdw->filetext)
		      {
			char varname[MAX_VARNAME];
			strncpy (varname, &temp_vars[vstart], vlen);
			varname[vlen] = '\000';
			nonfatalerror ( "can't intersect isolated variabe.",
					varname);
			goto invoke_bailout;

		      }
		    else
		      {      //  it's a cdw variable; go for it.
			if (vht[vht_index] -> vstart > istart)
			  istart = vht[vht_index] -> vstart;
			if ((vht[vht_index]->vstart + vht[vht_index]->vlen) 
			    < iend)
			  iend = vht[vht_index]->vstart + vht[vht_index]->vlen;
		      };
		  };
	      };
	    vstart = vstart + vlen;
	    if (temp_vars[vstart] == '\000')
	      done = 1;
	  };
	//
	//      all done with the looping, set the start and length of the 
	//      first var.
	vlen = iend - istart;
	if (vlen < 0 ) vlen = 0;
	crm_nextword (out_var, ovlen, 0, &ovstart, &ovlen);
	crm_set_windowed_nvar (&out_var[ovstart], ovlen, cdw->filetext, 
			       istart, vlen,
			       csl->cstmt);
      }
      break;
    case CRM_UNION:
      //           Calculate the union of N variables; the result
      //           replaces the captured value of the first variable.
      //           Captured values not in the data window are ignored.
      {
	char temp_vars [MAX_VARNAME];
	long tvlen;
	char out_var[MAX_VARNAME];
	long ovstart;
	long ovlen;
	long vstart;
	long vend;
	long vlen;
	long istart, iend, ilen, i_index;
	long mc;
	long done;
	
	if (user_trace)
	  fprintf (stderr, "executing a UNION statement");

	//    get the output variable (the one we're gonna whack)
	//
	crm_get_pgm_arg (out_var, MAX_VARNAME, apb->p1start, apb->p1len);
	ovstart = 0;
	ovlen = crm_nexpandvar (out_var, apb->p1len, MAX_VARNAME);
	

	//    get the list of variable names
	//
	//    since vars never contain wchars, we don't have to be 8-bit-safe
	crm_get_pgm_arg (temp_vars, MAX_VARNAME, apb->b1start, apb->b1len);
	tvlen = crm_nexpandvar (temp_vars, apb->b1len, MAX_VARNAME);
	if (internal_trace)
	  fprintf (stderr, "  Uniting vars: ***%s***\n", temp_vars);
	
	done = 0;
	mc = 0;
	vstart = 0;
	vend = 0;
	istart = cdw->nchars;
	iend = 0;
	ilen = 0;
	i_index = -1;
	while (!done)
	  {
	    while (temp_vars[vstart] < 0x021
		   && vstart < tvlen)    //  was temp_vars[vstart] != '\000')
	      vstart++;
	    vlen = 0;
	    while (temp_vars[vstart+vlen] >= 0x021
		   && vstart+vlen < tvlen)
	      vlen++;
	    if (vlen == 0) 
	      {
		done = 1;
	      }
	    else
	      {
		long vht_index;
		//
		//        look up the variable
		vht_index = crm_vht_lookup (vht, &temp_vars[vstart], vlen);
		if (vht[vht_index] == NULL )
		  {
		    char varname[MAX_VARNAME];
		    strncpy (varname, &temp_vars[vstart], vlen);
		    varname[vlen] = '\000';
		    nonfatalerror ( "can't intersect a nonexistent variable.",
				    varname);
		    goto invoke_bailout;

		  }
		else
		  { 
		    //  it was a good var, make sure it's in the data window
		    //
		    if (vht[vht_index] -> valtxt != cdw->filetext)
		      {
			char varname[MAX_VARNAME];
			strncpy (varname, &temp_vars[vstart], vlen);
			varname[vlen] = '\000';
			nonfatalerror ( "can't intersect isolated variable.",
					varname);
			goto invoke_bailout;
		      }
		    else
		      {      //  it's a cdw variable; go for it.
			if (vht[vht_index] -> vstart < istart)
			  istart = vht[vht_index] -> vstart;
			if ((vht[vht_index]->vstart + vht[vht_index]->vlen) 
			    > iend)
			  iend = vht[vht_index]->vstart + vht[vht_index]->vlen;
		      };
		  };
	      };
	    vstart = vstart + vlen;
	    if (temp_vars[vstart] == '\000')
	      done = 1;
	  };
	//
	//      all done with the looping, set the start and length of the 
	//      output var.
	vlen = iend - istart;
	if (vlen < 0 ) vlen = 0;
	crm_nextword (out_var, ovlen, 0, &ovstart, &ovlen);
	crm_set_windowed_nvar (&out_var[ovstart], ovlen, cdw->filetext, 
			       istart, vlen,
			       csl->cstmt);
	
      }
      break;

    case CRM_UNIMPLEMENTED:
      {
        char bogusbuffer[1024];
	char bogusstmt [1024];
        sprintf (bogusbuffer, "Statement %ld NOT YET IMPLEMENTED !!!  Here's the text: \n",
		 csl->cstmt);
	memmove(bogusstmt, 
		&csl->filetext[csl->mct[csl->cstmt]->start],
		csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start);
	bogusstmt [csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start]
	  = '\000';
        fatalerror (bogusbuffer, bogusstmt);
	goto invoke_bailout;

      };
      break;

    default:
      {
        char bogusbuffer[1024];
	char bogusstmt [1024];
        sprintf (bogusbuffer, "Statement %ld way, way bizarre !!!  Here's the text: \n",
		 csl->cstmt);
	memmove (bogusstmt, 
		&csl->filetext[csl->mct[csl->cstmt]->start],
		csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start);
	bogusstmt [csl->mct[csl->cstmt+1]->start - csl->mct[csl->cstmt]->start]
	  = '\000';
        fatalerror (bogusbuffer, bogusstmt);
	goto invoke_bailout;
      };
    }

  //    If we're in some sort of strange abort mode, and we just need to move
  //    on to the next statement, we branch here.
 invoke_bailout:

  //  grab end-of-statement timers ?
  if (profile_execution)
    {
      times ( (void *) &timer2);
      csl->mct[tstmt]->stmt_utime += (timer2.tms_utime - timer1.tms_utime);
      csl->mct[tstmt]->stmt_stime += (timer2.tms_stime - timer1.tms_stime);
    };


  //    go on to next statement (unless we're failing, laifing, etc,
  //    in which case we have no business getting to here.
  csl->cstmt ++;

  goto invoke_top;
  
 invoke_done:

  //     give the debugger one last chance to do things.
  if (debug_countdown == 0)
    i=crm_debugger ();
  if (csl->cstmt < csl->nstmts)
    goto invoke_top;

  //     if we asked for an output profile, give it to us.
  if (profile_execution)
    crm_output_profile (csl);

  return (status);
}

//     Helper routines 
//    

//     apocalyptic eerror - an error that can't be serviced on a TRAP - forces
//     exit, not a prayer of survival.
void untrappableerror (char *text1, char *text2)
{
  int iline, ichar;
  char reason[MAX_PATTERN];
  //
  //   Create our reason string.
  sprintf (reason, "\n%s: *UNTRAPPABLE ERROR* \n %s %s\nSorry, but I can't recover from that.\nThis happened at line %ld of file %s\n", 
	   prog_argv[0], text1, text2, csl->cstmt, csl->filename);

  //     Check to see - is there a trap available or is this a non-trap 
  //     program?
  //
  fprintf (stderr, "%s", reason);
      fprintf (stderr, "The line was: \n--> ");
  iline = csl->cstmt;
  if (iline > 0)
    for (ichar = csl->mct[iline]->fchar;
	 ichar < csl->mct[iline+1]->fchar;
	 ichar++)
      fprintf (stderr, "%c", csl->filetext[ichar]);
  fprintf (stderr, "\n");
  exit ( EXIT_FAILURE );
}


//     fatalerror - print a fatal error on stdout, trap if we can, else exit  
long fatalerror ( char *text1, char *text2 )
{
  int iline, ichar;
  char *rbuf;
  char reason[MAX_PATTERN];
  //
  //   Create our reason string.
  sprintf (reason, "\n%s: *ERROR* \n %s %s\n  Sorry, but this program is very sick and probably should be killed off.\nThis happened at line %ld of file %s\n", 
	   prog_argv[0], text1, text2, csl->cstmt, csl->filename);

  //     Check to see - is there a trap available or is this a non-trap 
  //     program?
  //
  if (csl->mct[csl->cstmt]->trap_index <  csl->nstmts)
    {
      long fresult;
      rbuf = malloc (MAX_PATTERN);
      if (!rbuf)
	{
          fprintf(stderr,
	    "Couldn't malloc rbuf in 'fatalerror()'!  \nIt's really bad when the error fixup routine gets an error!.\n");
          exit(EXIT_FAILURE);
	};
      strcpy (rbuf, reason);
      fresult = crm_trigger_fault (rbuf);
      if (fresult == 0)
	return (0);
    };
  fprintf (stderr, "%s", reason);
  fprintf (stderr, "The line was: \n--> ");
  iline = csl->cstmt;
  for (ichar = csl->mct[iline]->fchar;
       ichar < csl->mct[iline+1]->fchar;
       ichar++)
    fprintf (stderr, "%c", csl->filetext[ichar]);
  fprintf (stderr, "\n");
  exit ( EXIT_FAILURE );
}

long nonfatalerror ( char *text1, char *text2 )
{
  int iline, ichar;
  static int nonfatalerrorcount = 0;
  char *rbuf;
  char reason[MAX_PATTERN];

  //
  //    generate our reason string
  sprintf (reason, "\n%s: *WARNING* \n %s %s\nI'll try to keep working.\nThis happened at line %ld of file %s\n", 
	   prog_argv[0], text1, text2, csl->cstmt, csl->filename);
  //     Check to see - is there a trap available or is this a non-trap 
  //     program?
  //

  if (csl->mct[csl->cstmt]->trap_index < csl->nstmts)
    {
      long fresult;
      rbuf = malloc (MAX_PATTERN);
      if (!rbuf)
	{
          fprintf(stderr,
	    "Couldn't malloc rbuf in 'nonfatalerror()'! \nIt's really bad when the error fixup routine gets an error ! \n");
          exit(EXIT_FAILURE);
	};

      strcpy (rbuf, reason);
      fresult = crm_trigger_fault (rbuf);
      if (fresult == 0) return (0);
    };

  fprintf (stderr, "%s", reason);
  fprintf (stderr, "The line was: \n--> "); 
  iline = csl->cstmt;
  for (ichar = csl->mct[iline]->fchar;
       ichar < csl->mct[iline+1]->fchar;
       ichar++)
    fprintf (stderr, "%c", csl->filetext[ichar]);
  nonfatalerrorcount++;
  if (nonfatalerrorcount > MAX_NONFATAL_ERRORS) 
    fatalerror 
      ("Too many untrapped warnings; your program is very likely unrecoverably broken.\n",
       "\n\n  'Better shut her down, Scotty.  She's sucking mud again.'\n");
    return (0);
}

//     print out timings of each statement
void crm_output_profile ( CSL_CELL *csl)
{
  long i;
  fprintf (stderr, "\n  Statement Execution Time Profiling (0 times suppressed)");
  fprintf (stderr, "\n  line:      usertime   systemtime    totaltime\n");
  for (i = 0; i < csl->nstmts; i++)
    {
      if (csl->mct[i]->stmt_utime +csl->mct[i]->stmt_stime > 0)
	fprintf (stderr, " %5ld:   %10ld   %10ld   %10ld\n",
		 i,
		 csl->mct[i]->stmt_utime,
		 csl->mct[i]->stmt_stime,
		 csl->mct[i]->stmt_utime + csl->mct[i]->stmt_stime);
    };
}



//      crm preprocessor - pre-process a CRM file to make it 
//      palatable to the sorry excuse we have for a compiler.
int crm_preprocessor (CSL_CELL *csl, int flags)
{
  int lflag;
  int status;
  long i, j;
  long done;
  regex_t preg;
  regmatch_t matches[3];

  //     step 1 - go thru the program and replace all #inline files
  //     with the actual text of the insert file
  if (internal_trace )
    fprintf (stderr, "pass 1 of preprocessor - #insert processing...\n");

  lflag = 0;
  i = 0;
  done = 0;
  i = crm_regcomp (&preg, 
		   "[\n;][[:blank:]]*insert[[:blank:]]+([[:graph:]]+)", 
	   strlen ("[\n;][[:blank:]]*insert[[:blank:]]+([[:graph:]]+)"), 
		   REG_EXTENDED || REG_ICASE || REG_NEWLINE);
  if ( i != 0)
    {
      crm_regerror ( i, &preg, tempbuf, data_window_size);
      untrappableerror (
	 "Regular Expression Compilation Problem during INSERT processing:", 
	 tempbuf);
    };
  
  while (!done)
    {
      j  = crm_regexec ( &preg, csl->filetext, csl->nchars,
			 		3, matches, lflag, NULL);
      if ( j != 0) 
	{
	  if (internal_trace)
	    fprintf (stderr, "No insert files left to do.\n");
	  done = 1;
	}
      else 
	{
	  char insertfilename [259];
	  struct stat statbuf;
	  for (j = 0; j < matches[1].rm_eo - matches[1].rm_so; j++)
	    insertfilename[j] = csl->filetext[matches[1].rm_so + j];
	  insertfilename[j] = '\000';
	  
	  //   stat the file
	  status = stat ( insertfilename, &statbuf );
	  if (! status )
	    {
	      if (user_trace)
		{
		  fprintf (stderr, "Inserting file '%s' .\n", insertfilename);
		};
	      //     we need to make a hole of statbuf.st_size bytes.
	      memmove (&(csl->filetext[matches[1].rm_eo + statbuf.st_size + 2]),
		       &(csl->filetext[matches[1].rm_eo + 1]),
		       csl->nchars - matches[1].rm_eo + 1);

	      //   To keep the matcher from looping, we change the string
	      //   'insert' to 'insert=' .  Cool, eh?  
	      csl->filetext[matches[0].rm_so+7] = '=';   // smash the match 
	      
	      //      and copy the file into that hole
	      {
		int fd;
		fd = open (insertfilename, O_RDONLY);
		read (fd, &(csl->filetext[matches[1].rm_eo+1]), 
		      statbuf.st_size);
		close (fd);
		//    put in a newline at the end of the insert...
		csl->filetext[matches[1].rm_eo+1+statbuf.st_size] = '\n';
		if (internal_trace)
		  fprintf (stderr, "Old length: %ld, ", csl->nchars);
		csl->nchars += statbuf.st_size;
		if (internal_trace)
		  fprintf (stderr, "new length: %ld\n ", csl->nchars);

	      };
	    }
	  else
	    {
	      nonfatalerror (" I'm having a problem inserting file ",
			     insertfilename);
	    };
	  
	  i = matches[1].rm_so + 1;
	};
    };
  crm_regfree (&preg);


  if (internal_trace )
    fprintf (stderr, "pass 2 of preprocessor - statement breaks...\n");
  //     step 2 - go thru the program, and keep track of slash nesting
  //     If not in a slashnest, then 
  //     a) replace all semicolons with '\n'
  //
  {
    int seennewline;
    int in_comment ;
    int neednewline;
    seennewline = 1;
    neednewline = 0;
    in_comment = 0;
    for (i = 0; i < csl->nchars; i++)
      {
	//    now, no matter what, we're looking at a non-quoted character.
	//
	//   are we looking at a nonprinting character?
	if (csl->filetext[i] < 0x021 )
	  {
	    if (csl->filetext[i] == '\n')
	      {
		//    get rid of extraneous newlines.
		//if (internal_trace)
		//  fprintf (stderr, "  newline .");
		seennewline = 1;
		neednewline = 0;
		in_comment = 0;
	      };
	    //   other nonprinting characters do not change things.
	  }
	else
	  {
	    //   we don't do any processing inside a comment!
	    if ( in_comment )
	      {
		//   inside a comment, we don't do squat to printing chars.
	      }
	    else
	      {
		//    we are looking at a printing character, so maybe we have
		//    to add a newline.  Or maybe not...
		if (neednewline) 
		  {
		    //    we need a newline and are looking at a printingchar
		    //    so we need to insert a newline.  
		    memmove ( &(csl->filetext[i+1]), 
			      &(csl->filetext[i]), 
			      strlen (&csl->filetext[i]));
		    csl->filetext[i] = '\n';
		    i++;
		    csl->nchars++;
		    neednewline = 0;
		    seennewline = 1;
		  };
		//
		//     we don't need a newline any more, and quoted characters 
		//      have already been stepped over.  At most we need to set
		//      markers for needing newlines or inserting them.  Do 
		//      this on a per-printable-char basis.
		switch (csl->filetext[i])
		  {
		  case '\\':
		    {
		      //     if it's a backslash at the end of a line, 
		      //     delete +both+ the backslash and the newline, making
		      //     one big line out of it.
		      if (  csl->filetext[i+1] == '\n' )
			{
			  if (internal_trace)
			    fprintf (stderr, " backquoted EOL - splicing.");
			  memmove ( &(csl->filetext[i]), 
				&(csl->filetext[i+2]), 
				    strlen (&csl->filetext[i]));
			  csl->nchars--;
			  csl->nchars--;
			}
		      //  otherwise, if the next character is {, } or ;, 
		      //  the backslash itself gets eaten and 
		      //  the next character passes totally unscathed.  
		      else
			{
			  if ( csl->filetext[i+1] == '{' ||
			       csl->filetext[i+1] == '}' ||
			       csl->filetext[i+1] == ';' ||
			       csl->filetext[i+1] == '#' 
			       )
			    {
			      memmove ( &(csl->filetext[i]), 
					&(csl->filetext[i+1]), 
					strlen (&csl->filetext[i]));
			      csl->nchars--;
			    };
			};
		    };
		    break;
		  case '{':
		  case '}':
		    {
		      //    put an unquoted '{' or '}' onto it's own line.
		      //   do we need to put in a prefix new line?
		      //		  if (internal_trace)
		      //  fprintf (stderr, " Real { / }. ");
		      if (!seennewline )
			{
			  if (internal_trace)
			    fprintf (stderr, " preinserting a newline. ");
			  memmove ( &(csl->filetext[i+1]), 
				    &(csl->filetext[i]), 
				    strlen (&csl->filetext[i]));
			  csl->filetext[i] = '\n';
			  csl->nchars++;
			};
		      seennewline = 0;
		      //   and mark that we need a newline before any more
		      //   printable characters come through.
		      neednewline = 1;
		    };
		    break;
		  case ';':     
		    {
		      //       we can replace non-escaped semicolons with 
		      //       newlines.
		      if ( seennewline ) //  we just saw a newline
			{  
			  //  was preceded by a newline so just get rid 
			  //  of the ;
			  if (internal_trace)
			    fprintf (stderr, "superfluous semicolon, *poof*.");
			  memmove ( &(csl->filetext[i]), 
				    &(csl->filetext[i+1]), 
				    strlen (&csl->filetext[i]));
			  csl->nchars--;
			  i--;
			  neednewline = 0;
			  seennewline = 1;
			}
		      else
			{
			  //   this was not preceded by a newline,
			  //  so get rid of the semi and mark that we need a 
			  //  newline before any printed characters
			  if (internal_trace)
			    fprintf (stderr, " statement break semi. ");
			  memmove ( &(csl->filetext[i]), 
				    &(csl->filetext[i+1]), 
				    strlen (&csl->filetext[i]));
			  csl->nchars--;
			  i--;
			  neednewline = 1;
			  seennewline = 0;
			};
		    };
		    break;
		  case '#':
		    {  
		      //      now, we're in a comment - everything should be
		      //      done only with the comment thing enabled.
		      in_comment = 1;
		    };
		    break;

		  default: 
		    {
		      //      none of the above - it's a normal printing 
		      //  character - we can just do the 
		      //  clearing of all the "seen/need" flags 
		      seennewline = 0;
		      neednewline = 0;
		    };
		    break;
		  };
	      };
	  };
      };
  };
  return (0);
}
  

//      initialize the variable hash table (the vht)
void crm_vht_init (int argc, char **argv)
{
  long i, j, k;
  long uvstart = 0;
  long uvlist = 0;
  char uvset[MAX_VARNAME];
  extern char **environ;
  char posvars[MAX_VARNAME];


 //   create the variable hash table (one big one, shared )
  vht = (VHT_CELL **) malloc (sizeof (VHT_CELL *) * vht_size);
  if (!vht)
    untrappableerror("Couldn't malloc VHT cell.\n","No VHT cells, no variables, so no can run.  Sorry.");
  for (i = 0; i < vht_size; i++)
    vht[i] = NULL;


  //    initialize the temporary (non-data-window) area...
  tdw = malloc (sizeof (CSL_CELL));
  if (!tdw)
    untrappableerror("Couldn't malloc tdw.  We need the TDW for isolated variables.  Can't continue.  Sorry.\n","");
  tdw->filename = NULL;
  tdw->rdwr = 1;
  tdw->filedes = -1;
  tdw->filetext = malloc (sizeof (char) * data_window_size);
  if (!tdw->filetext)
      untrappableerror("Couldn't malloc tdw->filetext.\nWithout this space, you can't have any isolated variables,\n and we're stuck.  Sorry.","");
  tdw->filetext[0] = '\000';
  tdw->nchars = 0;
  tdw->hash = 0;
  tdw->mct = NULL;
  tdw->nstmts = -1;
  tdw->cstmt = -1;
  tdw->caller = NULL;

  //    install a few constants.

  crm_set_temp_var (":_nl:", "\n");
  crm_set_temp_var (":_ht:", "\t");
  crm_set_temp_var (":_bs:", "\b");
  crm_set_temp_var (":_sl:", "/");
  crm_set_temp_var (":_sc:", ";");
  crm_set_temp_var (":_crm_version:", VERSION);

  //    install the argc and argv values; restart argv values from [2]
  //    if a "--" metaflag is seen.
  {
    long i, j;
    char anamebuf [255];
    i = 0; j = 0; 
    for ( i = 0; argc > i; i++ )
      {
	//   check for the "--" metaflag
	if (strlen ( argv[i] ) == 2 && strncmp (argv[i], "--", 2) == 0)
	  {
	    if (internal_trace) 
	      fprintf (stderr, "Resetting _arg counter to 2\n");
	    j = 2;
	    if (uvstart == 0) uvstart = i;
	  }
	else
	  {
	    sprintf (anamebuf, ":_arg%ld:", j);
	    crm_set_temp_var ( anamebuf, argv[i] );
	    j++;
	  };
      };
    //
    //    and put the "user-visible" argc into a var as well.
    sprintf (anamebuf, "%ld", j);
    crm_set_temp_var (":_argc:", anamebuf);
    //
    //   Go through argv, and place positional arguments (that is,
    //   arguments that don't contain any '-' preambles) into 
    //   :_pos0:, :_pos1:, ...
    //  
    //   :_pos0: is always the name of the CRM114 engine.
    //   :_pos1: is always the name of the program being run.
    //   :_pos2: and so on are the command line args.
    //
    //    crm_set_temp_var (":_pos0:", argv[0]);
    //crm_set_temp_var (":_pos1:", argv[1]);
    //strcat (posvars, argv[0]);
    //strcat (posvars, " ");
    //strcat (posvars, argv[1]);
    //strcat (posvars, " ");
    //    prepare to treasure up the positional args
    posvars[0] = '\000';
    //j = 2;
    j = 0;
    //    if (uvstart < 2) uvstart = 2;
    for ( i = uvstart; i < argc ; i++ )
      {
	//
	//   check for the "-" sign; this is a positional argument only
	//                     if there is no "-" sign.
	if (argv[i][0] != '-')
	  {
	    sprintf (anamebuf, ":_pos%ld:", j);
	    crm_set_temp_var ( anamebuf, argv[i] );
	    j++;
	    if (j>0)    strcat (posvars, " ");
	    strcat (posvars, argv[i]);
	  };
      };
    sprintf (anamebuf, "%ld", j);
    crm_set_temp_var (":_posc:", anamebuf);
    crm_set_temp_var (":_pos_str:", posvars);
    //
    //   and set the fault to be a null string for now.
    crm_set_temp_var (":_fault:", "");
    //
  };
  
  //      now, we shove the whole contents of the ENVIRON 
  //      vector into the VHT.
  
  i = 0;
  if ( ! ignore_environment_vars)
    while (environ [i])
      {
	char *name;
	char *value ;
	j = 0;
	while (environ[i][j] != '=') j++;
	name = (char *) malloc ((sizeof (char)) * (j+200));
        if (!name)
          untrappableerror("Couldn't malloc :_env_ space.  Can't continue.\n","");
	strcpy (name, ":_env_");
	memmove (&(name[strlen(name)]), &(environ[i][0]), j);
	name[j+6] = '\000';
	strcat (name, ":");
	j++;                  //  step past the equals sign.
	k = 0;
	value = strdup (&(environ[i][j+k]));
	crm_set_temp_var (name, value);
	free (name);
	free (value);
	i++;  //   and do the next environment variable
      };
  
  //    see if argv [1] is a '-( whatever) arg, which limits the
  //    set of runtime parameters allowed on the command line.
  //    If so, we have the limit list.  We put spaces around the
  //    args so we can just use strstr(3) to see if an arg is permitted
  //    or if we should fault out.  Note that at this point,
  //    we've trashed the contents of uvlist (the parens and the
  //    trailing '--', if there was one.
  //
  if (strncmp (argv[1], "-(", 2) == 0)
    {
      long closepos;
      uvlist = 1;
      strcpy (uvset, " ");
      strncat (uvset, &argv[1][2], strlen (argv[1]) - 3);
      //   nuke the closing paren
      closepos = 2;
      while (uvset[closepos] != ')' && uvset[closepos] != '\000')
	closepos++;
      uvset[closepos] = '\000';
      strcat (uvset, " ");
      if (user_trace) fprintf (stderr, "UVset: =%s=\n", uvset);
    }
  //
  //
  //   go through argv again, but this time look for "--foo"
  //   and "--foo=bar" args.  
  //
  {
    long i, j, k;
    char anamebuf [MAX_VARNAME];
    char avalbuf [MAX_VARNAME];
    long isok;
    i = 0; j = 0; k = 0; 
    for ( i = uvstart; argc > i; i++ )
      {
	//   check for the "--" metaflag preamble
	if (strlen ( argv[i] ) > 2 && strncmp (argv[i], "--", 2) == 0)
	  {
	    isok = 1;
	    if (uvlist == 1)
	      {
		isok = 0;
		//   build a testable name out of the -- flagname
		strcpy (anamebuf, " ");
		j=2; k = 1;
		while (argv[i][j] != '\000' && argv[i][j] != '=')
		  {
		    anamebuf[k] = argv[i][j];
		    j++;
		    k++;
		  };
		anamebuf[k] = 0;
		strcat (anamebuf, " ");
		//
		//       now we have the var name, surrounded by spaces
		//       we strstr() it to see if it's allowed or not.
		if (strstr(uvset, anamebuf))  isok = 1;
		//
		//        Well, maybe the name by itself is too loose;
		//        also allow name=value
		strcpy (anamebuf, " ");
		strcat (anamebuf, &argv[i][2]);
		strcat (anamebuf, " ");
		if (strstr(uvset, anamebuf))  isok = 1;
	      }
	    if (isok)
	      {
		if (internal_trace) 
		  fprintf (stderr, "setting cmdline string %s", argv[i]);
		strcpy (avalbuf, "SET");
		j = 2; k = 0;
		//  copy the varname into anamebuf
		anamebuf[k] = ':';
		k++;
		while (argv[i][j] != '\000' && argv[i][j] != '=')
		  {
		    anamebuf[k] = argv[i][j];
		    j++;
		    k++;
		  };
		anamebuf[k] = ':';
		k++;
		anamebuf[k] = '\000';
		if (argv[i][j] == '=')
		  {
		    j++;      //  skip over the = sign
		    k = 0;
		    while (argv[i][j] != '\000')
		      {
			avalbuf[k] = argv[i][j];
			j++;
			k++;
		      }
		    avalbuf [k] = '\000';
		  }
		if (user_trace)
		  fprintf (stderr, "\n Setting cmdline var '%s' to '%s'\n",
			   anamebuf, avalbuf);
		crm_set_temp_var ( anamebuf, avalbuf );
	      }
	    else
	      {
		fprintf (stderr, 
			 "\n ***Warning*** This program does not accept the flag '%s' , \n", anamebuf);
		fprintf (stderr, 
			 " so we'll just ignore it for now. \n");
	      };
	  };
      };
  };
}
//           routine to put a variable into the temporary (tdw) buffer.
//           names and values end up on the same line, separated by newlines
//

void crm_set_temp_nvar (char *varname, char *value, long vallen)
{
  long namestart, namelen;
  long valstart;
  long i;
  long vnidx, vnlen;

  //       do the internal_trace thing
  if (internal_trace)
    fprintf (stderr, "  setting temp-area variable %s to value %s\n",
	     varname, value);

  if (0 == crm_nextword (varname,strlen (varname), 0, &vnidx, &vnlen) )
    {
      nonfatalerror ("Somehow, you are assigning a value to a variable with",
     "an unprintable name.  I'll permit it for now, but your program is probably broken.");
    };
      
  if ( (strlen (varname) + vallen + tdw->nchars + 64) > data_window_size)
    {
      fatalerror ("This program has overflowed the ISOLATEd data area with a variable that's just too big.  Please check this variable: ", varname);
      exit (EXIT_FAILURE);
    };

  //       check- is this the first time we've seen this variable?  Or
  //       are we re-assigning a previous variable?
  i = crm_vht_lookup (vht, &varname[vnidx], vnlen);
  if (vht[i] == NULL)
    {
      //  never assigned this variable before, so we stick it in the 
      //  tdr window. 
      //       do the name first.
      namestart = tdw->nchars;
      namelen = vnlen;
      // strcat (tdw->filetext, varname);
      memmove (&(tdw->filetext[tdw->nchars]), &(varname[vnidx]), namelen);
      tdw->nchars = tdw->nchars + namelen;
      //   
      //        and add a separater for good luck.
      memmove (&(tdw->filetext[tdw->nchars]), "\n", 1);
      tdw->nchars++;
      //       
      //       and the value second
      valstart = tdw->nchars;
      memmove (&tdw->filetext [tdw->nchars], value, vallen);
      tdw->nchars = tdw->nchars + vallen;
      tdw->filetext[tdw->nchars] = '\000';
	
      //   
      //        and add a separater for good luck.
      strcat (tdw->filetext, "\n");
      tdw->nchars++;
      
      //      now, we whack the actual VHT.
      crm_setvar (NULL, 0, 
		  tdw->filetext, namestart, namelen,
		  tdw->filetext, valstart, vallen, 
		  0);
      //     that's it.

    }
  else
    {
      //   This variable is preexisting.  Perform an ALTER on it.
      //   (this fixes the memory leak of multiple set-temps to the sam
      //   variable name
      //   
      //   WARNING WARNING WARNING!  This may cause problems if
      //   the original variable was a capture in the data window!
      //
      crm_destructive_alter_nvariable ( &varname[vnidx], vnlen, 
				       value, vallen );
    };
}

//     GROT GROT GROT this routin needs to replaced for 8-bit-safeness.
//     Use ONLY where you can be sure no embedded NULs will be seen (i.e.
//     fixed strings in the early startup.
//
void crm_set_temp_var (char *varname, char *value)
{
  crm_set_temp_nvar (varname, value, strlen (value));
}



//           routine to put a data-window-based (the cdw, that is)
//           variable's name into the tdw buffer, and the value stays in
//           the main data window (cdw) buffer.  names end up on a
//           single line per name.
//
//           This is equivalent to a "bind" operation - that is, the 
//           pointers move around, but the data window doesn't get
//           changed.


void crm_set_windowed_nvar ( char *varname,
			     long varlen,
			     char *valtext,
			     long start,
			     long len,
			     long stmtnum)
{
  long i;
  long namestart, namelen;
  //       do the internal_trace thing
  if (internal_trace)
    {
      long i;
      fprintf (stderr, "  setting data-window variable %s to value ",
	     varname);
      for (i = start; i < start+len; i++)
	fprintf (stderr, "%c", valtext[i]);
      fprintf (stderr, "\n");
    };

  //    check and see if the variable is already in the VHT
  i = crm_vht_lookup (vht, varname, varlen);
  if (vht[i] == NULL)
    {
      //  nope, never seen this var before, add it into the VHT
      //       namestart is where we are now.
      if (internal_trace)
	fprintf (stderr, "... new var\n");
      namestart = tdw->nchars;
      namelen = varlen;
      memmove (&tdw->filetext[namestart], varname, varlen);
      //      strcat (tdw->filetext, varname);
      tdw->nchars = tdw->nchars + namelen;
      tdw->filetext[tdw->nchars] = '\n';
      tdw->nchars++;
      //      now, we whack the actual VHT.
      crm_setvar (NULL, 0, 
		  tdw->filetext, namestart, namelen,
		  valtext, start, len, 
		  stmtnum);
      //     that's it.
    }
  else
    {
      //     else we've seen this var before, whack it with the 
      //     correct values in the vht.
      if (internal_trace)
	fprintf (stderr, "... old var\n");
      crm_setvar (NULL, 0,
		  vht[i]->nametxt, vht[i]->nstart, vht[i]->nlen,
		  valtext, start, len,
		  stmtnum);
    };
}

// GROT GROT  GROT THIS IS ONLY FOR USE DURING 8-BIT-SAFE CONVERSION

void crm_set_windowed_varzzz (char *varname, 
			   char *text, 
			   long start, 
			   long len,
			   long stmtnum)
{
  crm_set_windowed_nvar ( varname, strlen (varname),
			  text,
			  start,
			  len,
			  stmtnum);
}


//
//     Destructive alteration of a variable in a buffer.
//

void crm_destructive_alter_nvariable (char *varname, long varlen,
				     char *newstr, long newlen)
{
  int i;
  long vhtindex, oldlen, delta;

  //      get the first variable name and verify it exists.
  i = 0;
  while (varname[i] < 0x021) i++;
  vhtindex = crm_vht_lookup (vht, &(varname[i]), varlen);
  if (vht[vhtindex] == NULL)
    {
      nonfatalerror(" Attempt to alterthe value of a nonexistent variable, so I'mcreating an ISOLATED variable.  I hope that's OK.  The nonexistent variable is: ",
		 &(varname[i]));
      crm_set_temp_var (&varname[i], "");
    };
  
  //     make enough space in the input buffer to accept the new value
  oldlen = vht[vhtindex]->vlen;
  delta = newlen - oldlen;
  mdw = NULL;
  if (tdw->filetext == vht[vhtindex]->valtxt)
    mdw = tdw;
  if (cdw->filetext == vht[vhtindex]->valtxt)
    mdw = cdw;
  if (mdw == NULL)
    {
      fatalerror (" Bogus text bloc containing variable ", varname);  
      goto bailout;
    };
  //
  if (user_trace)  // major debug
    {
      fprintf (stderr, "\n     surgery on the var %s, ", varname);
      fprintf (stderr, "new value is: \n***%s***\n", newstr);
    }
  //     slice and splice the mdw text area, to make the right amount of 
  //     space...
  crm_slice_and_splice_window (mdw, vht[vhtindex]->vstart, delta);
  //
  //      Zap the mstart and mlen markers so that searches are reset to start
  //      of the variable.  Note that we have to do this _after_ we slice
  //      and splice, otherwise we mangle our own mstart and mlen.
  vht[vhtindex]->mstart = vht[vhtindex]->vstart;
  vht[vhtindex]->mlen = 0;
  //
  //     now we have space, and we can put in the characters from
  //     the new pattern
  memmove (&(mdw->filetext[vht[vhtindex]->vstart]),
	   newstr,
	   newlen);  
  
 bailout:
};


//      Surgically lengthen or shorten a window.   The window pointed to by
//      mdw gets delta extra characters added or cut at "where".  If the
//      allocated length is not enough, additional space can be malloced.
//      Finally, the vht is fixed up so everything still points "correctly".
void crm_slice_and_splice_window ( CSL_CELL *mdw, long where, long delta)
{
  char *taildest;
  char *tailsrc;
  long taillen;

  //    these are to keep the compiler quiet. 
  taildest = NULL;
  tailsrc = NULL;
  taillen = 0;

  if (delta + mdw->nchars > data_window_size - 10)
    {
      fatalerror (" Data window trying to get too long.", 
		" Try increasing the data window maximum size.");
      goto bailout;
    };
  if (delta == 0)
    {
      if (internal_trace)
	{
	  fprintf (stderr, " zero delta, no buffer hackery required\n");
	};
      return;
    };

  // bump chars in input window delta places
  if (internal_trace)
    {
      fprintf (stderr, "moving text in window %lx,", (long int) mdw->filetext);
      fprintf (stderr, " starting at %ld, ", where); 
      fprintf (stderr, "delta length is %ld\n", delta);
    };
  
  if (delta > 0)
    {             // lengthening alteration...
      taildest = &(mdw->filetext[where + delta]);
      tailsrc = &(mdw->filetext[where]);
      taillen = mdw->nchars - where;
    };

  if (delta < 0)  //   shortening alteration
    {
      taildest = &(mdw->filetext[where]);
      tailsrc = &(mdw->filetext[where - delta]); //  delta is minus already!!
      taillen = mdw->nchars + 1 - where;
    }
  if (internal_trace) 
    fprintf (stderr, 
	     "buffer sliding, tailsrc: %lx, taildest: %lx, length: %ld\n",
	     (long int) tailsrc, (long int) taildest, taillen);

  //     and move the actual data
  memmove ( taildest, tailsrc, taillen + 1 );

  //     update the length of the window as well.
  mdw->nchars = mdw->nchars + delta;

  //      and update all of our captured variables to have the right ptrs.
  crm_updatecaptures (mdw->filetext, 
		      where,
		      delta);
 bailout:
}

  //        allow_data_window_to_grow
#ifdef no_dont_do_this_yet 
  //    Grow the window to hold the incoming text, if needed.
  //    Grow it by 4x each time.
  while (delta + mdw->nchars > data_window_size - 1)
    {
      char *ndw;
      long odws, i;
      odws = data_window_size;
      data_window_size = 4 * data_window_size;
      nonfatalerror (" Data window trying to get too long.", 
  		   " increasing data window... ");
      ndw = (char *) malloc ( data_window_size);
      if (!ndw)
          untrappableerror("Couldn't malloc ndw.  This is bad too.\n","");

      //  now copy the old data window into the new one
      memmove (ndw, mdw->filetext, odws);

      //   and update the outstanding pointers, like the ones in the
      //   vht...
      for (i = 0; i < vht_size; i++)
	if (vht[i] != NULL)
	  {
	    if (vht[i]->nametxt == mdw->filetext) 
	      vht[i]->nametxt = ndw;
	    if (vht[i]->valtxt == mdw->filetext) 
	      vht[i]->valtxt = ndw;
	  };

      //    and lastly, point the cdw or tdw to the new larger window.
      free (mdw->filetext);
      mdw->filetext = ndw;
    };
#endif

//     crm_extractflag - given an arbitrary string cmd (start/len)
//     with words delimited by spaces, and a second string "flag"
//     (start/len).
//
//      1) does "flag" exist in "cmd"?
//      2) if so, where?
//      3) what is the start/len of flag in cmd?
//      4) what is the arg _after_ flag (start/len)
//   
//     Return value - pointer to start of flag in cmd.  It's
//     unnecessary to return the length of flag, as we already know
//     what it is.  also modifies nextarg start and length.
 
long crm_extractflag (const char *cmd, long cmdl, const char *flag, long flagl,
		       long *next, long *nextl)
{
  long i, j, k;
  long is;

  if (internal_trace)
    { 
      fprintf (stderr, "      searching for flag ");
      for (k = 0; k < flagl; k++)
	fprintf (stderr, "%c", flag[k]);
      fprintf (stderr, " in ");

      for (k = 0; k < cmdl; k++)
	fprintf (stderr, "%c", cmd[k]);
      fprintf (stderr, "\n");
    }
  //     loop until we find the first character of flag, or are past end
  //     of possible matching..
  is = 0;
 firstcharscan:
  while ( cmd[is] != flag[0] 
	  && is <= (cmdl - flagl))
    is++;
  
  if (is > cmdl - flagl) 
    return (-1);
  //    found the first char; now see if the rest of the chars match
  //    as well.
  for (  i = is,  j = 0; j < flagl ; i++, j++)
    {
      //    if we don't match, just go up to firstcharscan , move one
      //    character forward, and keep looking.
      if ( cmd [i] != flag [j] )
	{
	  is++;
	  goto firstcharscan;
	}
    };

  //    If we got to here, we got thru the loop and found the flag, or 
  //    we're ar the end of cmd..
  nextl = 0;
  if (is > cmdl - flagl )
    {
      if (next) *next = -1;  //  mark next arg as invalid, if possible.
      return (-1);
    };
  //    check - has the caller requested next arg too?  If not, we can
  //    just return right now.
  if ( next == NULL )
       return (is);
	
  //    If we got here, we've found the flag and there's a request for
  //    next argument as well.  It's just a matter of going thru and
  //    finding the next arg.

  //    GROT GROT GROT   This assumes that i is now pointed at the 
  //    last valid char of flag in cmd. That might not be true if 
  //    the flag was right at the end of the cmd.  If it was, then
  //    we should just return the data, markning next as invalid.

  if (i + 1 >= cmdl )
    {
      *next = -1;
      return (is);
    };    


  //    If we're here, there was space for a subsequent flag.
  //    step forward until we hit a space or go off the end.
  while ( cmd[i] >= 0x021 && i < cmdl)
    i++;

  //    did we fall off the end of cmd?  If so, return "no next arg"
  if (i >=  cmdl)
    {
      *next = -1;
      return (is);
    };

  //   No, we have valid spaces... skip over them.
  while (cmd[i] < 0x021 && i < cmdl)
    i++;
  //       now we're hopefully into the nonblank chars...
  *next = i;
  while ( cmd[i] >= 0x021 && i < cmdl)
    {
      i++;
      *nextl = (*nextl) + 1;
    };

  //   and now we're completely done.  Return from whence we came.

  return (is);
}

//     crm_nexpandvar - given a string and it's length, go through it
//     and if there's a variable expansion called for (by the :*:
//     operator) expand the variable.
//
//     the inputs are a buffer with the NULL-safe string in it, the
//     length of this string, and the maximum allocated length of the
//     buffer.  This function returns the new length of the buffer.
//     It will NOT increase the buffer length past maxlen, so
//     expansions beyond that will cause a nonfatal error and be
//     aborted.
//
//     Algorithm: 
//     1) efficiency check- do we need to do any expansions at all.
//     2) Start at buf[0], work up to buf[buflen]-3
//     2a) do \n, \r, \a, \xHH and \oOOO
//     3) are we looking at :*:?  
//     4) no: copy 1 character, increment from and to indexes, go to step 3
//     5) yes: skip from index ahead 3, from there to next : is the varname
//     6) copy var value to tbuf, incrementing tobuf index.
//     7) set from-index to third colon index + 1
//     8) go to 2 (modulo last two chars need copying)
//
long crm_nexpandvar (char *buf, long inlen, long maxlen)
{
  long is, id;
  long vht_index;
  long q;
  //  a temporary work buffer...
  char *tbuf;
  //  and another for variable names...
  char *vname; 

  char *cp;
  long vlen;

  //    efficiency check - do we even _have_ a :*: in the buffer?
  //

  if (inlen == 0)
    return (0);

  if (q_expansion_mode >= 2)
    return (crm_qexpandvar (buf, inlen, maxlen, NULL));

  //  GROT GROT GROT must fix this for 8-bit safe error messages
  if (inlen > maxlen)
    {
      q = fatalerror (
		      "You have blown the gaskets while building a string.  Orig string was: ",
		      buf);
      if (q == 0 )
	return (inlen);
      goto bailout;
    };  

  //   First thing- do the \-escapes
  //
  is = 0;
  id = 0;
  for (is = 0; is < inlen ; is++)  
    {
      if (buf[is] != '\\' )
	{
	  buf [id] = buf [is];
	  id++;
	}
      else
	{
	  //  we're looking at a '\\'.
	  //  
	  //   Check for a few common things: \n, \a, \xNN, \oNNN
	  is++;
	  //
	  switch (buf[is])
	    {
	    case '0':
	      {
		//   it's a NULL.
		buf[id] = '\0';
		id++;
	      }
	      break;
	    case 'b':
	      {
		//   it's a backspace
		buf[id] = '\b';
		id++;
	      }
	      break;
	    case 't':
	      {
		//   it's a tab
		buf[id] = '\t';
		id++;
	      }
	      break;
	    case 'n':
	      {
		//   it's a newline.  stuff in a newline.
		buf[id] = '\n';
		id++;
	      }
	      break;
	    case 'v':
	      {
		//   it's a vtab
		buf[id] = '\v';
		id++;
	      }
	      break;
	    case 'f':
	      {
		//   it's a form feed.
		buf[id] = '\f';
		id++;
	      }
	      break;
	    case 'r':
	      {
		//   it's a carriage return
		buf[id] = '\r';
		id++;
	      }
	      break;
	    case 'a':
	      {
		//   it's a BELL.  put that in.
		buf[id] = '\a';
		id++;
	      }
	      break;
	    case 'x':
	    case 'X':
	      {
		//   it's a hex char constant.  read it and stuff it.
		unsigned int value;
		is++;
		sscanf (&buf[is], "%2X", &value);
		buf[id] = value;
		id++;
		is++;
	      }
	      break;
	    case 'o':
	    case 'O':
	      {
		//   it's an octal char constant.  read it and stuff it.
		unsigned int value;
		is++;
		sscanf (&buf[is], "%o3", &value);
		buf[id] = value;
		id++;
		is++;
		is++;
	      }
	      break;
	    default:
	      {
		//other next characters are verbatim after a '\\'
		buf[id] = buf[is];
		id++;
	      };
	    };
	};
    };
  //     and update the new inlen
  inlen = id ;
  buf[inlen] = '\000';    // needed because slimy old GNU REGEX needs it.

  //    if no *, then no :*: and so no expansions needed
  cp = memchr (buf, '*', inlen);
  if (cp == NULL)
    {
      return (inlen);
    };

  //    OK, we might have a :*: substitution operator, so we actually have
  //    to do some work.
  //    allocate some memory for tbuf and vname;
  tbuf = (char *) malloc (maxlen);
  vname = (char *) malloc (maxlen);

  if (tbuf == NULL || vname == NULL)
    {
      q = fatalerror ("Couldn't allocate memory for variable expansion!",
		      "Try making the window set smaller with the -w option");
      if (q == 0)
	return (inlen);
    };
  
  is = 0;    //   is is the input position index
  id = 0;    //   id is the destination position index
  for (is = 0; is <= inlen && id < maxlen; is++)
    {
      if (is <= inlen - 5  //  check only if :*:c:" possible
	  && buf[is] == ':' 
	  && buf[is+1] == '*'
	  && buf[is+2] ==':')
	{
	  //   yes, it's a probable variable.
	  //    copy everything from the colon to the second colon
	  //    into the vname buffer.
	  is = is + 2;
	  vname [0] = buf[is];
	  vlen = 1;
	  is++;
	  while (is < maxlen
		 && is <= inlen
		 && buf [is] != ':')
	    {
	      vname[vlen] = buf[is];
	      is++;
	      vlen++;
	    };
	  //
	  //    check for the second colon as well...
	  if (buf[is] == ':')
	    {
	      vname[vlen] = ':';
	      vlen++;
	    }
	  vname [vlen] = '\000';
	  
	  //
	  //      Now we've got the variable name in vname, we can 
	  //      go get it's value and copy _that_ into tbuf as well.
	  if (internal_trace)
	    fprintf (stderr, "looking up variable >%s<\n", vname);
	  vht_index = crm_vht_lookup (vht, vname, vlen);
	  
	  if (vht[vht_index] == NULL)
	    {
	      //      there was no variable by that name, just put the
	      //     name itself there.  Note that we retain the :'s
	      //     but that the :* prefix goes away.
	      for (q = 0; q < vlen && id < maxlen; q++)
		{
		  tbuf[id] = vname[q];
		  id++;
		}
	    }
	  else
	    {
	      //     There really was a variable value by that name.
	      //     suck it out, and splice it in!
	      
	      for (q = 0; q < vht[vht_index]->vlen && id < maxlen; q++)
		{
		  tbuf[id] = vht[vht_index]->valtxt
		    [(vht[vht_index]->vstart)+q];
		  id++;
		}
	    };
	}
      //         Now, handle the case where we were NOT looking at
      //         :*:c: in buf
      else
	{
	  tbuf[id] = buf[is];
	  id++;
	}
    }
  //    That's all, folks!  Clean up the temporary buffer.  We null-terminate
  //    it in case we need to do stupid non-8-bit-clean IO on it.
  tbuf[id] = '\000';
  memmove (buf, tbuf, id);
  free (tbuf);
  free (vname);
  id--;  //  the actual length is id-1, since id is the next available char
  if (internal_trace)
    fprintf (stderr, " Returned length from nexpandvar is %ld\n", id);
  return (id);
 bailout:
  return (inlen);
}


//     crm_qexpandvar - "expanded" expandvar.  Like nexpandvar, but moreso.
//
//     nexpandvar just does \ and :*: expansion.  qexpandvar also does
//     :#:, :$:, and a bunch of other stuff.  (not recursively though.)
//
//     the inputs are a buffer with the NULL-safe string in it, the
//     length of this string, and the maximum allocated length of the
//     buffer.  This function returns the new length of the buffer.
//     It will NOT increase the buffer length past maxlen, so
//     expansions beyond that will cause a nonfatal error and be
//     aborted.
//
//     Algorithm: 
//     1) efficiency check- do we need to do any expansions at all.
//     2) Start at buf[0], work up to buf[buflen]-3
//     2a) do \n, \r, \a, \xHH and \oOOO
//     3) are we looking at :<some-operator>:?  
//     4) no: copy 1 character, increment from and to indexes, go to step 3
//     5) yes: skip from index ahead 3, from there to next : is the varname
//     6) copy var value to tbuf, incrementing tobuf index.
//     7) set from-index to third colon index + 1
//     8) go to 2 (modulo last two chars need copying)
//
long crm_qexpandvar (char *buf, long inlen, long maxlen, long *retstat)
{
  long is, id;
  long vht_index;
  long q;
  //  a temporary work buffer...
  char *tbuf;
  //  and another for variable names...
  char *vname; 

  char *cp;
  long vlen;

  char opchar;

  //    efficiency check - do we even _have_ a :*: in the buffer?
  //

  if (inlen == 0)
    return (0);

  if (internal_trace) 
    fprintf (stderr, "qexpandvar on =%s= len %ld\n", buf, inlen);

  //  GROT GROT GROT must fix this for 8-bit safe error messages
  if (inlen > maxlen)
    {
      q = fatalerror (
		      "You have blown the gaskets while building a string.  Orig string was: ",
		      buf);
      if (q == 0 )
	return (inlen);
      goto bailout;
    };  

  //   First thing- do the \-escapes
  //
  is = 0;
  id = 0;
  for (is = 0; is < inlen ; is++)  
    {
      if (buf[is] != '\\' )
	{
	  buf [id] = buf [is];
	  id++;
	}
      else
	{
	  //  we're looking at a '\\'.
	  //  
	  //   Check for a few common things: \n, \a, \xNN, \oNNN
	  is++;
	  //
	  switch (buf[is])
	    {
	    case '0':
	      {
		//   it's a NULL.
		buf[id] = '\0';
		id++;
	      }
	      break;
	    case 'b':
	      {
		//   it's a backspace
		buf[id] = '\b';
		id++;
	      }
	      break;
	    case 't':
	      {
		//   it's a tab
		buf[id] = '\t';
		id++;
	      }
	      break;
	    case 'n':
	      {
		//   it's a newline.  stuff in a newline.
		buf[id] = '\n';
		id++;
	      }
	      break;
	    case 'v':
	      {
		//   it's a vtab
		buf[id] = '\v';
		id++;
	      }
	      break;
	    case 'f':
	      {
		//   it's a form feed.
		buf[id] = '\f';
		id++;
	      }
	      break;
	    case 'r':
	      {
		//   it's a carriage return
		buf[id] = '\r';
		id++;
	      }
	      break;
	    case 'a':
	      {
		//   it's a BELL.  put that in.
		buf[id] = '\a';
		id++;
	      }
	      break;
	    case 'x':
	    case 'X':
	      {
		//   it's a hex char constant.  read it and stuff it.
		unsigned int value;
		is++;
		sscanf (&buf[is], "%2X", &value);
		buf[id] = value;
		id++;
		is++;
	      }
	      break;
	    case 'o':
	    case 'O':
	      {
		//   it's an octal char constant.  read it and stuff it.
		unsigned int value;
		is++;
		sscanf (&buf[is], "%o3", &value);
		buf[id] = value;
		id++;
		is++;
		is++;
	      }
	      break;
	    default:
	      {
		//other next characters are verbatim after a '\\'
		buf[id] = buf[is];
		id++;
	      };
	    };
	};
    };
  //     and update the new inlen
  inlen = id ;
  buf[inlen] = '\000';    // needed because slimy old GNU REGEX needs it.

  if (internal_trace)
    fprintf (stderr, "backslash expansion yields: =%s= len %ld\n", buf, inlen);

  //    if no :, then no operators possible.
  cp = memchr (buf, ':', inlen);
  if (cp == NULL)
    {
      return (inlen);
    };

  //    OK, we might have a :*: substitution operator, so we actually have
  //    to do some work.
  //    allocate some memory for tbuf and vname;
  tbuf = (char *) malloc (maxlen);
  vname = (char *) malloc (maxlen);

  if (tbuf == NULL || vname == NULL)
    {
      q = fatalerror ("Couldn't allocate memory for Q-variable expansion!",
		      "Try making the window set smaller with the -w option");
      if (q == 0)
	return (inlen);
    };
  
  is = 0;    //   is is the input position index
  id = 0;    //   id is the destination position index

  //
  //   First time through the loop, for :*: (variable expansion)
  //
  for (is = 0; is <= inlen && id < maxlen; is++)
    {
      if (is <= inlen - 5  //  check only if :*:c:" possible
	  && buf[is] == ':' 
	  && ( buf[is+1] == '*' )
	  && buf[is+2] ==':')
	{
	  //   yes, it's probably an expansion of some sort.
	  opchar = buf[is+1];
	  //    copy everything from the colon to the second colon
	  //    into the vname buffer.
	  is = is + 2;
	  vname [0] = buf[is];
	  vlen = 1;
	  is++;
	  while (is < maxlen
		 && is <= inlen
		 && buf [is] != ':')
	    {
	      vname[vlen] = buf[is];
	      is++;
	      vlen++;
	    };
	  //
	  //    check for the second colon as well...
	  if (buf[is] == ':')
	    {
	      vname[vlen] = ':';
	      vlen++;
	    }
	  vname [vlen] = '\000';
	  
	  //
	  //      Now we've got the variable name in vname, we can 
	  //      go get it's value and copy _that_ into tbuf as well.
	  if (internal_trace)
	    fprintf (stderr, "looking up variable >%s<\n", vname);
	  vht_index = crm_vht_lookup (vht, vname, vlen);
	  
	  if (vht[vht_index] == NULL)
	    {
	      //      there was no variable by that name, use the text itself
	      switch (opchar)
		{
		case '*':
		  {
		    //
		    //    simply copy text till the close colon
		    //
		    for (q = 0; q < vlen && id < maxlen; q++)
		      {
			tbuf[id] = vname[q];
			id++;
		      }
		  }
		  break;
		}
	    }
	  else
	    {
	      //     There really was a variable value by that name.
	      //     suck it out, and splice it's text value
	      switch (opchar)
		{
		case '*':
		  {
		    for (q = 0; q < vht[vht_index]->vlen && id < maxlen; q++)
		      {
			tbuf[id] = vht[vht_index]->valtxt
			  [(vht[vht_index]->vstart)+q];
			id++;
		      }
		  }
		  break;
		};
	    };
	}
      //         Now, handle the case where we were NOT looking at
      //         :*:c: in buf
      else
	{
	  tbuf[id] = buf[is];
	  id++;
	}
    }
  //
  //
  //   Second time through the loop - expand :#: (string lengths)
  //
  strncpy (buf, tbuf, id);
  buf[id] = '\000';
  inlen = id-1 ;        // since id gets one last increment.
  if (internal_trace)
    fprintf (stderr, " var-expand yields: =%s= len %ld\n", buf, inlen);
  id = 0;
  for (is = 0; is <= inlen && id < maxlen; is++)
    {
      if (is <= inlen - 5  //  check only if :#:c:" possible
	  && buf[is] == ':' 
	  && ( buf[is+1] == '#' )
	  && buf[is+2] ==':')
	{
	  //   yes, it's probably an expansion of some sort.
	  opchar = buf[is+1];
	  //    copy everything from the colon to the second colon
	  //    into the vname buffer.
	  is = is + 2;
	  vname [0] = buf[is];
	  vlen = 1;
	  is++;
	  while (is < maxlen
		 && is <= inlen
		 && buf [is] != ':')
	    {
	      vname[vlen] = buf[is];
	      is++;
	      vlen++;
	    };
	  //
	  //    check for the second colon as well...
	  if (buf[is] == ':')
	    {
	      vname[vlen] = ':';
	      vlen++;
	    }
	  vname [vlen] = '\000';
	  
	  //
	  //      Now we've got the variable name in vname, we can 
	  //      go get it's value and copy _that_ into tbuf as well.
	  if (internal_trace)
	    fprintf (stderr, "looking up variable >%s<\n", vname);
	  vht_index = crm_vht_lookup (vht, vname, vlen);
	  
	  if (vht[vht_index] == NULL)
	    {
	      //      there was no variable by that name, use the text itself
	      switch (opchar)
		{
		case '#':
		    {
		      char lentext[MAX_VARNAME];
		      int m, mm;
		      //   the vlen-2 is because we need to get rid of the ':' 
		      sprintf (lentext, "%ld", vlen-2);
		      mm = strlen (lentext);
		      for (m = 0; m < mm; m++)
			{
			  tbuf[id] = lentext[m];
			  id++;
			};
		    }
		  break;
		}
	    }
	  else
	    {
	      //     There really was a variable value by that name.
	      //     suck it out, and splice it's text value
	      switch (opchar)
		{
		case '#':
		  {
		    //
		    //   Actually, we want the _length_ of the variable
		    //
		    char lentext[MAX_VARNAME];
		    int m, mm;
		    sprintf (lentext, "%ld", vht[vht_index]->vlen);
		    mm = strlen (lentext);
		    for (m = 0; m < mm; m++)
		      {
			tbuf[id] = lentext[m];
			id++;
		      };
		  };
		  break;
		};
	    };
	}
      //         Now, handle the case where we were NOT looking at
      //         :*:c: in buf
      else
	{
	  tbuf[id] = buf[is];
	  id++;
	}
    }
  //
  //
  //      Third pass - handle :@:  (math evaluations)
  //
  //
  strncpy (buf, tbuf, id);
  buf[id] = '\000';
  inlen = id - 1;               //  since id got one extra increment
  if (internal_trace)
    fprintf (stderr, " length-expand yields: =%s= len %ld\n", buf, inlen);
  id = 0;
  for (is = 0; is <= inlen && id < maxlen; is++)
    {
      if (is <= inlen - 5  //  check only if :*:c:" possible
	  && buf[is] == ':' 
	  && ( buf[is+1] == '@' )
	  && buf[is+2] ==':')
	{
	  //   yes, it's probably an expansion of some sort.
	  opchar = buf[is+1];
	  //    copy everything from the colon to the second colon
	  //    into the vname buffer.
	  is = is + 2;
	  vname [0] = buf[is];
	  vlen = 1;
	  is++;
	  while (is < maxlen
		 && is <= inlen
		 && buf [is] != ':')
	    {
	      vname[vlen] = buf[is];
	      is++;
	      vlen++;
	    };
	  //
	  //    check for the second colon as well...
	  if (buf[is] == ':')
	    {
	      vname[vlen] = ':';
	      vlen++;
	    }
	  vname [vlen] = '\000';

	  //
	  //      Now we've got the variable name in vname, we can 
	  //      go get it's value and copy _that_ into tbuf as well.
	  if (internal_trace)
	    fprintf (stderr, "looking up variable >%s<\n", vname);
	  vht_index = crm_vht_lookup (vht, vname, vlen);
	  
	  if (vht[vht_index] == NULL)
	    {
	      //      there was no variable by that name, use the text itself
	      switch (opchar)
		{
		case '@':
		    {
		      char mathtext[MAX_VARNAME];
		      int m, mm;
		      strncpy (mathtext, &vname[1], vlen-2);
		      mathtext[vlen-2] = '\000';
		      if (internal_trace)
			fprintf (stderr, "In-Mathtext is -'%s'-\n", mathtext); 
		      m = strmath (mathtext, vlen-2, MAX_VARNAME, retstat);
		      if (internal_trace)
			fprintf (stderr, "Out-Mathtext is -'%s'-\n", mathtext);
		      if (retstat && *retstat < 0)
			{
			  q = fatalerror ("Problem during math evaluation of ",
					  mathtext);
			  if (q == 0)
			    return (inlen);
			  goto bailout;
			}
		      mm = strlen (mathtext);
		      for (m = 0; m < mm; m++)
			{
			  tbuf[id] = mathtext[m];
			  id++;
			};
		    }
		  break;
		}
	    }
	  else
	    {
	      //     There really was a variable value by that name.
	      //     suck it out, and splice it's text value
	      switch (opchar)
		{
		case '@':
		    {
		      char mathtext[MAX_VARNAME];
		      int m, mm;
		      m = 0;
		      for (q = 0; q < vht[vht_index]->vlen && m < maxlen; q++)
			{
			  mathtext[m] = vht[vht_index]->valtxt
			    [(vht[vht_index]->vstart)+q];
			  m++;
			}
		      mathtext[vlen-1] = '\000';
		      m = strmath (mathtext, vlen-2, MAX_VARNAME, retstat );
		      if (retstat && *retstat < 0)
			{
			  q = fatalerror ("Problem during math evaluation of ",
					  mathtext);
			  if (q == 0)
			    return (inlen);
			  goto bailout;
			}
		      mm = strlen (mathtext);
		      for (m = 0; m < mm; m++)
			{
			  tbuf[id] = mathtext[m];
			  id++;
			};
		    }
		  break;

		};
	    };
	}
      //         Now, handle the case where we were NOT looking at
      //         :*:c: in buf
      else
	{
	  tbuf[id] = buf[is];
	  id++;
	}
    }


  //    That's all, folks!  Clean up the temporary buffer.  We null-terminate
  //    it in case we need to do stupid non-8-bit-clean IO on it.
  tbuf[id] = '\000';
  memmove (buf, tbuf, id);
  id--;  //  the actual length is id-1, since id is the next available char
  if (internal_trace)
    fprintf (stderr, " math-expand yields: =%s= len %ld\n", buf, id);
  free (tbuf);
  free (vname);
  if (internal_trace)
    {
      fprintf (stderr, " Returned length from qexpandvar is %ld\n", id);
      if (retstat) fprintf (stderr, "retstat was: %ld\n", *retstat);
    };
  return (id);
 bailout:
  return (inlen);
}


//     strnhash - generate the hash of a string of length N
//     goals - fast, works well with short vars includng 
//     letter pairs and palindromes, not crypto strong, generates
//     hashes that tend toward relative primality against common
//     hash table lengths (so taking the output of this function
//     modulo the hash table length gives a relatively uniform distribution
//
//     In timing tests, this hash function can hash over 10 megabytes
//     per second (using as text the full 2.4.9 linux kernel source)
//     hashing individual whitespace-delimited tokens, on a Transmeta
//     666 MHz.

long strnhash (char *str, long len)
{
  long i;
  long hval;
  char *hstr;
  char chtmp;

  // initialize hval
  hval= len;

  hstr = (char *) &hval;

  //  for each character in the incoming text:

  for ( i = 0; i < len; i++)
    {
      //    xor in the current byte against each byte of hval
      //    (which alone gaurantees that every bit of input will have
      //    an effect on the output)
      hstr[0] = (hstr[0] & ( ~ str[i] ) ) | ((~ hstr [0]) & str[i]);
      hstr[1] = (hstr[1] & ( ~ str[i] ) ) | ((~ hstr [1]) & str[i]);
      hstr[2] = (hstr[2] & ( ~ str[i] ) ) | ((~ hstr [2]) & str[i]);
      hstr[3] = (hstr[3] & ( ~ str[i] ) ) | ((~ hstr [3]) & str[i]);

      //    add some bits out of the middle as low order bits.
      hval = hval + (( hval >> 12) & 0x0000ffff) ;
		     
      //     swap bytes 0 with 3 
      chtmp = hstr [0];
      hstr[0] = hstr[3];
      hstr [3] = chtmp;

      //    rotate hval 3 bits to the left (thereby making the
      //    3rd msb of the above mess the hsb of the output hash)
      hval = (hval << 3 ) + (hval >> 29);
    }
  return (hval);
}


//
//           strmath - evaluate a string for the mathematical result
//
long strmath (char *buf, long inlen, long maxlen, long *retstat)
{
  if (q_expansion_mode == 0 || q_expansion_mode == 2)
    {
      return (stralmath (buf, inlen, maxlen, retstat));
    }
  else
    {
      return (strpnmath (buf, inlen, maxlen, retstat));
    }
}

//    strpnmath - do a basic math evaluation of very simple expressions.
//
//    This does math, in RPN, on a string, and returns a string value.
//
long strpnmath (char *buf, long inlen, long maxlen, long *retstat)
{
  double stack [100];     // the evaluation stack
  double sd;             //  how many 10^n's we've seen since a decimal
  long od;               //  output decimal flag               
  long ip, op;           //  in string pointer, out string pointer
  long sp;               //  stack pointer - points to next (vacant) space
  long sinc;             //  stack incrmenter - do we push on next digit in?



  //    start off by initializing things
  ip = 0;    //  in pointer is zero
  op = 0;    // output pointer is zero
  sp = 0;    // still at the top of the stack
  od = 0;    // no decimals seen yet, so no flag to output in decimal
  sinc = 0;  // no autopush.

  //     now our number-inputting hacks
  stack[sp] = 0.0 ; 
  sd = 1.0;

  //      all initialized... let's begin.
  
  if (internal_trace) 
    fprintf (stderr, "Math on '%s' len %ld retstat %lx \n", 
	     buf, inlen, (long) retstat);

  for (ip = 0; ip < inlen; ip++)
    {
      if (internal_trace)
	fprintf (stderr, "ip = %ld, sp = %ld, stack[sp] = %f, ch='%c'\n", 
	       ip, sp, stack[sp], buf[ip]);
      switch (buf[ip])
	{
	  //
	  //        a digit - for now, that's a 10x "look" + the digit value
	  //
	case '0':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 0; }
	    else
	      { stack[sp] = stack[sp] + 0 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '1':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 1; }
	    else
	      { stack[sp] = stack[sp] + 1 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '2':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 2; }
	    else
	      { stack[sp] = stack[sp] + 2 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '3':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 3; }
	    else
	      { stack[sp] = stack[sp] + 3 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '4':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 4; }
	    else
	      { stack[sp] = stack[sp] + 4 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '5':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 5; }
	    else
	      { stack[sp] = stack[sp] + 5 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '6':
	  {	
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 6; }
	    else
	      { stack[sp] = stack[sp] + 6 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '7':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 7; }
	    else
	      { stack[sp] = stack[sp] + 7 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '8':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 8; }
	    else
	      { stack[sp] = stack[sp] + 8 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '9':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    if (sd == 1.0) 
	      { stack[sp] = stack[sp] * 10 + 9; }
	    else
	      { stack[sp] = stack[sp] + 9 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '.':
	  {
	    if (sinc) { sp++; stack[sp] = 0.0; };
	    sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	  //
	  //         and some basic math ops...
	  //
	case '+':
	  {
	    if (sp > 0)
	      {
		sp--;
		stack[sp] = stack[sp] + stack[sp+1];
		sinc = 1;
	      }
	  };
	  break;
	case '-':
	  {
	    if (sp > 0)
	      {
		sp--;
		stack[sp] = stack[sp] - stack[sp+1];
		sinc = 1;
	      }
	  };
	  break;
	case '*':
	  {
	    if (sp > 0)
	      {
		sp--;
		stack[sp] = stack[sp] * stack[sp+1];
		sinc = 1;
	      }
	  };
	  break;
	case '/':
	  {
	    if (sp > 0)
	      {
		sp--;
		if(stack[sp+1] != 0.0)
		  {
		    stack[sp] = stack[sp] / stack[sp+1];
		    sinc = 1;
		  }
		else
		  {
		    if (retstat) *retstat = -1;
		    nonfatalerror ("Attempt to divide by zero in this:",
				   inbuf);
		  }
	      }
	  };
	  break;
	case '%':
	  {
	    if (sp > 0)
	      {
		sp--;
		stack[sp] = ((long) stack[sp]) % ((long)stack[sp+1]);
		sinc = 1;
	      }
	  };
	  break;

	case '=':
	  {
	    if (sp > 0)
	      {
		sp--;
		if (stack[sp] == stack[sp+1])
		  {
		    if (retstat) *retstat = 0;
		    stack[sp] = 1;
		  }
	        else 
		  {
		    if (retstat) *retstat = 1;
		    stack[sp] = 0;
		  };
		sinc = 1;
	      }
	  };
	  break;

	case '>':
	  {
	    if (sp > 0)
	      {
		sp--;
		if (stack[sp] > stack[sp+1])
		  {
		    if (retstat) *retstat = 0;
		    stack[sp] = 1;
		  }
	        else 
		  {
		    if (retstat) *retstat = 1;
		    stack[sp] = 0;
		  };
		sinc = 1;
	      }
	  };
	  break;

	case '<':
	  {
	    if (sp > 0)
	      {
		sp--;
		if (stack[sp] < stack[sp+1])
		  {
		    if (retstat) *retstat = 0;
		    stack[sp] = 1;
		  }
	        else 
		  {
		    if (retstat) *retstat = 1;
		    stack[sp] = 0;
		  };
		sinc = 1;
	      }
	  };
	  break;

	case ' ':
	default:
	  //
	  //        a space is just an end-of-number - push the number we're 
	  //        seeing.  
	  {
	    sinc = 1;
	  };
	  break;
	};
    };

  if (internal_trace)
    {
      fprintf (stderr, 
	     "Final qexpand state:  ip = %ld, sp = %ld, stack[sp] = %f, ch='%c'\n", 
	       ip, sp, stack[sp], buf[ip]);
      if (retstat) 
	fprintf (stderr, "retstat = %ld\n", *retstat);
    };

  //      now the top of stack contains the result of the calculation.
  //      fprintf it into the output buffer, and we're done.

  //
  //      print out 0 as 0
  //
  if (stack[sp] == 0.0 )
    {
      sprintf (buf, "0");
      goto formatdone;
    }
  //
  //       use E notation if bigger than 10 million
  //
  if (stack[sp] > 100000000.0 || stack[sp] < -100000000.0 )
    {
      sprintf (buf, "%10.3e", stack[sp]);
      goto formatdone;
    }
  // 
  //       use E notation if smaller than .0000001
  //
  if (stack[sp] < 0.000001 && stack[sp] > -0.000001  )
    {
      sprintf (buf, "%10.3e", stack[sp]);
      goto formatdone;
    }
  //
  //       if an integer value, print with 0 precision
  //
  if (((long)(stack[sp]*2.0) / 2) == stack[sp])
    {
      sprintf (buf, "%.0f", stack[sp]);
      goto formatdone;
    }
  //
  //       if none of the above, print with three digits precision
  //
  sprintf (buf, "%.3f", stack[sp]);
  //
  //         one way or another, once we're here, we've sprinted it.
 formatdone:
  return (strlen (buf));
}

//
//      stralnmath - evaluate a mathematical expression in algebraic
//    (that is, infix parenthesized) notation.
//
//      The algorithm is this:
//    see an open parenthesis - push an empty level
//    see a close parethesis -  try to "reduce", then pop over the empty 
//    see an operator - push it onto opstack, sp++
//    see a number - push it, then try to "reduce" if there's a valid op.
//
//    reduce: 
//         if sp > 0 
//             if opstack[sp-1] valid
//                   execute opstack[sp-1] on [sp], [sp-1]
//                   replace sp-1 with result
//                   sp--
//

long stralmath (char *buf, long inlen, long maxlen, long *retstat)
{
  double valstack [100];     // the evaluation value stack
  char opstack [100];     // the evaluation operator stack (char)
  double sd;             //  how many 10^n's we've seen since a decimal
  long od;               //  output decimal flag               
  long ip, op;           //  in string pointer, out string pointer
  long sp;               //  stack pointer - points to next (vacant) space
  long sinc;             //  stack incrmenter - do we push on next digit in?


  //    start off by initializing things
  ip = 0;    //  in pointer is zero
  op = 0;    // output pointer is zero
  sp = 0;    // still at the top of the stack
  od = 0;    // no decimals seen yet, so no flag to output in decimal
  sinc = 0;  // no autopush.

  //     now our number-inputting hacks
  valstack[sp] = 0.0 ; 
  opstack[sp] = '\000';
  sd = 1.0;

  //      all initialized... let's begin.

  if (internal_trace) 
    fprintf (stderr, "Math on '%s' len %ld *retstat %lx \n", 
	     buf, inlen, (long) retstat);

  for (ip = 0; ip < inlen; ip++)
    {
      if (internal_trace)
	fprintf (stderr, "ip = %ld, sp = %ld, valstack[sp] = %f," 
		 "opstack = '%c', h='%c'\n",
	       ip, sp, valstack[sp], opstack[sp], buf[ip]);
      switch (buf[ip])
	{
	  //
	  //        a digit - for now, that's a 10x "look" + the digit value
	  //
	case '0':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000'; }; 
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 0; }
	    else
	      { valstack[sp] = valstack[sp] + 0 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '1':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 1; }
	    else
	      { valstack[sp] = valstack[sp] + 1 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '2':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 2; }
	    else
	      { valstack[sp] = valstack[sp] + 2 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '3':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 3; }
	    else
	      { valstack[sp] = valstack[sp] + 3 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '4':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 4; }
	    else
	      { valstack[sp] = valstack[sp] + 4 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '5':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 5; }
	    else
	      { valstack[sp] = valstack[sp] + 5 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '6':
	  {	
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 6; }
	    else
	      { valstack[sp] = valstack[sp] + 6 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '7':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 7; }
	    else
	      { valstack[sp] = valstack[sp] + 7 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '8':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 8; }
	    else
	      { valstack[sp] = valstack[sp] + 8 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '9':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    if (sd == 1.0) 
	      { valstack[sp] = valstack[sp] * 10 + 9; }
	    else
	      { valstack[sp] = valstack[sp] + 9 * sd; };
	    if (sd < 1.0) sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	case '.':
	  {
	    if (sinc) { sp++; valstack[sp] = 0.0; opstack[sp] = '\000';};
	    sd = sd / 10.0;
	    sinc = 0;
	  };  
	  break;
	  //
	  //         and now the parenthesis ops
	  //
	  //        open paren- we start a new stack level
	case '(':
	  {
	    sd = 1.0;
	    sp++;
	    valstack[sp] = 0.0;
	    opstack[sp] = '\000';
	  };
	  break;
	  //
	  //        close paren- we finish evaluation down to the open,
	  //        then supply the result to the next lower level.
	case ')':
	  {
	    long state;
	    if (sp > 0) sp--;
	    state = stralmath_reduce (valstack, opstack, &sp);
	    if (retstat) *retstat = state;
	    if (sp > 0) sp--;
	    //   and get rid of that extra level we inserted before.
	    valstack[sp] = valstack[sp+1];
	  };
	  break;
	  //   
	  //      The operators just put themselves on the opstack.
	  //
	case '+':
	case '-':
	case '*':
	case '/':
	case '%':
	case '=':
	case '>':
	case '<':
	  {
	    if (sp > 0 && opstack[sp-1] != '\000')
	      {
		long state;
		if (sp > 0) sp--;
		state = stralmath_reduce (valstack, opstack, &sp);
		if (retstat) *retstat = state;
	      };
	    sd = 1.0;
	    sinc = 1;
	    opstack[sp] = buf[ip];
	    if (opstack[sp] == ' ') opstack[sp] = '\000';
	  };
	  break;

	default:
	  break;
	};
    };

  //  Now do final executes....
  if (sp > 0) 
    {
      long state;
      sp--;
      state = stralmath_reduce (valstack, opstack, &sp);
      if (retstat) *retstat = state; 
    };



  if (internal_trace)
    {
      fprintf (stderr, 
	     "Final qexpand state:  ip = %ld, sp = %ld, valstack[sp] = %f, ch='%c'\n", 
	       ip, sp, valstack[sp], buf[ip]);
      if (retstat) 
	fprintf (stderr, "retstat = %ld\n", *retstat);
    };

  //      now the top of stack contains the result of the calculation.
  //      fprintf it into the output buffer, and we're done.

  //
  //      print out 0 as 0
  //
  if (valstack[sp] == 0.0 )
    {
      sprintf (buf, "0");
      goto formatdone;
    }
  //
  //       use E notation if bigger than 10 million
  //
  if (valstack[sp] > 100000000.0 || valstack[sp] < -100000000.0 )
    {
      sprintf (buf, "%10.3e", valstack[sp]);
      goto formatdone;
    }
  // 
  //       use E notation if smaller than .0000001
  //
  if (valstack[sp] < 0.000001 && valstack[sp] > -0.000001  )
    {
      sprintf (buf, "%10.3e", valstack[sp]);
      goto formatdone;
    }
  //
  //       if an integer value, print with 0 precision
  //
  if (((long)(valstack[sp]*2.0) / 2) == valstack[sp])
    {
      sprintf (buf, "%.0f", valstack[sp]);
      goto formatdone;
    }
  //
  //       if none of the above, print with three digits precision
  //
  sprintf (buf, "%.3f", valstack[sp]);
  //
  //         one way or another, once we're here, we've sprinted it.
 formatdone:
  return (strlen (buf));
}
//
//            stralmath_reduce - actually do the math for algebraic arithmetic
//
long stralmath_reduce (double *valstack, char *opstack, long *sp)
{
  long retval;
  retval = 0;
  if (internal_trace)
    fprintf (stderr, "  start: *sp = %3ld, "
	     "vs[*sp] = %6.3f, vs[*sp+1] = %6.3f, "
	     "op[*sp] = '%c'\n", 
	     *sp, valstack[*sp], valstack[*sp+1], opstack[*sp]);
  
  while (*sp >= 0 && opstack[*sp] != '\000')
    {
      if (internal_trace)
	fprintf (stderr, "running: *sp = %3ld, "
		 "vs[*sp] = %6.3f, vs[*sp+1] = %6.3f, "
		 "op[*sp] = '%c'\n", 
		 *sp, valstack[*sp], valstack[*sp+1], opstack[*sp]);
      switch (opstack[*sp])
	{
	case '+':
	  valstack[*sp] = valstack[*sp] + valstack[*sp+1];
	  opstack[*sp] = '\000';
	  break;
	case '-':
	  valstack[*sp] = valstack[*sp] - valstack[*sp+1];
	  opstack[*sp] = '\000';
	  break;
	case '*':
	  valstack[*sp] = valstack[*sp] * valstack[*sp+1];
	  opstack[*sp] = '\000';
	  break;
	case '/':
	  if (valstack[*sp+1] == 0.0) 
	    {
	      opstack[*sp+1] = '\000';
	      nonfatalerror ("Attempt to divide by zero.","");
	      return (-1);		    
	    };
	  valstack[*sp] = valstack[*sp] / valstack[*sp+1];
	  opstack[*sp+1] = '\000';
	  break;
	case '%':
	  valstack[*sp] = ((long)valstack[*sp]) % ((long) valstack[*sp+1]);
	  opstack[*sp] = '\000';
	  break;
	case '=':
	  if (valstack[*sp] == valstack[*sp+1])
	    {
	      valstack[*sp] = 1 ;
	    }
	  else
	    {
	      valstack[*sp] = 0;
	    }
	  opstack[*sp] = '\000';
	  retval = 1 - valstack[*sp]; 
	  break;
	case '>':
	  if (valstack[*sp] > valstack[*sp+1])
	    {
	      valstack[*sp] = 1 ;
	    }
	  else
	    {
	      valstack[*sp] = 0;
	    }
	  opstack[*sp] = '\000';
	  retval = 1 - valstack[*sp]; 
	  break;
	case '<':
	  if (valstack[*sp] < valstack[*sp+1])
	    {
	      valstack[*sp] = 1 ;
	    }
	  else
	    {
	      valstack[*sp] = 0;
	    }
	  opstack[*sp] = '\000';
	  retval = 1 - valstack[*sp]; 
	  break;
	default:
	  break;
	};
      if (*sp > 0 && opstack[(*sp)-1] != '\000') *sp = *sp - 1;
    };
  if (internal_trace)
    fprintf (stderr, " finish: *sp = %3ld, "
	     "vs[*sp] = %6.3f, vs[*sp+1] = %6.3f, "
	     "op[*sp] = '%c'\n", 
	     *sp, valstack[*sp], valstack[*sp+1], opstack[*sp]);
  return (retval);
}

//
//    crm_vht_lookup - given a char *start, long len, varnam 
//    finds and returns the vht index of the variable
//    or the index of the appropriate NULL slot to put
//    the var in, if not found.

long crm_vht_lookup (VHT_CELL **vht, char *vname, long vlen)
{
  long hc;
  long i, j, k;
  int done;
  long vsidx;
  long vslen;

  j = 0;  // just so J is used.

  crm_nextword ( vname, vlen, 0, &vsidx, &vslen);

  if (internal_trace)
    {
      fprintf (stderr, " variable len %ld, name is -", vslen);
      for (k = vsidx; k < vsidx+vslen; k++)
	fprintf (stderr, "%c", vname[k]);

      fprintf (stderr, "- .\n");
    };
 
  hc = (strnhash ( &vname[vsidx], vslen)) % vht_size;

  if (hc < 0) 
    hc = (hc + vht_size) % vht_size;

  //  go exploring - find either an empty cell (meaning that this
  //  is the first time this variable name has been entered into the
  //  vht) or find the variable already entered.  Or find that we've
  //  gone the whole way 'round the vht, in which case the vht is full
  //  and we should print ut a message and fatal error away (or maybe 
  //  even build a bigger vht?)

  i = hc;

  //   consider a "wrap" to have occurred if we even think about 
  //   the slot just before the hashcoded slot

  done = 0;
  while ( ! done )
    {
      //  is there anything here yet?
      if (vht[i] == NULL)
	{
	  if (internal_trace) 
	    fprintf (stderr, "var search at %ld but it's not here yet\n", i);
	  return (i);
	};
      
      //  there's something here - is it what we have been seeking

      //      if (strncmp (&((vht[i]->nametxt)[vht[i]->nstart]),
      if (memcmp  (&((vht[i]->nametxt)[vht[i]->nstart]),
		   vname,
		   vlen) == 0)
	{    //  Yes, we found it.
	  if (internal_trace)
	    fprintf (stderr, "  var found at %ld\n", i);
	  return (i);
	};

      i++;
      //  check wraparound
      if (i >= vht_size)
	i = 0;
      
      //   check for hash table full - if it is, right now we 
      //   do a fatal error.  Eventually we should just resize the 
	  //   hash table.  Even better- we should keep track of the number
      //   of variables, and thereby resize automatically whenever we 
      //   get close to overflow.
      if (i == (hc - 1))
	{
	  fatalerror (" hash table overflow while looking for variable: " ,
		      "");
	  done = 1;
	  return (0);
	};
    };
  return (0);
}
  
  
//
//    crm_setvar - set the value of a variable into the VHT
//
void crm_setvar (   
		 char *filename,
		 int filedesc,
		 char *nametxt, 
		 long nstart,   
		 long nlen,     
		 char *valtxt,  
		 long vstart,   
		 long vlen,     
		 long linenumber
		 )
{
  int i, j;     // some indices to bang on
  
  //  first off, see if the variable is already stored.

  i = crm_vht_lookup (vht, &(nametxt[nstart]), nlen);


  if (vht[i] == NULL)
    {
      //    Nope, this is an empty VHT slot
      
      //  allocate a fresh, empty VHT cell
      vht[i] = (VHT_CELL *) malloc (sizeof (VHT_CELL));
      if (!vht[i])
	untrappableerror("Couldn't malloc space for VHT cell.\nWe need VHT cells for variables.  We can't continue.","");
      
      //  fill in the name info data
      vht[i]->filename = filename;
      vht[i]->filedesc = filedesc;
      vht[i]->nametxt = nametxt;
      vht[i]->nstart = nstart;
      vht[i]->nlen = nlen;
      //  and now that the slot has proper initial information,
      //  we can use the same code as is used in an update to do 
      //  the initial setting of values.  This is good because 
      //  if we someday change the way variable values are stored,
      //  we need change it only in one place.
    }
  
  //   Either way, the cell is now here, so we can set the value.
  //
  vht[i]->valtxt = valtxt;
  vht[i]->vstart = vstart;
  vht[i]->vlen = vlen;
  vht[i]->mstart = vstart;
  vht[i]->mlen = 0;
  vht[i]->linenumber = linenumber;

  if(internal_trace)
    {
      fprintf (stderr, "  Successful set value of ");
      for (j = 0; j < vht[i]->nlen; j++)
	fprintf (stderr, "%c", vht[i]->nametxt[vht[i]->nstart+j]);

      fprintf (stderr, " at vht entry %d ", i);
      
      fprintf (stderr, " with value -");
      for (j = 0; j < vht[i]->vlen; j++)
	fprintf (stderr, "%c", vht[i]->valtxt[vht[i]->vstart+j]);
      fprintf (stderr, "- (length %ld)", vht[i]->vlen); 

      fprintf (stderr, "\n");
    };
  
}

//  look up what the line number is of a variable.
//
long crm_lookupvarline (VHT_CELL **vht, char *text, long start, long len)
{
  int i;     // some indices to bang on
  char *deathfu ;
  
  i = crm_vht_lookup (vht, &(text[start]), len);


  //    GROT GROT GROT
  //    We should check here for GOTOing a label that isn't in
  //    the current file (i.e. the equivalent of a C "longjmp").
  if (vht[i] != NULL)
	{           // Yes, we found it.  Return the line number
	  if (internal_trace) 
	    fprintf (stderr, "  looked up ... line number %ld\n", 
		     vht[i]->linenumber);
	  return (vht[i]->linenumber);
	}
  else
    {
      long q;
      deathfu = (char *) malloc ( len+10);
      if (!deathfu)
	untrappableerror("Couldn't malloc 'deathfu'.\n  Time to die. ","");
      strncpy (deathfu, &(csl->filetext[start]), len);
      q = fatalerror ("GOTO a non-existent variable", deathfu);
      //  If fatalerror found a TRAP for this error, cstmt now points to 
      //  the TRAP - 1.  We want to go to the trap itself, no auto-incr...
      if ( q == 0) 
	return ( csl->cstmt + 1);
    };
  return (0);
}

//      Update the start and length of all captured variables whenever
//      the input buffer gets mangled.  Mangles are all expressed in
//      the form of a start point and a delta.  Yes, this is a mangle,
//      as occasionally it's counterintuitive, but all other alterntives 
//      exhibit uncomfortable hidden edge conditions.
//
//     Note to the Reader - yes, I consider the nonlinearity of this 
//     function to be a grossitude.  Not quite an obscenity, but definitely
//     a wart.

void crm_updatecaptures (char *text, long loc, long delta)
{
  int vht_index;
  long istart;
  long iend;

  if (internal_trace)
    fprintf (stderr, "\n    updating captured values start %ld len %ld\n",
	     loc, delta);

  //        check each VHT entry for a need to relocate
  for (vht_index = 0; vht_index < vht_size; vht_index++)
    {
      //      is this an actual entry?
      if (vht[vht_index] != NULL)
	{
	  if (vht[vht_index]->valtxt == text)
	    {
	      if (internal_trace > 1)
		{
		  int i;
		  fprintf (stderr, "\n  checking var ");
		  for (i = 0; i < vht[vht_index]->nlen; i++)
		    fprintf (stderr, "%c",
			vht[vht_index]->nametxt[vht[vht_index]->nstart+i]);
		  fprintf (stderr, " ");
		  fprintf (stderr, " s: %ld, l:%ld, e:%ld ...",
			   vht[vht_index]->vstart,
			   vht[vht_index]->vlen,
			   vht[vht_index]->vstart+vht[vht_index]->vlen);
		};
	      istart = vht[vht_index]->vstart;
	      iend = istart + vht[vht_index]->vlen;
	      //      is this an entry in this particular text buffer?
	      //        now branch on the three possibilities:
	      //        1) the var is entirely before the mangle;
	      //            make no change.
	      //        2) the var bridges the mangle:
	      //            expand / contract the var
	      //        3) the var is after the mangle - move it, but 
	      //            the length doesn't change.
	      if (iend < loc)
		{
		  //   do nothing
		};
	      if (istart <= loc && iend >= loc )
		{
		  //    change length by delta
		  iend += delta;
		  if (iend < loc) iend = loc;
		  vht[vht_index]->vlen = iend - istart; 
		  if (internal_trace) 
		    fprintf (stderr, "vlen<-%ld...", iend-istart);
		};
	      if (istart > loc)
		{
		  //     move the variable
		  vht[vht_index]->vstart += delta;
		  if (internal_trace) 
		    fprintf (stderr, "vstart<-%ld...", vht[vht_index]->vstart);
		};
	      //   
	      //        And do the same for mstart/mlen (match start/length)
	      istart = vht[vht_index]->mstart;
	      iend = istart + vht[vht_index]->mlen;
	      if (iend < loc)
		{
		  //   do nothing
		};
	      if (istart <= loc && iend >= loc )
		{
		  //    change length by delta
		  iend += delta;
		  if (iend < loc) iend = loc;
		  vht[vht_index]->mlen = iend - istart; 
		  if (internal_trace) 
		    fprintf (stderr, "mlen...");
		};
	      if (istart > loc)
		{
		  //     move the variable
		  vht[vht_index]->mstart += delta;
		  if (internal_trace) 
		    fprintf (stderr, "mstart...");
		};
	    }
	  //      Don't forget entries that may be varNAMES, not just
	  //      var values!
	  if (vht[vht_index]->nametxt == text)
	    {
	      istart = vht[vht_index]->nstart;
	      iend = istart + vht[vht_index]->nlen;
	      //        now branch on the three possibilities:
	      //        1) the var is entirely before the mangle;
	      //            make no change.
	      //        2) the var bridges the mangle:
	      //            expand / contract the var
	      //        3) the var is after the mangle - move it, but 
	      //            the length doesn't change.
	      if (iend < loc)
		{
		  //   do nothing
		};
	      if (istart <= loc && iend >= loc )
		{
		  //    change length by delta
		  iend += delta;
		  if (iend < loc) iend = loc;
		  vht[vht_index]->nlen = iend - istart; 
		  if (internal_trace) 
		    fprintf (stderr, "nlen...");
		};
	      if (istart > loc)
		{
		  //     move the variable
		  vht[vht_index]->nstart += delta;
		  if (internal_trace) 
		    fprintf (stderr, "nstart.");
		};
	    }
	}
    }
      if (internal_trace)
	fprintf (stderr, "\n    end of updates\n");
}


//    The magic flag parser.  Given a string of input, return the
//    codes that were found as the (long int) return value.  If an
//    unrecognized code is found, squalk an error (whether it is fatal
//    or not is another issue)
//
//    Note that since flags (like variables) are always ASCII, we don't
//    need to worry about 8-bit-safety.
//      
long crm_flagparse (char *input, long inlen)            //  the user input
{
  char flagtext [MAX_PATTERN];
  char *remtext;
  long remlen;
  char *wtext;
  long flagsearch_start_here;
  long wstart;
  long wlen;
  long outcode;

  int done;
  int i;
  int j;
  int k;
  int recog_flag;
  char *flagstrings[32] = {
    "fromstart",   // 0
    "fromnext",    // 1
    "fromend",     // 2
    "newend",      // 3
    "fromcurrent", // 4
    "nocase",      // 5
    "absent",      // 6
    "basic",       // 7
    "nomultiline", // 8
    "backwards",   // 9
    "bychar",      // 10
    "byline",      // 11
    "byeof",       // 12
    "eofend",      // 13
    "keep",        // 14
    "async",       // 15
    "refute",      // 16
    "microgroom",  // 17
    "logboost",    // 18 
    "append",      // 19 
    "literal",     // 20
    "",
    "",
    ""
  };
 
  outcode = 0;

  memmove (flagtext, input, inlen);
  flagtext[inlen] = '\000';

  if (internal_trace)
    fprintf (stderr, "Flag string: %s\n", flagtext);

  //  now loop on thru the nextwords,
  remtext = flagtext;
  done = 0;
  remlen = inlen;
  wstart = 0;
  wlen = 0;
  flagsearch_start_here = 0;
  while (!done && remlen > 0)
    {
      i=crm_nextword (remtext, remlen, flagsearch_start_here, &wstart, &wlen);
      flagsearch_start_here = wstart + wlen + 1; 
      if (wlen > 0)
	{
	  //    We got a word, so aim wtext at it
	  wtext = &(remtext[wstart]);
	  if (internal_trace)
	    {
	      fprintf (stderr, "found flag, len %ld: ", wlen) ;
	      for (j = 0; j < wlen; j++) fprintf (stderr, "%c", wtext[j]);
	      fprintf (stderr, "\n");
	    };
	  
	  //    find sch in our table, squalk a nonfatal/fatal if necessary.
	  recog_flag = 0;
	  for (j = 0; j <= CRM_MAXFLAGS; j++)
	    {
	      k = strlen (flagstrings[j]);
	      //fprintf (stderr, " Trying %s\n", flagstrings[j]);
	      if (0 == strncasecmp (wtext, flagstrings[j], k))
		{
		  //    mark this flag as valid so we don't squalk an error
		  recog_flag = 1;
		  //     and OR this into our outcode
		  outcode = outcode | (0x00000001 << j);
		  if (user_trace) 
		    {
		      fprintf (stderr, "Mode #%d, '%s' turned on. \n",
			       j,
			       flagstrings[j]);
		    };
		};
	    };
	  
	  //   check to see if we need to squalk an error condition
	  if (recog_flag == 0)
	    {
	      long q;
	      char foo[1024];
	      strncpy (foo, wtext, 128);
	      foo[wlen] = '\000';
	      q = nonfatalerror ("Darn...  unrecognized flag :", foo);
	    };
	  
	  
	  //  and finally,  move sch up to point at the remaining string
	  if (remlen <= 0) done = 1;
	}
      else
	done = 1;
    };
  
  if (internal_trace )
    fprintf (stderr, "Flag code is : %ld\n", outcode);
  
  return (outcode);
}

//     Get the next word in a string.  "word" is defined by the 
//     continuous span of characters that are above ascii ! (> hex 0x20
//
//     The search starts at the "start" position given; the start position
//     is updated on each call and so is mutilated.  To step through a
//     arglist, you must add the returned value of "len" to the returned
//     value of start!
//     
//     The returned value is 0/1 as to whether we found
//     a valid word, and *start and *length, which give it's position.
//
long crm_nextword ( char *input,
		    long inlen,
		    long starthere,
		    long *start,
		    long *len)
{	
  *start = starthere;
  *len = 0;
  //   find start of string (if it exists)
  while (*start < inlen && input [*start] <= 0x20 ) *start = *start + 1;

  //  check - did we hit the end and still be invalid?  If so, return 0
  if (*start == inlen) return (0);

  //    if we get to here, then we have a valid string.
  *len = 0;
  while ((*start+*len) <= inlen 
	 && input [*start+*len] > 0x20 ) *len = *len + 1;
  
  return (*len);
}



int crm_expr_eval (CSL_CELL *csl, ARGPARSE_BLOCK *apb)
{
  //      Here we evaluate the slash-string _twice_, not just
  //      once as in ALTER.  This is how you can do 
  //      indirections (you may need to wrap this in a 
  //      LIAF loop to do infinite indirection
  //      Syntax is "eval <flags> (var) /newvalue/

	char varname[MAX_VARNAME];
	long varnamelen;
	long newvallen;
	unsigned long ihash;
	unsigned long ahash [MAX_EVAL_STATES_LOOKBACK];
	long ahindex;
	long itercount;
	long loop_abort;
	long qex_stat;
	// should use tempbuf for this instead.
	//   char newstr [MAX_PATTERN];
	if (user_trace)
	  fprintf (stderr, "Executing an EVALuation\n");
	
	qex_stat = 0;
	//     get the variable name
       	crm_get_pgm_arg (varname, MAX_VARNAME, apb->p1start, apb->p1len);
	if (apb->p1len < 2) 
	  {
	    nonfatalerror (
			   "This statement is missing the variable to alter,\n",
			   "so I'll ignore the whole statement.");
	    return (0);
	  };
	
	//      do variable substitution on the variable name
	varnamelen = crm_nexpandvar (varname, apb->p1len, MAX_VARNAME);
	if (varnamelen < 2) 
	  {
	    nonfatalerror (
			   "The variable you're asking me to alter has an utterly bogus name\n",
			   "so I'll ignore the whole statement.");
	    return (0);
	  };
	
	//     get the new pattern, and expand it.
	crm_get_pgm_arg (tempbuf, data_window_size, apb->s1start, apb->s1len);
	
	ihash = 0;
	itercount = 0;
	ahindex = 0;
	ahash[ahindex] = 0;
	loop_abort = 0;
	//     and a loop - while it continues to change, keep looping.
	newvallen = apb->s1len;
        while (ihash != strnhash (tempbuf, newvallen)
	       && itercount < MAX_EVAL_ITERATIONS 
	       && ! (loop_abort))
	  {
	    int i;
	    itercount++;
	    ihash = strnhash (tempbuf, newvallen);
	    for (i = 0; i < MAX_EVAL_STATES_LOOKBACK; i++)
	      if (ahash[i] == ihash)
		loop_abort = 1;
	    ahash[ahindex] = ihash;
	    ahindex++;
	    if (ahindex >= MAX_EVAL_STATES_LOOKBACK )
	      ahindex = 0;
	    newvallen = crm_qexpandvar (tempbuf, newvallen, 
					data_window_size, &qex_stat );
	  };
	
	if (itercount == MAX_EVAL_ITERATIONS )
	  {
	    nonfatalerror ("The variable you're attempting to EVAL seems to loop infinitely, and hence I cannot compute it.  I did try a lot, though.  Your variable was named ", varname);
	    return (0);
	  }
	if (loop_abort == 1)
	  {
	    nonfatalerror ("The variable you're attempting to EVAL seemed to get the same value after iterating, so it is probably an infinite loop.  I think I should give up.  Your variable was named ", varname);
	    return (0);
	  };
	
	//     and shove it out to wherever it needs to be shoved.
	//
		crm_destructive_alter_nvariable (varname, varnamelen, 
				 tempbuf, newvallen);
	
	if (internal_trace)
	  fprintf (stderr, "Final qex_stat was %ld\n", qex_stat);

	//    for now, use the qex_stat that came back from qexpandvar.
	if (qex_stat > 0)
	  {
	    if (user_trace)
	      fprintf (stderr, "Mathematical expression at line was not satisfied, doing a FAIL at line %ld\n", csl->cstmt);
	    csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
	    csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = -1;
	  }
	return (0);  
}

int crm_expr_alter (CSL_CELL *csl, ARGPARSE_BLOCK *apb)
{
  //      here's where we surgiclly alter a variable.  We have to
  //      watch out in case a variable is not in the cdw (it might
  //      be in tdw; that's legal as well.
  //      syntax is to replace the contents of the variable in the
  //      varlist with the evaluated string.
  //      Syntax is "alter <flags> (var) /newvalue/
  
	char varname[MAX_VARNAME];
	long varnamestart;
	long varnamelen;
	long newvallen;
	// should use tempbuf for this instead.
	//   char newstr [MAX_PATTERN];
	if (user_trace)
	  fprintf (stderr, "Executing an ALTERation\n");
	
	//     get the variable name
       	crm_get_pgm_arg (varname, MAX_VARNAME, apb->p1start, apb->p1len);
	if (apb->p1len < 2) 
	  {
	    nonfatalerror (
		     "This statement is missing the variable to alter,\n",
			   "so I'll ignore the whole statement.");
	    return (0);
	  };
	
	//      do variable substitution on the variable name
	varnamelen = crm_nexpandvar (varname, apb->p1len, MAX_VARNAME);
	crm_nextword (varname, varnamelen, 0, &varnamestart, &varnamelen);
	if (varnamelen - varnamestart < 2) 
	  {
	    nonfatalerror (
	  "The variable you're asking me to alter has an utterly bogus name\n",
		"so I'll ignore the whole statement.");
	    return (0);
	  };

	//     get the new pattern, and expand it.
	crm_get_pgm_arg (tempbuf, data_window_size, apb->s1start, apb->s1len);
	newvallen = crm_nexpandvar (tempbuf, apb->s1len, data_window_size);
	
	crm_destructive_alter_nvariable (&varname[varnamestart], varnamelen, 
					tempbuf, newvallen);
	return (0);
};

//        And the match routine.  What a humungous mess...
int crm_expr_match (CSL_CELL *csl, ARGPARSE_BLOCK *apb)
{
  long i; 
  long j;
  long k;
  char pch[MAX_PATTERN];
  long pchlen;
  char auxstr[MAX_PATTERN];
  long auxstr_len;
  regex_t preg;
  int casep, nomultilinep, absentp, fromp, extended_regex_p, literal_pattern_p;
  int cflags, eflags;
  char *mtext;
  long mtextlen;
  long textoffset;
  char bindable_vars[MAX_PATTERN]; 
  long bindable_vars_len;
  char svname[MAX_PATTERN];
  long svnamelen;
  regmatch_t matches [MAX_SUBREGEX];
  long nmatches;
  long vtextoffset, vtextend;
  long vpmstart, vpmend;
  long vmidx;
  //    And it all comes down to this, right here.  Matching a regex.
  //    This is the cruxpoint of the whole system.  We parse the
  //    program line args, get the flags out of the <> brackets, get the
  //    bound values out of the () parens, and the regex pattern out
  //    of the // delimiters.  Then we regcomp the pattern, and apply
  //    it to the data window (or to the variable if one is supplied).
  //    Then we either pass thru or fail to the failpoint.
  //
  //    get the flags out of the <> brackets
  //        cflags = REG_EXTENDED + REG_ICASE + REG_NEWLINE;
  //
  //  Translate to the cflags REG_EXTENDED, REG_ICASE, REG_NEWLINE
  //  (newline doesn't match wldcrd) and the eflags REG_NOTBOL (no
  //  newline at start) REG_NOTEOL (no newline at end)

  casep = 0;
  nomultilinep = 0;
  absentp = 1;
  fromp = 0;
  extended_regex_p = 1;
  literal_pattern_p = 0;

  //       Go through the flags
  //      is the ignore case flag set?
  if (apb->sflags & CRM_NOCASE)
    {
      if (user_trace)
	fprintf (stderr, "  nocase turned on...\n");
      casep = 1;
    };
  //      is the "basic regex" (obsolete, but useful) flag set?
  if (apb->sflags & CRM_BASIC)
    {
      if (user_trace)
	fprintf (stderr, "  basic regex match turned on...\n");
      extended_regex_p = 0;
    };

  if (apb->sflags & CRM_NOMULTILINE)
    {
      if (user_trace)
	fprintf (stderr, "  nomultiline turned on...\n");
      nomultilinep = 1;
    };
  
  if (apb->sflags & CRM_ABSENT)
    {
      if (user_trace)
	fprintf (stderr, "  absent flag turned on...\n");
      absentp = 0;
    };

  if (apb->sflags & CRM_LITERAL)
    {
      if (user_trace)
	fprintf (stderr, "  literal pattern search turned on...\n");
      literal_pattern_p = 1;
    };

  
  //  default is NO special fromming...
  fromp = 0;

  if (apb->sflags & CRM_FROMSTART)
    {
      if (user_trace)
	fprintf (stderr, "  fromstart turned on...\n");
      fromp = CRM_FROMSTART;
    };

  if (apb->sflags & CRM_FROMNEXT)
    {
      if (user_trace)
	fprintf (stderr, "  fromnext turned on...\n");
      fromp = CRM_FROMNEXT;
    };
  
  if (apb->sflags & CRM_FROMEND)
    {
      if (user_trace)
	fprintf (stderr, "  fromend turned on...\n");
      fromp = CRM_FROMEND;
    };
  
  if (apb->sflags & CRM_NEWEND)
    {
      if (user_trace)
	fprintf (stderr, "  newend turned on...\n");
      fromp = CRM_NEWEND;
    };

  if (apb->sflags & CRM_BACKWARDS)
    {
      if (user_trace)
	fprintf (stderr, "  backwards search turned on...\n");
      fromp = CRM_BACKWARDS;
    };
  
  if (apb->sflags & CRM_FROMCURRENT)
    {
      if (user_trace)
	fprintf (stderr, "  from-current search turned on...\n");
      fromp = CRM_FROMCURRENT;
    };
  
  //   Now, from the flags, calculate the cflags and eflags
  cflags = casep * REG_ICASE 
    + nomultilinep * REG_NEWLINE 
    + extended_regex_p * REG_EXTENDED
    + literal_pattern_p * REG_LITERAL;
  
  eflags = 0; //  
  
  //    get the bound values out of the () parenthesis
  //
  //   DANGER WILL ROBINSON!!  TAKE COVER, DOCTOR SMITH!!!  We
  //   have to be really careful here, because we need durable
  //   variable names to reference the VHT to, and an
  //   expandvar'ed variable doesn;t have that durable text
  //   string somewhere.  So, we have to stuff the variable name
  //   in as a temp var and then immediately reassign it.
  //    
  crm_get_pgm_arg (bindable_vars, MAX_PATTERN, apb->p1start, apb->p1len);
  if (internal_trace)
    fprintf (stderr, " bindable vars: ***%s***\n", bindable_vars);
  bindable_vars_len = crm_nexpandvar (bindable_vars, apb->p1len, MAX_PATTERN);
  
  
  //     here's where we look for a sub-match - specifically,
  //     to match inside a previously bound variable.
  crm_get_pgm_arg (svname, MAX_PATTERN, apb->b1start, apb->b1len);
  svnamelen = crm_nexpandvar (svname, apb->b1len, MAX_PATTERN);
  //    if no svname, then we're defaulted to :_dw:
  if (strlen (svname) == 0)
    {
      strcat (svname, ":_dw:");
      svnamelen = strlen (svname);
    };
  if (internal_trace)
    fprintf (stderr, " search-in-var: ***%s***\n", svname);
  
  //    get the regex pattern out of the // slashes
  crm_get_pgm_arg (pch, MAX_PATTERN, apb->s1start, apb->s1len);
  if (internal_trace)
    fprintf (stderr, " match pattern: =%s=\n", pch );
  pchlen = crm_nexpandvar (pch, apb->s1len, MAX_PATTERN);
  
  if (internal_trace)
    fprintf (stderr, " match pattern expands to =%s= len %ld flags %x %x \n",
	     pch, pchlen, cflags, eflags);
  
  //    regcomp the pattern
  i = crm_regcomp (&preg, pch, pchlen, cflags);
  if ( i > 0)
    {
      crm_regerror ( i, &preg, tempbuf, data_window_size);
      nonfatalerror ("Regular Expression Compilation Problem:", tempbuf);
      goto nonfatal_route_outwards;
    };

  //    get the aux match string, if one exists.
  crm_get_pgm_arg (auxstr, MAX_PATTERN, apb->s2start, apb->s2len);
  if (internal_trace) 
    fprintf (stderr, " aux string pattern: =%s=\n", auxstr);
  auxstr_len = crm_nexpandvar (auxstr, apb->s2len, MAX_PATTERN);
  
  //     Get the string to be matched upon...
  vmidx = crm_vht_lookup (vht, svname, svnamelen);
  if (vht[vmidx] == NULL)
    {
      nonfatalerror (" Attempt to match inside nonexistent variable ( always fails!) ",
		     svname);
      goto nonfatal_route_outwards;
    }
  //    do a check- if the vht->valtxt points into the
  //    cdw, or the tdw?
  mdw = cdw;    //  assume cdw unless otherwise proven...
  if (vht[vmidx]->valtxt == tdw->filetext)
    mdw=tdw;
  //  sanity check - must be tdw or cdw for searching!
  if (vht[vmidx]->valtxt != tdw->filetext
      && vht[vmidx]->valtxt != cdw->filetext)
    {
      long q;
      q = fatalerror ("Bogus text block (neither cdw nor tdw) on var ",
		svname);
      if (q != 0)
	exit (EXIT_FAILURE);
    };
  vtextoffset = vht[vmidx]->vstart;
  vtextend = vtextoffset + vht[vmidx]->vlen;
  vpmstart = vht[vmidx]->mstart;
  vpmend =  vpmstart + vht[vmidx]->mlen; 
  
  //    set up the start/end of the text we're matching against
  
  //   default is CRM_FROMSTART 
  textoffset = vtextoffset;
  if (fromp == CRM_FROMSTART)
    {
      textoffset = vtextoffset;
    }
  if (fromp == CRM_FROMCURRENT)
    {    
      textoffset = vpmstart;
    };
  if (fromp == CRM_NEWEND)
    {
      textoffset = vpmstart;
    };
  if (fromp == CRM_FROMNEXT)
    {
      textoffset = vpmstart + 1 ;
    };
  if (fromp == CRM_FROMEND)
    {
      textoffset = vpmend ;
    };
  if (fromp == CRM_BACKWARDS)
    { 
      if ( vpmstart > 0) 
	{ textoffset = vpmstart - 1; } else {textoffset = vpmstart; };
    };

  mtextlen = vtextend - textoffset;

  if (internal_trace)
    {
      fprintf (stderr, "    start matchable zone: %ld, end %ld, length %ld\n",
	       textoffset, vtextend, vtextend - textoffset);
    };

  //       Here is the crux.  Do the REGEX match, maybe in a loop
  //       if we're iterating to find a result with a different end 
  //       point than previous matches.
  nmatches = MAX_SUBREGEX;
  switch (fromp)
    {
    case CRM_NEWEND:
      {
	long oldend;
	long done;
	oldend = vpmend;
	done = 0;
	//        loop until we either get a match that goes 
	//        past the previous match, or until we are 
	//        at the end of the matchable text.
	while (textoffset <= vtextend - 1 
	       && done == 0)
	  {
	    textoffset++;
	    mtextlen--;
	    mtext = &mdw->filetext[textoffset];
	    i = crm_regexec (&preg, mtext, mtextlen,
			     nmatches, matches, eflags, auxstr);
	    j = matches[0].rm_eo;
	    if (( (textoffset + j) > oldend) && (i == 0)) done = 1;
	  };
      };
      break;
    case CRM_BACKWARDS:
      {
	long oldstart;
	oldstart = vpmstart;
	i = -1;
	j = oldstart + 1;
	matches[0].rm_so = j;
	//        loop until we either get a match or until we have hit
	//        the start of this (possibly captured-variable) region.
	while (textoffset > vtextoffset  
	       && (i != 0 || j > oldstart)  )
	  {
	    textoffset--;
	    mtextlen++;
	    mtext = &mdw->filetext[textoffset];
	    i = crm_regexec (&preg, mtext, mtextlen,
			     nmatches, matches, eflags, auxstr);
	    j = matches[0].rm_so;
	  };
      };
    default:
      {
	mtext = &mdw->filetext[textoffset];
	i = crm_regexec ( &preg, mtext, mtextlen, 
			  nmatches, matches, eflags, auxstr);
      };
    };
  
  crm_regfree (&preg);
  
  //    and now we FAIL or not...


  if ((absentp == 1 && i != 0) || (absentp == 0 && i == 0))
    {
      if (user_trace) 
	fprintf (stderr, "Match failed, failing forward\n");
      csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
      csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = -1;
    }
  else
    {
      if (user_trace)
	fprintf (stderr, "Match succceeded.\n");
      //     if the match was succcessful, we may need to 
      //     bind some variables, so see if there's a () 
      //     (note that we cannot use the grab_delimited_string
      //     routine results here, because crm_setvar uses 
      //     indices into the program to define variable names,
      //     not char * strings.   We _could_ cheat and malloc
      //     the variable name (perhaps we should!) and then
      //     clean up when we're done, but this at least will
      //     work for now.
      
      
      //   CAREFUL HERE - we need to correct due to offsets
      //   from the front of the text
      
      
      //     set the "last match was here" data...
      vht[vmidx]-> mstart = matches[0].rm_so + textoffset;
      vht[vmidx]-> mlen = matches[0].rm_eo - matches[0].rm_so;
      
      if ( bindable_vars_len > 0)
	{
	  long vstart;
	  long vlen;
	  long vnext;
	  long mc;
	  long done;
	  
	  done = 0;          //  loop till we've captured all the vars
	  mc = 0;
	  vstart = 0;
	  while ( !done)
	    {
	      //    find the start of the variable
	      while (bindable_vars[vstart] > 0x0
		     && bindable_vars[vstart] < 0x021 
		     && bindable_vars[vstart] != ')' )
		vstart++;
	      if (bindable_vars[vstart] == ')'
		  || bindable_vars[vstart] == 0x0 )
		{
		  done = 1;
		}
	      else
		{
		  
		  //    Now, the next space or ) ends the variable
		  vlen = 0;
		  while (bindable_vars[vstart+vlen] >=0x21 
			 && bindable_vars[vstart+vlen] != ')' )
		    vlen++;
		  //    have the next variable name, put out debug info.
		  if (internal_trace)
		    {
		      fprintf (stderr, "variable -");
		      for (k = 0; k < vlen; k++)
			fprintf (stderr, "%c", bindable_vars[vstart+k]);
		      fprintf (stderr, "- will be assigned value ");
		      for (k = matches[mc].rm_so + textoffset; 
			   k < matches[mc].rm_eo + textoffset; 
			   k++)
			fprintf (stderr, "%c", mdw->filetext[k]);
		      fprintf (stderr, "\n");
		    };
		  vnext = vstart + vlen;
		  //  HERE'S THE DANGEROUS PART..  because varible
		  //  names have been expanded, we can't assume
		  //  that the variablename in the program text
		  //  will be usable.  So, we create the varname as a temp
		  //  var, and then can reassign it with impunity.
		  {
		    char *vn;
		    vn = (char *) malloc (vlen + 4);
		    if (!vn)
		      untrappableerror("Couldn't malloc vn.\n Can't fix that.","");
		    strncpy (vn, &(bindable_vars[vstart]), vlen);
		    vn[vlen] = '\000';
		    if (strcmp (vn, ":_dw:") != 0)
		      {
			crm_set_windowed_nvar (vn, 
					       vlen,
					       mdw->filetext, 
					       matches[mc].rm_so + textoffset,
					       matches[mc].rm_eo
					       -matches[mc].rm_so,
					       csl->cstmt);
		      }
		    else
		      {
			nonfatalerror ("This program tried to re-define the data window!  That's very deep and profound, but not acceptable.  ", 
				       "Therefore, I'm ignoring this re-definition.");
		      };
		    free (vn);
		  };
		  //    and move on to the next binding (if any)
		  vstart = vnext;
		  mc++;
		  if (mc >= MAX_SUBREGEX)
		    nonfatalerror (
			"Exceeded MAX_SUBREGEX limit-too many parens in match",
			" Looks like you blew the gaskets on 'er.\n"); 
		  
		};
	    };
	};
    };
  if (0)
    {
    nonfatal_route_outwards:
      if (user_trace)
	fprintf (stderr, "The MATCH FAULTed and we're taking the TRAP out");
    };
  return (0);
};  

int crm_expr_learn (CSL_CELL *csl, ARGPARSE_BLOCK *apb)
{
  //     learn the sparse spectrum of this input window as 
  //     belonging to a particular type.
  //     learn <flags> (classname) /word/
  //
  long i, j, k;
  char ptext[MAX_PATTERN];  //  the regex pattern
  long plen;
  char ltext[MAX_PATTERN];  //  the variable to learn
  long llen;
  char htext[MAX_PATTERN];  //  the hash name
  long hlen;
  long cflags, eflags;
  struct stat statbuf;      //  for statting the hash file
  int hfd;                  //  hashfile fd
  long hfsize;              //  size of the hash file
  FEATUREBUCKET_TYPE *hashes;  //  the text of the hash file
  unsigned long hashpipe[5];   //  longest association set in the hashing
  regex_t regcb;
  regmatch_t match[5];      //  we only care about the outermost match
  long textoffset;
  long textmaxoffset;
  long sense;
  long vhtindex;
  long microgroom;
  long logboost;
  long fev;

  if (internal_trace)
    fprintf (stderr, "executing a LEARN\n");
  
  
  //           extract the hash file name
  crm_get_pgm_arg (htext, MAX_PATTERN, apb->p1start, apb->p1len);
  hlen = apb->p1len;
  hlen = crm_nexpandvar (htext, hlen, MAX_PATTERN);
  //
  //           extract the variable name (if present)
  crm_get_pgm_arg (ltext, MAX_PATTERN, apb->b1start, apb->b1len);
  llen = apb->b1len;
  llen = crm_nexpandvar (ltext, llen, MAX_PATTERN);
  
  //     get the "this is a word" regex
  crm_get_pgm_arg (ptext, MAX_PATTERN, apb->s1start, apb->s1len);
  plen = apb->s1len;
  plen = crm_nexpandvar (ptext, plen, MAX_PATTERN);
  
  //            set our cflags, if needed.  The defaults are
  //            "case" and "affirm", (both zero valued).
  //            and "microgroom" disabled.
  cflags = REG_EXTENDED;
  eflags = 0;
  sense = +1;
  if (apb->sflags & CRM_NOCASE)
    {
      cflags = cflags || REG_ICASE;
      eflags = 1;
      if (user_trace)
	fprintf (stderr, "turning oncase-insensitive match\n");
    };
  if (apb->sflags & CRM_REFUTE)
    {
      sense = -sense;
      if (user_trace)
	fprintf (stderr, " refuting learning\n");
    };
  microgroom = 0;
  if (apb->sflags & CRM_MICROGROOM)
    {
      microgroom = 1;
      if (user_trace)
	fprintf (stderr, " enabling microgrooming.\n");
    };
  logboost = 0;
  if (apb->sflags & CRM_LOGBOOST)
    {
      logboost = 1;
      if (user_trace)
	fprintf (stderr, " enabling LOGBOOST learning.\n");
    };

  //
  //             grab the filename, and stat the file
  //      note that neither "stat", "fopen", nor "open" are
  //      fully 8-bit or wchar clean...
  i = 0;
  while (htext[i] < 0x021) i++;
  j = i;
  while (htext[j] >= 0x021) j++;

  //             filename starts at i,  ends at j. null terminate it.
  htext[j] = '\000';

  //             and stat it to get it's length
  k = stat (&htext[i], &statbuf);

  //             quick check- does the file even exist?
  if (k != 0)
    {
      //      file didn't exist... create it
      FILE *f;
      if (user_trace)
	fprintf (stderr, "\nHad to create new .CSS file %s\n", &htext[i]);
      f = fopen (&htext[i], "w");
      if (!f)
	{
	  fprintf (stderr, 
		"\n Couldn't open your new file %s for writing; errno=%d .\n",
		 &htext[i], errno);
	  exit (EXIT_FAILURE);
        };
      //       put in sparse_spectrum_file_length entries of NULL
      for (j = 0; 
	   j < sparse_spectrum_file_length 
	     * sizeof ( FEATUREBUCKET_TYPE);
	   j++)
	fputc ('\000', f);
      //	fprintf (f,"%c", '\000');
      
      fclose (f);
      //    and reset the statbuf to be correct
      k = stat (&htext[i], &statbuf);
    };
  //    
  hfsize = statbuf.st_size;
  if (user_trace) 
    fprintf (stderr, "Sparse spectra file %s has length %ld bins\n",
	     &htext[i], hfsize / sizeof (FEATUREBUCKET_TYPE));

  //
  //         open the hash file into memory so we can bitwhack it
  //
  hfd = open (&(htext[i]), O_RDWR);
  if (hfd < 0) 
    {
      fev = fatalerror ("Couldn't open the .css file named: ",
			&htext[i]);
      return (fev);
    };

  //
  //      map the .css file into memory
  //
  hashes = (FEATUREBUCKET_TYPE *) mmap (NULL, hfsize, 
					PROT_READ + PROT_WRITE,
					MAP_SHARED, hfd, 0);
  if (hashes == MAP_FAILED)
    {
      fev = fatalerror ("Couldn't memory-map the .css file named: ",
			&htext[i]);
      return (fev);
    };

  //
  //        check the version of the file
  //
  if (hashes[0].hash != 0 ||
      hashes[0].key  != 0 ||
      hashes[0].value!= 0)
    {
      fev =fatalerror ("The .css file is the wrong version!  Filename is: ",
		       &htext[i]);
      return (fev);
    };
  //
  //   now set the hfsize to the number of entries, not the number
  //   of bytes total
  hfsize = hfsize / sizeof ( FEATUREBUCKET_TYPE );

  //    LOGBOOST IS DISABLED IN THIS RELEASE.  DO NOT ENABLE IT,
  //    AS IT GIVES _BAD_ RESULTS!!!!
  //  
  //    if logboost is engaged, get the log of the featurecount
  //
  if ( 0 ) // disabled  logboost)
    {
      for (i = 1; i < hfsize; i++)
	{
	  logboost = logboost + hashes[i].value;
	};
      logboost = log10 (log10 ((logboost * 1.0))) * FEATURE_HIT_INCREMENT_SIZE;
      if (logboost < 1) logboost = 1;
      logboost = 1;
    };

  //   and if logboost is engaged, mangle "sense" as well.
  //
  if (logboost)
    sense = sense * logboost ;

  //   compile the word regex
  //
  if ( internal_trace)
    fprintf (stderr, "\nWordmatch pattern is %s", ptext);
  i = crm_regcomp (&regcb, ptext, plen, cflags);
  if ( i > 0)
    {
      crm_regerror ( i, &regcb, tempbuf, data_window_size);
      nonfatalerror ("Regular Expression Compilation Problem:", tempbuf);
      goto regcomp_failed;
    };
  
  
  //   Start by priming the pipe... we will shift to the left next.
  //     sliding, hashing, xoring, moduloing, and incrmenting the
  //     hashes till there are no more.
  k = 0;
  j = 0;
  i = 0;
  if (llen > 0)
    {
      vhtindex = crm_vht_lookup (vht, ltext, llen);
    }
  else
    {
      vhtindex = crm_vht_lookup (vht, ":_dw:", 5);
    };
  
  if (vht[vhtindex] == NULL)
    {
      long q;
      q = fatalerror (" Attempt to LEARN from a nonexistent variable ",
		  ltext);
      return (q);
    };
  mdw = NULL;
  if (tdw->filetext == vht[vhtindex]->valtxt)
    mdw = tdw;
  if (cdw->filetext == vht[vhtindex]->valtxt)
    mdw = cdw;
  if (mdw == NULL)
    {
      long q;
      q = fatalerror (" Bogus text block containing variable ", ltext);  
      return (q);
    }
  textoffset = vht[vhtindex]->vstart;
  textmaxoffset = textoffset + vht[vhtindex]->vlen;
  
  //   init the hashpipe with 0xDEADBEEF 
  hashpipe [0] = hashpipe[1] = hashpipe[2] = hashpipe[3] = 
    hashpipe[4] = 0xDEADBEEF;
  
  //    and the big loop...
  i = 0;
  while (k == 0 && textoffset <= textmaxoffset)
    {
      long wlen;
      long slen;
      //  do the regex
      //  slen = endpoint (= start + len) 
      //        - startpoint (= curr textoffset)
      slen = vht[vhtindex]->vstart + vht[vhtindex]->vlen - textoffset ;
      k = crm_regexec ( &regcb, &(mdw->filetext[textoffset]),
			slen, 
			5, match, 0, NULL);
      
      if (k != 0 || textoffset > textmaxoffset)
	goto learn_end_regex_loop;
      
      {
	wlen = match[0].rm_eo - match[0].rm_so;
	memmove (tempbuf, 
		 &(mdw->filetext[textoffset + match[0].rm_so]),
		 wlen);
	tempbuf[wlen] = '\000';
	
	if (internal_trace)
	  {
	    fprintf (stderr, 
		     "  Learn #%ld t.o. %ld strt %ld end %ld len %ld is -%s-\n", 
		     i, 
		     textoffset,
		     (long) match[0].rm_so, 
		     (long) match[0].rm_eo,
		     wlen,
		     tempbuf);
	  };
	if (match[0].rm_eo == 0)
	  {
	    nonfatalerror ( "The LEARN pattern matched zero length! ",
			    "\n Forcing an increment to avoid an infinite loop.");
	    match[0].rm_eo = 1;
	  };
	hashpipe [4] = hashpipe[3];
	hashpipe [3] = hashpipe[2];
	hashpipe [2] = hashpipe[1];
	hashpipe [1] = hashpipe[0];
	
	//  and put new hash into pipeline
	hashpipe[0] = strnhash (tempbuf, wlen);
	
	if (internal_trace)
	  fprintf (stderr, "  Hashpipe contents: %lx %lx %lx %lx %lx \n",
		   hashpipe[4],
		   hashpipe[3],
		   hashpipe[2],
		   hashpipe[1],
		   hashpipe[0]);
	
	//  and account for the text used up.
	textoffset = textoffset + match[0].rm_eo;
	i++;

	//        is the pipe full enough to do the hashing?
	if (1)   //  we always run the hashpipe now.  (i >=5)
	  {
	    unsigned long hindex;
	    unsigned long h1, h2;
	    long th = 0;         // a counter used for TSS tokenizing
	    unsigned long incrs;
	    long j;
	    //
	    //     Hash polynomial: h0 + 3h1 + 5h2 +11h3 +23h4
	    //     (coefficients chosen by requiring superincreasing,
	    //     as well as prime)
	    //
	    th = 0;
	    for ( j = 0; j <= 15 ; j++)
	      {
#ifdef FOO
		//
		//   First Order Only - 1 token per.
		//
		hindex = hashpipe [0] ;
		if (hindex == 0) hindex = 1;
		h1 = hindex;
		hindex = hindex % hfsize;
		if (hindex == 0) hindex = 1;
		
		//   this is the secondary (crosscut) hash, used for
		//   confirmation of the key value.
		h2 =   hashpipe[0];
		if (h2 == 0) h2 = 0xdeadbeef;
		j = 16;
#endif
#ifdef TGB
		//
		//   Token Grab Bag - ignore sequence, distance. alias
		//
		hindex = hashpipe [0] 
		  + ( hashpipe[1] * ((j>>0) & 0x0001))
		  + ( hashpipe[2] * ((j>>1) & 0x0001))
		  + ( hashpipe[3] * ((j>>2) & 0x0001))
		  + ( hashpipe[4] * ((j>>3) & 0x0001));
		if (hindex == 0) hindex = 1;
		h1 = hindex;
		hindex = hindex % hfsize;
		if (hindex == 0) hindex = 1;
		
		//   this is the secondary (crosscut) hash, used for
		//   confirmation of the key value.
		h2 =   hashpipe[0] 
		  + (  hashpipe[1] * ((j>>0) & 0x0001))
		  + (  hashpipe[2] * ((j>>1) & 0x0001))
		  + (  hashpipe[3] * ((j>>2) & 0x0001))
		  + (  hashpipe[4] * ((j>>3) & 0x0001));
		if (h2 == 0) h2 = 0xdeadbeef;
#endif
#ifdef TGB2
		//
		//   Token Grab Bag - ignore sequence, distance.
		//
		hindex = hashpipe [0] 
		  + ( hashpipe[1] * ((j>>0) & 0x0001))
		  + ( hashpipe[2] * ((j>>1) & 0x0001))
		  + ( hashpipe[3] * ((j>>2) & 0x0001))
		  + ( hashpipe[4] * ((j>>3) & 0x0001));
		if (hindex == 0) hindex = 1;
		h1 = hindex;
		hindex = hindex % hfsize;
		if (hindex == 0) hindex = 1;
		
		//   this is the secondary (crosscut) hash, used for
		//   confirmation of the key value.
		h2 =   hashpipe[0] * hashpipe[0]
		  + (  hashpipe[1] * hashpipe[1] * ((j>>0) & 0x0001))
		  + (  hashpipe[2] * hashpipe[2] * ((j>>1) & 0x0001))
		  + (  hashpipe[3] * hashpipe[3] * ((j>>2) & 0x0001))
		  + (  hashpipe[4] * hashpipe[4] * ((j>>3) & 0x0001));
		if (h2 == 0) h2 = 0xdeadbeef;
#endif
#ifdef TSS
	      //
	      //   Token Grab Bag - ignore sequence, distance, prevent
	      //   aliasing (quadratic H2)
	      //
	      hindex = hashpipe [0] 
		+ ( hashpipe[1] * ((j>>0) & 0x0001))
		+ ( hashpipe[2] * ((j>>1) & 0x0001))
		+ ( hashpipe[3] * ((j>>2) & 0x0001))
		+ ( hashpipe[4] * ((j>>3) & 0x0001));
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      hindex = hindex % hfsize;
	      if (hindex == 0) hindex = 1;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      th = 2;
	      h2 =   hashpipe[0]; 
	      if ((j>>0) & 0x0001) 
		{
		  h2 = h2 + hashpipe[1] * th;
		  th++;
		};
	      if ((j>>1) & 0x0001) 
		{
		  h2 = h2 + hashpipe[2] * th;
		  th++;
		};
	      if ((j>>2) & 0x0001) 
		{
		  h2 = h2 + hashpipe[3] * th;
		  th++;
		};
	      if ((j>>3) & 0x0001) 
		{
		  h2 = h2 + hashpipe[4] * th;
		  th++;
		};
	      if (h2 == 0) h2 = 0xdeadbeef;
#endif

#ifdef SBPH
		hindex = hashpipe [0] 
		  + (  3 * hashpipe[1] * ((j>>0) & 0x0001))
		  + (  5 * hashpipe[2] * ((j>>1) & 0x0001))
		  + ( 11 * hashpipe[3] * ((j>>2) & 0x0001))
		  + ( 23 * hashpipe[4] * ((j>>3) & 0x0001));
		h1 = hindex;

		//   and what's our primary hash index?  Note that
		//   hindex = 0 is reserved for our version and 
		//   usage flags, so we autobump those to hindex=1
		hindex = hindex % hfsize;
		if (hindex == 0) hindex = 1;

		//   this is the secondary (crosscut) hash, used for
		//   confirmation of the key value.  Note that it shares
		//   no common coefficients with the previous hash.
		h2 =    7 * hashpipe [0] 
		  + (  13 * hashpipe[1] * ((j>>0) & 0x0001))
		  + (  29 * hashpipe[2] * ((j>>1) & 0x0001))
		  + (  51 * hashpipe[3] * ((j>>2) & 0x0001))
		  + ( 101 * hashpipe[4] * ((j>>3) & 0x0001));
		if (h2 == 0) h2 = 0xdeadbeef;
#endif

		//
		//   we now look at both the primary (h1) and 
		//   crosscut (h2) indexes to see if we've got
		//   the right bucket or if we need to look further
		//
		incrs = 0;
		while ( hashes[hindex].key != 0
			&&  ( hashes[hindex].hash != h1
			      || hashes[hindex].key  != h2 ))
		  {
		    //
		    incrs++;
		    //  
		    //       If microgrooming is enabled, and we've found a 
		    //       chain that's too long, we groom it down.
		    //
		    if (microgroom && (incrs > MICROGROOM_CHAIN_LENGTH))
		      {
			//     set the random number generator up...
			//     note that this is repeatable for a
			//     particular test set, yet dynamic.  That
			//     way, we don't always autogroom away the
			//     same feature; we depend on the previous
			//     feature's key.
			srand ( (unsigned int) h2);
			//
			//   and do the groom.
			crm_microgroom (hashes, hfsize, hindex);
			//  since things may have moved after a
			//  microgroom, restart our search
			hindex = h1 % hfsize;
			if (hindex == 0) hindex = 1;
			incrs = 0;
		      };
		    //      check to see if we've incremented ourself all the
		    //      way around the .css file.  If so, we're full, and
		    //      can hold no more features (this is unrecoverable)
		    if (incrs > hfsize - 3)
		      {
			nonfatalerror ("Your program is stuffing too many features into this size .css file.  Adding any more features is impossible in this file.",
				       "You are advised to build a larger .css file and merge your data into it.");
			goto learn_end_regex_loop;
		      };
		    hindex++;
		    if (hindex >= hfsize) hindex = 1;
		  };
		
		if (internal_trace)
		  {
		    if (hashes[hindex].value == 0)
		      {
			fprintf (stderr,"New feature at %ld\n", hindex);
		      }
		    else
		      {
			fprintf (stderr, "Old feature at %ld\n", hindex);
		      };
		  };
		//      always rewrite hash and key, as they may be incorrect
		//    (on a reused bucket) or zero (on a fresh one)
		//
		hashes[hindex].hash = h1;
		hashes[hindex].key  = h2;
		
		//      watch out - sense may be both + or -, so check before 
		//      adding it...
		//
		if (sense > 0 && 
		    hashes[hindex].value + sense >= FEATUREBUCKET_VALUE_MAX-1)
		  hashes[hindex].value = FEATUREBUCKET_VALUE_MAX - 1;
		else if ( sense < 0 && hashes[hindex].value <= -sense ) 
		  hashes[hindex].value = 0;
		else 
		  hashes[hindex].value += sense;
	      };
	  };
      };
    };   //   end the while k==0
 learn_end_regex_loop:
 regcomp_failed:
  //  and remember to let go of the mmap and the pattern bufffer
  munmap (hashes, hfsize);
  close (hfd);
  crm_regfree (&regcb);
  return (0);
}

//     How to microgroom a .css file that's getting full
//
//     NOTA BENE NOTA BENE NOTA BENE NOTA BENE
//      
//         This whole section of code is under intense develoment; right now
//         it "works" but not any better than nothing at all.  Be warned
//         that any patches issued on it may well never see the light of
//         day, as intense testing and comparison may show that the current
//         algorithms are, well, suckful.
//
//
//     There are two steps to microgrooming - first, since we know we're
//     already too full, we execute a 'zero unity bins'.  Then, we see
//     how the file looks, and if necessary, we do a "scale by R", where
//     R is the "MICROGROOM_RESCALE_FACTOR"
//
void crm_microgroom (FEATUREBUCKET_TYPE *h, long hs, long hindex)
{
  long i, j, k;
  static long microgroom_count = 0;
  long steps;
  long packstart;
  long packlen;
  //  for stochastic grooming we need a place for the random...
  unsigned long randy;
  long zeroed_countdown;
  j = 0;
  k = 0;
  zeroed_countdown = MICROGROOM_STOP_AFTER;

  //

  i = j = k = 0;
  microgroom_count++;

  if (user_trace)
    {
      if (microgroom_count == 1)
	fprintf (stderr, "CSS file too full: microgrooming this css chain: ");
      fprintf (stderr, " %ld ",
	   microgroom_count);
    };

  //   micropack - start at initial chain start, move to back of 
  //   chain that overflowed, then scale just that chain.
  i = hindex % hs;
  if (i == 0) i = 1;
  while (h[i].value != 0)
    {
      i--;
      if (i < 1) i = hs - 1;
      // fprintf (stderr, "-");
    }

  packstart = i;
  //     now, move our index to point to the first bucket in this chain.
  i++;
  if (i >= hs) i = 1;

  //   set our stochastic amnesia matcher - note that we add 
  //   our microgroom count so that we _eventually_ can knock out anything
  //   even if we get a whole string of buckets with hash keys that all alias
  //   to the same value.
  //
  //   We also keep track of how many buckets we've zeroed and we stop
  //   zeroing additional buckets after that point.
  //
  randy = rand() + microgroom_count;
  steps = 0;
  while (h[i].value != 0 && zeroed_countdown != 0 )
    {
      //      fprintf (stderr, "=");
      if ((( h[i].key + randy ) & MICROGROOM_STOCHASTIC_MASK ) 
	  == MICROGROOM_STOCHASTIC_KEY)
	h[i].value = h[i].value * MICROGROOM_RESCALE_FACTOR;
      if (h[i].value == 0) zeroed_countdown--;
      i++;
      if (i >= hs ) i = 1;
      steps++;
    }
  packlen = steps;

  //   now we pack the buckets
  crm_packcss (h, hs, packstart, packlen);

}

void crm_packcss (FEATUREBUCKET_TYPE *h, long hs, long packstart, long packlen)
{
  //    How we pack...
  //   
  //    We look at each bucket, and attempt to reinsert it at the "best"
  //    place.  We know at worst it will end up where it already is, and
  //    at best it will end up lower (at a lower index) in the file, except
  //    if it's in wraparound mode, in which case we know it will not get
  //    back up past us (since the file must contain at least one empty)
  //    and so it's still below us in the file.

  if (packstart+packlen < hs)   //  no wraparound in this case
    {
      crm_packseg (h, hs, packstart, packlen);
    }
  else    //  wraparound mode - do it as two separate repacks
    {
      crm_packseg (h, hs, packstart, (hs - (packstart + 1)));
      crm_packseg (h, hs, 1, (packlen - (hs - (packstart + 1))));
    };
}
      
void crm_packseg (FEATUREBUCKET_TYPE *h, long hs, long packstart, long packlen)
{
  unsigned long ifrom, ito;
  unsigned long thash, tkey, tvalue;

  if (internal_trace) fprintf (stderr, " < %ld %ld >", packstart, packlen);

  for (ifrom = packstart; ifrom < packstart + packlen; ifrom++)
    {
      //  Is it an empty bucket?  (remember, we're compressing out 
      //  all placeholder buckets, so any bucket that's zero-valued
      //  is a valid target.)
      if ( h[ifrom].value == 0)
	{
	  //    Empty bucket - turn it from marker to empty
	  h[ifrom].key = 0;
	  h[ifrom].hash = 0;
	}
    }

  //  Our slot values are now somewhat in disorder because empty
  //  buckets may now have been inserted into a chain where there used
  //  to be placeholder buckets.  We need to re-insert slot data in a
  //  bucket where it will be found.
  //
  for (ifrom = packstart; ifrom < packstart+packlen; ifrom++)
    {
      //    Now find the next bucket to place somewhere
      //
      thash  = h[ifrom].hash;
      tkey   = h[ifrom].key;
      tvalue = h[ifrom].value;

      if (tvalue != 0)
	{
	  ito = thash % hs;
	  if (ito == 0) ito = 1;
	  // fprintf (stderr, "a %ld", ito);

	  while ( ! (h[ito].value == 0 || 
		     (h[ito].hash == thash && h[ito].key == tkey )))
	    {
	      ito++;
	      if (ito >= hs) ito = 1;
	      //  fprintf (stderr, "A %ld", ito);
	    };
	  
	  //
	  //    found an empty slot, put this value there, and zero the
	  //    original one.  Sometimes this is a noop.  We don't care.

	  // fprintf (stderr, "b %ld", ito);
	  
	  h[ifrom].hash  = 0;
	  h[ifrom].key   = 0;
	  h[ifrom].value = 0;
	  
	  // fprintf (stderr, "c %ld ", ito );

	  h[ito].hash  = thash;
	  h[ito].key   = tkey;
	  h[ito].value = tvalue;
	  // fprintf (stderr, "d %ld", ito);
	};
    };
}


//      How to CLASSIFY some text.
//
int crm_expr_classify (CSL_CELL *csl, ARGPARSE_BLOCK *apb)
{
  //      classify the sparse spectrum of this input window 
  //      as belonging to a particular type.
  //
  //       This code should look very familiar- it's cribbed from
  //       the code for LEARN
  //
  long i, j, k;
  char ptext[MAX_PATTERN];  //  the regex pattern
  long plen;
  char ltext[MAX_PATTERN];  //  the variable to classify
  long llen;
  char htext[MAX_PATTERN];  //  the hash file names
  long hlen;
  char stext[MAX_PATTERN];  //  the match statistics variable inbuf
  long slen;
  char svrbl[MAX_PATTERN];  //  the match statistics text buffer
  long svlen;
  long fnameoffset;
  char fname[MAX_FILE_NAME_LEN];
  long eflags;
  long cflags;
  long vhtindex;

  struct stat statbuf;      //  for statting the hash file
  unsigned long hashpipe[5];  //  longest association set in the hashing
  regex_t regcb;
  regmatch_t match[5];      //  we only care about the outermost match

  unsigned long fcounts[MAX_CLASSIFIERS]; // total counts for feature normalize
  unsigned long totalcount = 0;

  double cpcorr[MAX_CLASSIFIERS];  // corpus correction factors
  double hits[MAX_CLASSIFIERS];  // actual hits per feature per classifier
  long totalhits[MAX_CLASSIFIERS];  // actual total hits per classifier
  long totalfeatures;   //  total features
  double htf;             // hits this feature got.
  double tprob;         //  total probability in the "success" domain.

  double textlen;    //  text length  - rougly corresponds to
                        //  information content of the text to classify
  
  double ptc[MAX_CLASSIFIERS]; // current running probability of this class
  double renorm = 0.0;
  double pltc[MAX_CLASSIFIERS]; // current local probability of this class
  double plltc[MAX_CLASSIFIERS]; // current local probability of this class
  
  int hfds[MAX_CLASSIFIERS];
  FEATUREBUCKET_TYPE *hashes[MAX_CLASSIFIERS];
  long hashlens[MAX_CLASSIFIERS];
  char *hashname[MAX_CLASSIFIERS];
  long succhash;
  long vbar_seen;	// did we see '|' in classify's args?
  long maxhash;
  long fnstart, fnlen;
  long fn_start_here;
  long textoffset;
  long textmaxoffset;
  long bestseen;
  long thistotal;
  long entropic_weight = 1;

  double top10scores[10];
  long top10polys[10];
  char top10texts[10][MAX_PATTERN];


  if (internal_trace)
    fprintf (stderr, "executing a CLASSIFY\n");
  
  //           extract the variable name (if present)
  //
  crm_get_pgm_arg (ltext, MAX_PATTERN, apb->b1start, apb->b1len);
  llen = apb->b1len;
  llen = crm_nexpandvar (ltext, llen, MAX_PATTERN);
  
  //           extract the hash file names
  crm_get_pgm_arg (htext, MAX_PATTERN, apb->p1start, apb->p1len);
  hlen = apb->p1len;
  hlen = crm_nexpandvar (htext, hlen, MAX_PATTERN);
  
  //           extract the "this is a word" regex
  //
  crm_get_pgm_arg (ptext, MAX_PATTERN, apb->s1start, apb->s1len);
  plen = apb->s1len;
  plen = crm_nexpandvar (ptext, plen, MAX_PATTERN);
  
  //            extract the optional "match statistics" variable
  //
  crm_get_pgm_arg (svrbl, MAX_PATTERN, apb->p2start, apb->p2len);
  svlen = apb->p2len;
  svlen = crm_nexpandvar (svrbl, svlen, MAX_PATTERN);
  { 
    long vstart, vlen;
    crm_nextword (svrbl, svlen, 0, &vstart, &vlen);
    memmove (svrbl, &svrbl[vstart], vlen);
    svlen = vlen;
    svrbl[vlen] = '\000';
  };
  
  //     status variable's text (used for output stats)
  //    
  stext[0] = '\000';
  slen = 0;
  
  //            set our flags, if needed.  The defaults are
  //            "case" 
  cflags = REG_EXTENDED;
  eflags = 0;

  if (apb->sflags & CRM_NOCASE)
    {
      cflags += REG_ICASE;
      eflags = 1;
    };

  //   compile the word regex
  if ( internal_trace)
    fprintf (stderr, "\nWordmatch pattern is %s", ptext);
  i = crm_regcomp (&regcb, ptext, plen, cflags);
  if ( i > 0)
    {
      crm_regerror ( i, &regcb, tempbuf, data_window_size);
      nonfatalerror ("Regular Expression Compilation Problem:", tempbuf);
      goto regcomp_failed;
    };
  

  
  //       Now, the loop to open the files.  
  bestseen = 0;
  thistotal = 0;
  //  goodcount = evilcount = 1;   // prevents a divide-by-zero error.
  //cpgood = cpevil = 0.0;
  //ghits = ehits = 0.0 ;
  //psucc = 0.5;
  //pfail = (1.0 - psucc);
  //pic = 0.5;
  //pnic = 0.5;
  

  //      initialize our arrays for N .css files
  for (i = 0; i < MAX_CLASSIFIERS; i++) 
    {
      fcounts[i] = 0;    // check later to prevent a divide-by-zero 
    			 // error on empty .css file
      cpcorr[i] = 0.0;   // corpus correction factors
      hits[i] = 0.0;     // absolute hit counts 
      totalhits[i] = 0.0;     // absolute hit counts 
      ptc[i] = 0.5;      // priori probability
      pltc[i] = 0.5;     // local probability
    };

  for (i = 0; i < 10; i++)
    {
      top10scores[i] = 0;
      top10polys[i] = 0;
      strcpy (top10texts[i], "");
    };
  //        --  probabilistic evaluator ---
  //     S = success; A = a testable attribute of success
  //     ns = not success, na = not attribute
  //     the chain rule we use is:
  //
  //                   P(A|S) P(S)
  //  P (S|A) =   -------------------------
  //             P(A|S) P(S) + P(A|NS) P(NS)
  //    
  //     and we apply it repeatedly to evaluate the final prob.  For
  //     the initial a-priori probability, we use 0.5.  The output
  //     value (here, P(S|A) ) becomes the new a-priori for the next
  //     iteration.
  //
  //     Extension - we generalize the above to I classes as and feature
  //      F as follows:
  //
  //                         P(F|Ci) P(Ci)
  //    P(Ci|F) = ----------------------------------------
  //              Sum over all classes Ci of P(F|Ci) P(Ci)
  //
  //     We also correct for the unequal corpus sizes by multiplying
  //     the probabilities by a renormalization factor.  if Tg is the
  //     total number of good features, and Te is the total number of
  //     evil features, and Rg and Re are the raw relative scores,
  //     then the corrected relative scores Cg aqnd Ce are
  //
  //     Cg = (Rg / Tg)
  //     Ce = (Re / Te)
  //    
  //     or  Ci = (Ri / Ti)
  //    
  //     Cg and Ce can now be used as "corrected" relative counts
  //     to calculate the naive Bayesian probabilities.
  //
  //     Lastly, the issue of "over-certainty" rears it's ugly head.
  //     This is what happens when there's a zero raw count of either
  //     good or evil features at a particular place in the file; the
  //     strict but naive mathematical interpretation of this is that
  //     "feature A never/always occurs when in good/evil, hence this
  //     is conclusive evidence of good/evil and the probabilities go
  //     to 1.0 or 0.0, and are stuck there forevermore.  We use the
  //     somewhat ad-hoc interpretation that it is unreasonable to
  //     assume that any finite number of samples can appropriately
  //     represent an infinite continuum of spewage, so we can bound
  //     the certainty of any meausre to be in the range:
  //   
  //        limit: [ 1/featurecount+2 , 1 - 1/featurecount+2].  
  //
  //     The prior bound is strictly made-up-on-the-spot and has NO
  //     strong theoretical basis.  It does have the nice behavior
  //     that for feature counts of 0 the probability is clipped to 
  //     [0.5, 0.5], for feature counts of 1 to [0.333, 0.666]
  //     for feature counts of 2 to [0.25, 0.75], for 3 to
  //     [0.2, 0.8], for 4 to [0.166, 0.833] and so on.
  // 
  
  vbar_seen = 0;
  maxhash = 0;
  succhash = 0;
  fnameoffset = 0;
  //    now, get the file names and mmap each file
  //     get the file name (grody and non-8-bit-safe, but doesn't matter
  //     because the result is used for open() and nothing else.
  //   GROT GROT GROT  this isn't NULL-clean on filenames.  But then
  //    again, stdio.h itself isn't NULL-clean on filenames.
  if (user_trace)
    fprintf (stderr, "Classify list: -%s- \n", htext);
  fn_start_here = 0;
  fnlen = 1;
  while ( fnlen > 0 && ((maxhash < MAX_CLASSIFIERS-1)))
    {
      crm_nextword (htext,
		    hlen, fn_start_here,
		    &fnstart, &fnlen); 
      if (fnlen > 0)
	{
	  strncpy (fname, &htext[fnstart], fnlen);
	  fn_start_here = fnstart + fnlen + 1;
	  fname[fnlen] = '\000';
	  if (user_trace)
	    fprintf (stderr, "Classifying with file -%s- "\
			     "succhash=%ld, maxhash=%ld\n",
			     fname, succhash, maxhash);
	  if ( fname[0] == '|' && fname[1] == '\000')
	    {
	      if (vbar_seen)
		{
		  nonfatalerror ("Only one ' | ' allowed in a CLASSIFY. \n" ,
				 "We'll ignore it for now.");
		}
	      else
		{
		  succhash = maxhash;
		};
	      vbar_seen ++;
	    }
	  else
	    {
	      //  be sure the file exists 
	      //             stat the file to get it's length
	      k = stat (fname, &statbuf);
	      //             quick check- does the file even exist?
	      if (k != 0)
		{
		  nonfatalerror ("Nonexistent Classify table named: ",
				 fname);
		}
	      else
		{
		  //  file exists - do the open/process/close      
		  //    
		  hashlens[maxhash] = statbuf.st_size;
		  //  mmap the hash file into memory so we can bitwhack it
		  hfds[maxhash] = open (fname, O_RDONLY);
		  if (hfds[maxhash] < 0) 
		    {
		      nonfatalerror ("Couldn't open the table file",
				     fname);
		    }
		  else
		    {
		      hashes[maxhash] = (FEATUREBUCKET_TYPE *) 
			mmap (NULL, 
			      hashlens[maxhash],
			      PROT_READ,
			      MAP_SHARED, 
			      hfds[maxhash], 
			      0);
		      if (hashes[maxhash] == MAP_FAILED )
			{
			  nonfatalerror ("Couldn't memory-map the table file",
					 fname);
			}
		      else
			{
			  //
			  //     Check to see if this file is the right version
			  //
			  long fev;
			  if (hashes[maxhash][0].hash != 0 ||
			      hashes[maxhash][0].key  != 0 ||
			      hashes[maxhash][0].value!= 0)
			    {
			      fev =fatalerror ("The .css file is the wrong version!  Filename is: ",
					       &htext[i]);
			      return (fev);
			    };
			  //  set this hashlens to the length in features instead
			  //  of the length in bytes.
			  hashlens[maxhash] = hashlens[maxhash] / sizeof (FEATUREBUCKET_TYPE);
			  hashname[maxhash] = (char *) malloc (fnlen+10);
			  if (!hashname[maxhash])
			    untrappableerror(
			      "Couldn't malloc hashname[maxhash]\n","We need that part later, so we're stuck.  Sorry.");
			  strncpy(hashname[maxhash],fname,fnlen);
			  hashname[maxhash][fnlen]='\000';
			  maxhash++;
			};
		    };
		};
	    };
	  if (maxhash > MAX_CLASSIFIERS-1)
	    nonfatalerror ("Too many classifier files.",
			   "Some may have been disregarded");
	};
    };

  //
  //    If there is no '|', then all files are "success" files.  
  if (succhash == 0) 
    succhash = maxhash;

  //    a CLASSIFY with no arguments is always a "success".
  if (maxhash == 0)
    return (0);

  //    now, set up the normalization factor fcount[]
  if (user_trace)
    fprintf (stderr, "Running with %ld files for success out of %ld files\n",
	     succhash, maxhash );
  // sanity checks...  Uncomment for super-strict CLASSIFY.
  // 
  //	do we have at least 1 valid .css files?
  if (maxhash == 0)
    {
      fatalerror ("Couldn't open at least 2 .css files for classify().", "");
    };
  //	do we have at least 1 valid .css file at both sides of '|'?
  //if (!vbar_seen || succhash < 0 || (maxhash < succhash + 2))
  //  {
  //    nonfatalerror (
  //      "Couldn't open at least 1 .css file per SUCC | FAIL classes "
  //	" for classify().\n","Hope you know what are you doing.");
  //  };


  {
    long ifile;
    long k;
    //      count up the total first
    for (ifile = 0; ifile < maxhash; ifile++)
      {
	fcounts[ifile] = 0;
	for (k = 1; k < hashlens[ifile]; k++)
	  fcounts [ifile] = fcounts[ifile] + hashes[ifile][k].value;
	if (fcounts[ifile] == 0) fcounts[ifile] = 1;
	totalcount = totalcount + fcounts[ifile];
      };
    //
    //     calculate cpcorr (count compensation correction)
    //

    for (ifile = 0; ifile < maxhash; ifile++)
      {
	//  cpcorr [ifile] = ( totalcount / (fcounts[ifile] * (maxhash-1)));
	//
	//   disable cpcorr for now... unclear that it's useful.
	cpcorr[ifile] = 1.0;
      };
  };
  
  //
  //   now all of the files are mmapped into memory,
  //   and we can do the polynomials and add up points.
  i = 0;
  j = 0;
  k = 0;
  thistotal = 0;

  if (llen > 0)
    {
      vhtindex = crm_vht_lookup (vht, ltext, llen );
    }
  else
    {
      vhtindex = crm_vht_lookup (vht, ":_dw:", 5);
    }
  if (vht[vhtindex] == NULL)
    {
      return (fatalerror (" Attempt to CLASSIFY from a nonexistent variable ",
			  ltext));
    };
  mdw = NULL;
  if (tdw->filetext == vht[vhtindex]->valtxt)
    mdw = tdw;
  if (cdw->filetext == vht[vhtindex]->valtxt)
    mdw = cdw;
  if (mdw == NULL)
    return ( fatalerror (" Bogus text block containing variable ", ltext));  
  textoffset = vht[vhtindex]->vstart;
  textmaxoffset = textoffset + vht[vhtindex]->vlen;

  textlen = (vht[vhtindex]->vlen);
  if (textlen < 1.0) textlen = 1.0; 

  //   init the hashpipe with 0xDEADBEEF 
  hashpipe [0] = hashpipe[1] = hashpipe[2] = hashpipe[3] 
    = hashpipe[4] = 0xDEADBEEF;


  totalfeatures = 0;
  
  //  stop when we no longer get any regex matches
  //   possible edge effect here- last character must be matchable, yet
  //    it's also the "end of buffer".
  while (k == 0 && textoffset <= textmaxoffset)
    {
      long wlen;
      long slen;
      //  do the regex  
      slen = vht[vhtindex]->vstart + vht[vhtindex]->vlen - textoffset ;
      k = crm_regexec (&regcb, &(mdw->filetext[textoffset]),
		       slen, 1, match, 0, NULL);
      if (k != 0 || textoffset > textmaxoffset)
	goto classify_end_regex_loop;
      
      wlen = match[0].rm_eo - match[0].rm_so;
      memmove (tempbuf, 
	       &(mdw->filetext[textoffset + match[0].rm_so]),
	       wlen);
      tempbuf[wlen] = '\000';
      
      if (match[0].rm_eo == 0)
	{
	  nonfatalerror("The CLASSIFY pattern matched zero length! ",
			"\n Forcing an increment to avoid an infinite loop.");
	  match[0].rm_eo = 1;
	};
      if (internal_trace)
	{		   
	  fprintf (stderr, 
		   "  Classify #%ld t.o. %ld strt %ld end %ld len %ld is -%s-\n", 
		   i, 
		   textoffset,
		   (long) match[0].rm_so, 
		   (long) match[0].rm_eo,
		   wlen,
		   tempbuf);
	  
	};
      if (match[0].rm_eo == 0)
	{
	  nonfatalerror ( "The CLASSIFYpattern matched zero length! ",
			  "\n Forcing an increment to avoid an infinite loop.");
	  match[0].rm_eo = 1;
	};
      //  slide previous hashes up 1
      hashpipe [4] = hashpipe[3];
      hashpipe [3] = hashpipe[2];
      hashpipe [2] = hashpipe[1];
      hashpipe [1] = hashpipe[0];
      
      //  and put new hash into pipeline
      hashpipe[0] = strnhash ( tempbuf, wlen);
      
      if (0) fprintf (stderr, "  Hashpipe: %lx %lx %lx %lx %lx \n",
		      hashpipe[0],
		      hashpipe[1],
		      hashpipe[2],
		      hashpipe[3],
		      hashpipe[4]);
      
      //   account for the text we used up...
      textoffset = textoffset + match[0].rm_eo;
      i++;
      
      //        is the pipe full enough to do the hashing?
      if (1)   //  we init with 0xDEADBEEF, so the pipe is always full (i >=5)
	{
	  int j, k, l;
	  unsigned th=0;          //  a counter used only in TSS hashing
	  unsigned long hindex;
	  unsigned long h1, h2;
	  //unsigned long good, evil;
	  //
	  //     Hash polynomial: h0 + 3h1 + 5h2 +9h3 +17h4
	  //     (coefficients chosen by formula of 1 + 2^n)
	  //
	  //    GROT GROT GROT  make the order of the SBPH
	  //    a compile-time parameter
	  th = 0;
	  for ( j = 0; j < 16 ; j++)
	    {

#ifdef FOO
	      //
	      //   First Order Only - only use 1 token
	      //
	      hindex = hashpipe [0] ;
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      h2 =   hashpipe[0] ;
	      if (h2 == 0) h2 = 0xdeadbeef;
	      j = 16;
#endif
#ifdef TGB
	      //
	      //   Token Grab Bag - ignore sequence, distance, allow
	      //   aliasing ( note that H2 is linear ).
	      //
	      hindex = hashpipe [0] 
		+ ( hashpipe[1] * ((j>>0) & 0x0001))
		+ ( hashpipe[2] * ((j>>1) & 0x0001))
		+ ( hashpipe[3] * ((j>>2) & 0x0001))
		+ ( hashpipe[4] * ((j>>3) & 0x0001));
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      h2 =   hashpipe[0] 
		+ (  hashpipe[1] * ((j>>0) & 0x0001))
		+ (  hashpipe[2] * ((j>>1) & 0x0001))
		+ (  hashpipe[3] * ((j>>2) & 0x0001))
		+ (  hashpipe[4] * ((j>>3) & 0x0001));
	      if (h2 == 0) h2 = 0xdeadbeef;
#endif
#ifdef TGB2
	      //
	      //   Token Grab Bag - ignore sequence, distance, prevent
	      //   aliasing (note that H2 is quadratic ).
	      //
	      hindex = hashpipe [0] 
		+ ( hashpipe[1] * ((j>>0) & 0x0001))
		+ ( hashpipe[2] * ((j>>1) & 0x0001))
		+ ( hashpipe[3] * ((j>>2) & 0x0001))
		+ ( hashpipe[4] * ((j>>3) & 0x0001));
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      h2 =   hashpipe[0] * hashpipe[0] 
		+ (  hashpipe[1] * hashpipe[1] * ((j>>0) & 0x0001))
		+ (  hashpipe[2] * hashpipe[2] * ((j>>1) & 0x0001))
		+ (  hashpipe[3] * hashpipe[3] * ((j>>2) & 0x0001))
		+ (  hashpipe[4] * hashpipe[4] * ((j>>3) & 0x0001));
	      if (h2 == 0) h2 = 0xdeadbeef;
#endif
#ifdef TSS
	      //
	      //   Token Seqence Sensitive - ignore distance, prevent
	      //   aliasing (quadratic H2)
	      //
	      hindex = hashpipe [0] 
		+ ( hashpipe[1] * ((j>>0) & 0x0001))
		+ ( hashpipe[2] * ((j>>1) & 0x0001))
		+ ( hashpipe[3] * ((j>>2) & 0x0001))
		+ ( hashpipe[4] * ((j>>3) & 0x0001));
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      th = 2;
	      h2 =   hashpipe[0]; 
	      if ((j>>0) & 0x0001) 
		{
		  h2 = h2 + hashpipe[1] * th;
		  th++;
		};
	      if ((j>>1) & 0x0001) 
		{
		  h2 = h2 + hashpipe[2] * th;
		  th++;
		};
	      if ((j>>2) & 0x0001) 
		{
		  h2 = h2 + hashpipe[3] * th;
		  th++;
		};
	      if ((j>>3) & 0x0001) 
		{
		  h2 = h2 + hashpipe[4] * th;
		  th++;
		};
#endif
#ifdef SBPH
	      hindex = hashpipe [0] 
		+ (  3 * hashpipe[1] * ((j>>0) & 0x0001))
		+ (  5 * hashpipe[2] * ((j>>1) & 0x0001))
		+ ( 11 * hashpipe[3] * ((j>>2) & 0x0001))
		+ ( 23 * hashpipe[4] * ((j>>3) & 0x0001));
	      if (hindex == 0) hindex = 1;
	      h1 = hindex;
	      
	      //   this is the secondary (crosscut) hash, used for
	      //   confirmation of the key value.
	      h2 =    7 * hashpipe [0] 
		+ (  13 * hashpipe[1] * ((j>>0) & 0x0001))
		+ (  29 * hashpipe[2] * ((j>>1) & 0x0001))
		+ (  51 * hashpipe[3] * ((j>>2) & 0x0001))
		+ ( 101 * hashpipe[4] * ((j>>3) & 0x0001));
	      if (h2 == 0) h2 = 0xdeadbeef;
#endif

	      //
	      //    Note - a strict interpretation of Bayesian
	      //    chain probabilities should use 0 as the initial
	      //    state.  However, because we rapidly run out of
	      //    significant digits, we use a much less strong
	      //    initial state.   Note also that any nonzero
	      //    positive value prevents divide-by-zero
	      //
//#define ENTROPIC_ENABLED
#ifdef ENTROPIC_ENABLED
	      //  
	      //   Calculate entropic weight of this feature.  
	      //   (because these are correllated features, this is
	      //   linear, not logarithmic)
	      //
	      //   These weights correspond to the number of elements
	      //   in the hashpipe that are used for this particular
	      //   calculation, == 1 + bitcount(Jval)
	      
	      {
		long ew[16] =  // Jval
		  { 1,         // 0
		    2,         // 1
		    2,         // 2
		    3,         // 3
		    2,         // 4
		    3,         // 5
		    3,         // 6
		    4,         // 7
		    2,         // 8
		    3,         // 9
		    3,         // 10 
		    4,         // 11
		    3,         // 12
		    4,         // 13
		    4,         // 14
		    5 };        // 15
		
		entropic_weight = ew[j];
	      };
#endif
	      //
	      //       Zero out "Hits This Feature"
	      htf = 0;
	      totalfeatures++;
	      //
	      for (k = 0; k < maxhash; k++)
		{
		  long lh, lh0;
		  lh = hindex % (hashlens[k]);
		  if (lh==0) lh=1;
		  lh0 = lh;
		  hits[k] = 0;
		  while ( hashes[k][lh].key != 0 
			  && ( hashes[k][lh].hash != h1    
			       || hashes[k][lh].key  != h2 ))
		    {
		      lh++;
		      if (lh >= hashlens[k]) lh = 1;
		      if (lh == lh0) break; // wraparound
		    };
		  if (hashes[k][lh].hash == h1 && hashes[k][lh].key == h2)
		    {
		      l = hashes[k][lh].value * entropic_weight;
		      totalhits[k] = totalhits[k] + l;  // remember totalhits
		      hits[k] = l * cpcorr [k];     	// remember corr. hits 
		      htf = htf + hits[k];            // and hits-this-feature
		    };
		};
	      

	      //      now update the probabilities.  
	      //      
	      //     NOTA BENE: there are a bunch of different ways to
	      //      calculate local probabilities.  The text below
	      //      refers to an experiment that may or may not make it
	      //      as the "best way".
	      //
	      //      The hard part is this - what is the local in-class
	      //      versus local out-of-class probability given the finding
	      //      of a particular feature?  
	      //
	      //      I'm guessing this- the validity is the differntial
	      //      seen on the feature (that is, fgood - fevil )
	      //      times the entropy of that feature as seen in the 
	      //      corpus (that is,
	      //
	      //              Pfeature*log2(Pfeature)
	      //              
	      //      = 
	      //        totalcount_this_feature
	      //            ---------------    * log2 (totalcount_this_feature)
	      //         totalcount_all_features
	      //    
	      //    (note, yes, the last term seems like it should be 
	      //    relative to totalcount_all_features, but a bit of algebra
	      //    will show that if you view fgood and fevil as two different
	      //    signals, then you end up with + and - totalcount inside
	      //    the logarithm parenthesis, and they cancel out.
	      //    (the 0.30102 converts "log10" to "log2" - it's not
	      //    a majic number, it's just that log2 isn't in glibc)
	      //

	      //  HACK ALERT- this code here is still under development
	      //  and should be viewed with the greatest possible 
	      //  suspicion.  :=)

#ifdef OLD_LOCAL_PROBABILITIES
	      //fgood = good;
	      //fevil = evil;
	      //pmag = (fgood - fevil) / ( 2 * (fgood + fevil + 1 )  );
	      //plnic = 0.5 - pmag;
	      //plic = 1.0 - plnic;
	      
	      for (k = 0; k < maxhash; k++)
		{
		  pltc[k] = 0.5 +
		    (( hits[k] - (htf - hits[k])) 
		       / (LOCAL_PROB_DENOM * (htf + 1.0)));
		};
#else
	      // fgood = good;
	      //fevil = evil;
	      //pmag = (fgood - fevil) / ( (fgood + fevil + 1) * textlen);
	      //plnic = 0.5 - pmag;
	      //plic = 1.0 - plnic;

	      for (k = 0; k < maxhash; k++)
		{
		  pltc[k] = 0.5 + 
		    (( hits[k] - (htf - hits[k])) / 
				    ((htf + 1)*textlen));
		};
#endif


	      //    Now, some funky magic.  Our formula above is
	      //    mathematically correct (if features were
	      //    independent- something we conveniently ignore.),
	      //    but because of the limited word length in a real
	      //    computer, we can quickly run out of dynamic range
	      //    early in a CLASSIFY (P(S) indistinguishable from
	      //    1.00) and then there is no way to recover.  To
	      //    remedy this, we use two alternate forms of the
	      //    formula (in Psucc and Pfail) and use whichever
	      //    form that yields the smaller probability to
	      //    recalculate the value of the larger.
	      //
	      //    The net result of this subterfuge is a nonuniform
	      //    representation of the probability, with a huge dynamic
	      //    range in two places - near 0.0, and near 1.0 , which
	      //    is where we actually care.
	      //
	      //      update the global probabilities
	      
	      //  ptemp = ( pic*plic)  / ((pic * plic) + (pnic * plnic));
	      //  pnic  = (pnic*plnic) / ((pic * plic) + (pnic * plnic));
	      //  pic   = ptemp;
	      
	      //   calculate renormalizer (the Bayesian formula's denomenator)
	      renorm = 0.0;
	      //   now calculate the per-ptc numerators
	      plltc[0] = 0;    // keep gcc from complaining about unused vars.
#ifdef USE_PEAK
	      for (k = 0; k < maxhash; k++)
		renorm = renorm + (ptc[k]*plltc[k]);

	      if ( j == 0)
		for (k = 0; k < maxhash; k++)
		  plltc[k] = pltc[k];
	      if ( j < 15)
		for (k = 0; k < maxhash; k++)
		  if (pltc[k] > plltc[k])
		    plltc[k] = pltc[k];
		     
	      if ( j == 15)
		for (k = 0; k < maxhash; k++)
		  ptc[k] = (ptc[k] * plltc[k]) / renorm;
#else
	      for (k = 0; k < maxhash; k++)
		renorm = renorm + (ptc[k]*pltc[k]);

	      for (k = 0; k < maxhash; k++)
		ptc[k] = (ptc[k] * pltc[k]) / renorm;
#endif

	      //   if we have underflow (any probability == 0.0 ) then
	      //   bump the probability back up to 10^-308, or
	      //   whatever a small multiple of the minimum double
	      //   precision value is on the current platform.
	      //
	      for (k = 0; k < maxhash; k++)
		if (ptc[k] < 10*DBL_MIN) ptc[k] = 10 * DBL_MIN;

	      //
	      //      part 2) renormalize to sum probabilities to 1.0
	      //
	      renorm = 0.0;
	      for (k = 0; k < maxhash; k++)
		renorm = renorm + ptc[k];
	      for (k = 0; k < maxhash; k++)
		ptc[k] = ptc[k] / renorm;
	
	      for (k = 0; k < maxhash; k++)
		if (ptc[k] < 10*DBL_MIN) ptc[k] = 10 * DBL_MIN;

	      //if (pnic < pic)
	      //	{ pic = 1.0 - pnic;} else { pnic = 1.0 - pic; };
	      
	      if (internal_trace) 
		{
		  for (k = 0; k < maxhash; k++)
		    {
		      fprintf (stderr, 
		       " feature: %d, HTF: %7.0f, hits: %7.0f, Pl: %6.4e, Pc: %6.4e\n",
			       k, htf, hits[k], pltc[k], ptc[k]);
		    };
		};
	    };
	};
    };      //  end of repeat-the-regex loop
 classify_end_regex_loop:
  
  //  cleanup time!
  //  remember to let go of the fd's and mmaps
  for (k = 0; k < maxhash; k++)
    {
      close (hfds [k]);
      munmap (hashes[k], hashlens[k]);
    };
  //  and let go of the regex buffery      
  crm_regfree (&regcb);

  //   and one last chance to force probabilities into the non-stuck zone
  //
  //  if (pic == 0.0 ) pic = DBL_MIN;
  //if (pnic == 0.0 ) pnic = DBL_MIN;
  for (k = 0; k < maxhash; k++)
    if (ptc[k] < 10*DBL_MIN) ptc[k] = 10 * DBL_MIN;

  
  if (user_trace) 
    {
      for (k = 0; k < maxhash; k++)
	fprintf (stderr, "Probability of match for file %ld: %f\n", k, ptc[k]);
    };
  //
  tprob = 0.0;
  for (k = 0; k < succhash; k++)
    tprob = tprob + ptc[k];
  if (svlen > 0)
    {
      char buf[1024];
      double accumulator;
      double remainder;
      double overall_pR;
      long m;
      buf [0] = '\000';
      accumulator = 10 * DBL_MIN;
      for (m = 0; m < succhash; m++)
	  {
	    accumulator = accumulator + ptc[m];
	  };
      remainder = 10 * DBL_MIN;
      for (m = succhash; m < maxhash; m++)
	if (bestseen != m)
	  {
	    remainder = remainder + ptc[m];
	  };
      overall_pR = log10 (accumulator) - log10 (remainder);

      //   note also that strcat _accumulates_ in stext.
      //  There would be a possible buffer overflow except that _we_ control
      //   what gets written here.  So it's no biggie.

      if (tprob > 0.5000)
	{
	  sprintf (buf, "CLASSIFY succeeds; success probability: %6.4f  pR: %6.4f\n", tprob, overall_pR );
	}
      else
	{
	  sprintf (buf, "CLASSIFY fails; success probability: %6.4f  pR: %6.4f\n", tprob, overall_pR );
	};
      strcat (stext, buf);
      bestseen = 0;
      for (k = 0; k < maxhash; k++)
	if (ptc[k] > ptc[bestseen] ) bestseen = k;
      remainder = 10 * DBL_MIN;
      for (m = 0; m < maxhash; m++)
	if (bestseen != m)
	  {
	    remainder = remainder + ptc[m];
	  };
      sprintf (buf, "Best match to file #%ld (%s) "\
    		    "prob: %6.4f  pR: %6.4f  \n",
	       bestseen,
	       hashname[bestseen],
	       ptc[bestseen],
	       (log10(ptc[bestseen]) - log10(remainder)));
      strcat (stext, buf);
      sprintf (buf, "Total features in input file: %ld\n", totalfeatures); 
      strcat (stext, buf);
      for (k = 0; k < maxhash; k++)
	{
	  long m;
	  remainder = 10 * DBL_MIN;
	  for (m = 0; m < maxhash; m++)
	      if (k != m)
		{
		  remainder = remainder + ptc[m];
		};
	  sprintf (buf, 
		   "#%ld (%s):"\
		   " features: %ld, hits: %ld, prob: %3.2e, pR: %6.2f \n", 
		   k,
		   hashname[k],
		   fcounts[k],
		   totalhits[k],
		   ptc[k], 
		   (log10 (ptc[k]) - log10 (remainder) )  );
	  strcat (stext, buf);
	};
      crm_destructive_alter_nvariable (svrbl, svlen, 
				       stext, strlen (stext));
    };
  
  //
  //  Free the hashnames, to avoid a memory leak.
  //
  for (i = 0; i < maxhash; i++)
    free (hashname[i]);
  if (tprob < 0.5000)
    {
      if (user_trace)
	fprintf (stderr, "CLASSIFY was a FAIL, skipping forward.\n");
      //    and do what we do for a FAIL here
      csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
      csl->aliusstk [csl->mct[csl->cstmt]->nest_level] = -1;
      return (0);
    };
  
  
  //    
  //   all done... if we got here, we should just continue execution
  if (user_trace)
    fprintf (stderr, "CLASSIFY was a SUCCESS, continuing execution.\n");
 regcomp_failed:
  return (0);
};

// crm_trigger_fault - whenever there's a fault, this routine Does The
//  Right Thing, in terms of looking up the next TRAP statement and
//  turning control over to it.
//  
//  Watch out, this routine must return cleanly with the CSL top set up
//  appropriately so execution can continue at the chosen statement.
//  That next statement executed will be a TRAP statement whose regex
//  matches the fault string.  All other intervening statements will
//  be ignored.
//
//  This routine returns 0 if execution should continue, or 1 if there
//  was no applicable trap to catch the fault and we should take the
//  default fault action (which might be to exit).
//
//  Routines that get here must be careful to NOT trash the constructed
//  csl frame that causes the next statement to be the TRAP statement.
//  In particular, we act like the MATCH and CLASSIFY and FAIL statments
//  and branch to the chosen location -1 (due to the increment in the 
//  main invocation loop)

long crm_trigger_fault ( char *reason) 
{
  //
  //    and if the fault is correctly trapped, this is the fixup.
  //
  char trap_pat[MAX_PATTERN];
  long pat_len;
  regex_t preg;
  long i, j;
  ARGPARSE_BLOCK apb;
  long slen;
  long done;
  long trapline;

  //  Non null fault_text rstring -we'll take the trap 
  if (user_trace)
    {
      fprintf (stderr, "Catching FAULT generated on line %ld\n", 
	       csl->cstmt);
      fprintf (stderr, "FAULT reason: \n %s \n",
	       reason );
    };

  trapline = csl->cstmt;

  done = 0;
  while ( ! done)
    {
      if (csl->mct[trapline]->trap_index >= csl->nstmts)
	{
	  if (user_trace)
	    {
	      fprintf (stderr, "     ... no applicable TRAP. \n");
	    }
	  return (1);
	};
	      
      trapline = csl->mct[trapline]->trap_index;

      //      
      //        make sure we're really on a trap statement.
      //
      if (csl->mct[trapline]->stmt_type != CRM_TRAP)
	return (1);

      //       OK, we're here, with a live trap.  
      slen = (csl->mct[trapline+1]->fchar) 
	- (csl->mct[trapline ]->fchar);
      
      i = crm_statement_parse(
		      &(csl->filetext[csl->mct[trapline]->fchar]),
		      slen,
		      &apb);
      if (user_trace)
	{
	  fprintf (stderr, "Trying trap at line %ld: \n", trapline);
	  for (j =  (csl->mct[trapline ]->fchar);
	       j < (csl->mct[trapline+1]->fchar);
	       j++)
	    fprintf (stderr, "%c", csl->filetext[j]);
	  fprintf (stderr, "\n");
	};
      
      //  Get the trap pattern and  see if we match.
      crm_get_pgm_arg (trap_pat, MAX_PATTERN,
		       apb.s1start, apb.s1len);
      //
      //      Do variable substitution on the pattern
      pat_len = crm_nexpandvar (trap_pat, apb.s1len, MAX_PATTERN);
      
      //
      //      if there's no pattern, complain.
      if (pat_len == 0)
	nonfatalerror ("This TRAP statement has a zero length pattern.",
		       "It will never trap anything.");
      if (user_trace)
	{
	  fprintf (stderr, "This TRAP will trap anything matching =%s= . \n", 
		   trap_pat);
	};
      //       compile the regex
      i = crm_regcomp (&preg, trap_pat, pat_len, REG_EXTENDED);
      if ( i > 0)
	{
	  crm_regerror ( i, &preg, tempbuf, data_window_size);
	  fatalerror ("Regular Expression Compilation Problem in TRAP pattern:", 
		      tempbuf);
	  goto trap_regcomp_error;
	};
      //      and run the regexe
      i = crm_regexec (&preg, 
		       reason,
		       strlen (reason),
      //     HACK HACK HACK
      //     We don't actually use "matches", actually
      //     this is a workaround for TRE to not get upset.
		       //     1, matches, 0, NULL);
      //    Note to Ville Laurikari- the following line cause an 
      //    occasional segfault on some (but not all) regexes.
      //     FIXED 20031105 (thanks, Ville.)
		              0, NULL, 0, NULL);


      crm_regfree (&preg);

      
    trap_regcomp_error:
      //    if i != 0, we didn't match - kick the error further
      if (i == 0)
	{
	  if (user_trace)
	    {
	      fprintf (stderr, "TRAP matched.\n");
	      fprintf (stderr, "Next statement will be %ld\n",
		       trapline);
	    }
	  //
	  //   set the next statement to execute to be
	  //   the TRAP statement itself.

	  
	  csl->cstmt = trapline;
	  csl->aliusstk [ csl->mct[csl->cstmt]->nest_level ] = 1;
	  //
	  //     If there's a trap variable, modify it.
	  {
	    char reasonname [MAX_VARNAME];
	    long rnlen;
	    crm_get_pgm_arg(reasonname, MAX_VARNAME, apb.p1start, apb.p1len);
	    if (apb.p1len > 2)
	      {
		rnlen = crm_nexpandvar (reasonname, apb.p1len, MAX_VARNAME);
		//   crm_nexpandvar null-terminates for us so we can be
		//   8-bit-unclean here
		crm_set_temp_nvar (reasonname, 
				   reason,
				   strlen (reason));
	      };
	    done = 1;
	  }
	}
      else
	{
	  //   The trap pattern didn't match - move outward to 
	  //   the surrounding trap (if it exists)
	  if (user_trace)
	    {
	      fprintf (stderr, 
		       "TRAP didn't match- trying next trap in line.\n");
	    };
	};
    };
  return (0);
}

//
//    experimental code for a statement-type-sensitive parser.
//   Not in use yet... but someday... goal is to provide better error 
//   detection.

int crm_profiled_statement_parse ( char *in,
				   long slen,
				   ARGPARSE_BLOCK *apb,
				   long amin, long amax,
				   long pmin, long pmax,
				   long bmin, long bmax,
				   long smin, long smax)
{
  return (0);
}

//      parse a CRM114 statement; this is mostly a setup routine for 
//     the generic parser.

int crm_statement_parse ( char *in, 
			  long slen,
			  ARGPARSE_BLOCK *apb)
{
#define CRM_STATEMENT_PARSE_MAXARG 10
  int i,  k;

  long ftype[CRM_STATEMENT_PARSE_MAXARG];
  long fstart[CRM_STATEMENT_PARSE_MAXARG];
  long flen [CRM_STATEMENT_PARSE_MAXARG];

  //     we call the generic parser with the right args to slice and
  //     dice the incoming statement into declension-delimited parts
  k = crm_generic_parse_line ( in, 
			       slen,
			       "<([/",
			       ">)]/",
			       "\\\\\\\\",      // this is four backslashes
			       CRM_STATEMENT_PARSE_MAXARG,
			       ftype,
			       fstart,
			       flen);

  //   now we have all these nice chunks... we split them up into the
  //   various allowed categories.


  //   start out with empties on each possible chunk
  apb->a1start = NULL; apb->a1len = 0;
  apb->p1start = NULL; apb->p1len = 0; 
  apb->p2start = NULL; apb->p2len = 0;
  apb->p3start = NULL; apb->p3len = 0;
  apb->b1start = NULL; apb->b1len = 0;
  apb->s1start = NULL; apb->s1len = 0;
  apb->s2start = NULL; apb->s2len = 0;

  //   Scan through the incoming chunks
  for (i = 0; i < k; i++)
    {
      switch (ftype[i])
       	{
	case CRM_ANGLES:
	  {   
	    //  Grab the angles, if we don't have one already 
	    if (apb->a1start == NULL)
	      {
		apb->a1start = &in[fstart[i]];
		apb->a1len = flen [i];
	      }
	    else nonfatalerror 
		   ("There are multiple flag sets on this line.",
		    " ignoring all but the first");
	  }
	  break;
	case CRM_PARENS:
	  {
	    //  grab a set of parens, cascading till we find an one
	    if (apb->p1start == NULL)
	      {
		apb->p1start = &in[fstart[i]];
		apb->p1len = flen [i];
	      }
	    else
	      if (apb->p2start == NULL)
		{
		  apb->p2start = &in[fstart[i]];
		  apb->p2len = flen [i];
		}
	      else
		if (apb->p3start == NULL)
		  {
		    apb->p3start = &in[fstart[i]];
		    apb->p3len = flen [i];
		  }
		else
		  nonfatalerror 
		    ("Too many parenthesized varlists.",
		     "ignoring the excess varlists.");
	  }
	  break;
	case CRM_BOXES:
	  {   
	    //  Grab the angles, if we don't have one already 
	    if (apb->b1start == NULL)
	      {
		apb->b1start = &in[fstart[i]];
		apb->b1len = flen [i];
	      }
	    else nonfatalerror 
		   ("There are multiple domain limits on this line.",
		    " ignoring all but the first");
	  }
	  break;
	case CRM_SLASHES:
	  {
	    //  grab a set of parens, cascading till we find an one
	    if (apb->s1start == NULL)
	      {
		apb->s1start = &in[fstart[i]];
		apb->s1len = flen [i];
	      }
	    else
	      if (apb->s2start == NULL)
		{
		  apb->s2start = &in[fstart[i]];
		  apb->s2len = flen [i];
		}
	      else
		nonfatalerror (
		       "There are too many regex sets in this statement,",
	          " ignoring all but the first.");
	  }
	  break;
	default:
	  fatalerror( "Declensional parser returned an undefined typecode!",
		      "What the HECK did you do to cause this?");
	};
    }
  return (k);    // return value is how many declensional arguments we found.
};


//     The new and improved line core parser routine.  Instead of
//     being totally ad hoc, this new parser actually retains context
//     durng the parse.
//
//     this hopefully will keep the parser from getting confused by [] in
//     the slash matching and other such abominations.
//
//     (one way to view this style of parsing is that each arg in a 
//     CRM114 statement is "declined" by it's delimiters to determine 
//     what role this variable is to play in the statement.  Kinda like
//     Latin - to a major extent, you can mix the parts around and it 
//     won't make any difference.

int crm_generic_parse_line ( 
		    char *txt,       //   the start of the program line
		    long len,        //   how long is the line
		    char *schars,    //   characters that can "start" an arg
		    char *fchars,    //   characters that "finish" an arg
		    char *echars,    //   characters that escape in an arg
		    long maxargs,    //   howm many things to search for (max)
		    long *ftype,     //   type of thing found (index by schars)
		    long *fstart,    //   starting location of found arg
		    long *flen       //   length of found arg
		    )
{
  //    the general algorithm here is to move along the input line,
  //    looking for one of the characters in schars.  When we find it,
  //    we lock onto that and commit to finding an arg of that type.
  //    We then start scanning ahead keeping count of schars minus echars.
  //    when the count hits zero, it's end for that arg and we move onward
  //    to the next arg, with the same procedure.  
  //
  //    note that when we are scanning for a new arg, we are open to args
  //    of any type (as defined by the members of schars, while in an arg
  //    we are looking only for the unescaped outstanding echar and are blind
  //    to everything else.  
  //    
  //    when not in an arg, we do not have any escape character active.
  //    
  //     We return the number of args found

  long chidx;
  char curchar;
  long argc;
  long i; 
  long itype;
  long depth;

  //    zeroize the outputs to start...
  for (i = 0; i < maxargs; i++)
    { 
      ftype[i] = -1;
      fstart[i] = 0;
      flen[i] = 0;
    };
    

  //    scan forward, looking for any member of schars

  depth = 0;
  chidx = -1;
  argc = 0;
  itype = -1;

  if (internal_trace)
    {
      fprintf (stderr, " declensional parsing for %ld chars on: ", len);
      for (i = 0; i < len; i++)
	fprintf (stderr, "%c", txt[i]);
      fprintf (stderr, "\n");
    }

  while (chidx < len  &&  argc <= maxargs)
    {
      chidx++;
      curchar = txt[chidx];
      if (itype == -1)     // are we looking for an argstart char?
	{
	  //    is curchar one of the start chars?  (this is 8-bit-safe,
	  //     because schars is always normal ASCII)
	  for (i = 0; i < strlen (schars); i++)
	    if (curchar == schars[i])
	      {
		if (internal_trace)
		  fprintf (stderr, "   found opener %c at %ld,",curchar,chidx);
		itype = i;
		fstart[argc] = chidx + 1;
		ftype [argc] = itype;
		depth = 1;
	      };
	  //  if it wasn't a start-character for an arg, we are done.
	}
      else    // nope, we're in an arg, so we check for unescaped schar
	     // and fchar characers
	{
	  if (curchar == fchars [itype] && txt[chidx-1] != echars[itype])
	    {
	      depth--;
	      if (depth == 0)
		{
		  //   we've found the end of the text arg.  Close it off and
		  //   note it into the output vectors
		  flen [argc] = chidx - fstart[argc] ;
		  if (internal_trace)
		    {
		      int q;
		      fprintf (stderr, " close %c at %ld --", curchar, chidx);
		      for (q = fstart[argc]; q < fstart[argc]+flen[argc]; q++)
			fprintf (stderr, "%c", txt[q]);
		      fprintf (stderr, "-- len %ld\n", flen[argc]);
		    };
		  itype = -1;
		  argc++;
		};
	    }
	  else
	    if (curchar == schars [itype] && txt[chidx-1] != echars[itype])
	      {
		depth++;
	      };
	};
      //    if we weren't a schar or an unexcaped echar, we're done!
    };
  if (depth != 0)
    {
      char errstmt[MAX_PATTERN];
      flen[argc] = chidx - fstart[argc];
      strncpy ( errstmt, &txt[fstart[argc]], 
		flen[argc] );
      nonfatalerror (" This operand doesn't seem to end.  Bug?  \n -->  ",
		     errstmt);
      argc++;
    };
  return (argc);
}

//    and to avoid all the mumbo-jumbo, an easy way to get a copy of
//    an arg found by the declensional parser.
void crm_get_pgm_arg (char *to, long tolen, char *from, long fromlen) 
{
  long len;
  
  if (to == NULL)
    return;

  if (from == NULL)
    {
      to[0] = '\000';
    }
  else
    {
      len = tolen - 1;
      if (len > fromlen ) len = fromlen ;
      memmove (to, from, len);
      to[len] = '\000';
    }
}

//         If we got to here, we need to run some user-interfacing
//         (i.e. debugging).
//
//         possible return values:
//         1: reparse and continue execution
//         -1: exit immediately
//         0: continue

long crm_debugger ()
{
  long ichar;
  static int firsttime = 1;
  static FILE *mytty;

  if (firsttime)
    {
      fprintf (stderr, "CRM114 Debugger - type \"h\" for help.  ");
      fprintf (stderr, "User trace turned on.\n");
      user_trace = 1;
      firsttime = 0;
      mytty = fopen ("/dev/tty", "r");
      clearerr (mytty);
    };

  if (csl->mct[csl->cstmt]->stmt_break > 0)
    fprintf (stderr, "Breakpoint tripped at statement %ld\n", csl->cstmt);
 debug_top:
  //    let the user know they're in the debugger
  //
  fprintf (stderr, "\ncrm-dbg> ");

  //    find out what they want to do
  //
  ichar = 0;

  while (!feof (mytty) 
	 && ichar < data_window_size - 1 
	 && (inbuf [ichar-1] != '\n' ) )	    
    {
      inbuf[ichar] = fgetc (mytty);
      ichar++;
    };  
  inbuf [ichar] = '\000';

  if (feof (mytty) )  
    {
      fprintf (stderr, "Quitting\n");
      exit ( EXIT_SUCCESS );
    };


  //   now a big siwtch statement on the first character of the command
  //
  switch (inbuf[0])
    {
    case 'q':
    case 'Q':
      {
	if (user_trace)
	  fprintf (stderr, "Quitting.\n");
	exit ( EXIT_SUCCESS );
      };
      break;
    case 'n':
    case 'N':
      {
	debug_countdown = 0;
	return (0);
      }
      break;
    case 'c':
    case 'C':
     {
       sscanf (&inbuf[1], "%ld", &debug_countdown);
       if (debug_countdown <= 0) 
	 {
	   debug_countdown = -1;
	   fprintf (stderr, "continuing execution...\n");
	 }
       else
	 {
	   fprintf (stderr, "continuing %ld cycles...\n", debug_countdown);
	 };
       return (0);
     };
     break;
    case 't':
      if (user_trace == 0 ) 
	{
	  user_trace = 1 ;
	  fprintf (stderr, "User tracing on");
	}
      else
	{
	  user_trace = 0;
	  fprintf (stderr, "User tracing off");
	};
      break;
    case 'T':
      if (internal_trace == 0 ) 
	{
	  internal_trace = 1 ;
	  fprintf (stderr, "Internal tracing on");
	}
      else
	{
	  internal_trace = 0;
	  fprintf (stderr, "Internal tracing off");
	};
      break;
    case 'e':
      {
	fprintf (stderr, "expanding: \n");
	memmove (inbuf, &inbuf[1], strlen (inbuf) -1);
	crm_nexpandvar (inbuf, strlen(inbuf) -1, data_window_size);
	fprintf (stderr, "%s", inbuf);
      };
      break;
    case 'i':
      {
	fprintf (stderr, "Isolating %s", &inbuf[1]);
	fprintf (stderr, "NOT YET IMPLEMENTED!  Sorry.  \n");
      }
      break;
    case 'v':
      {
	long i, j;
	long stmtnum;
	i = sscanf (&inbuf[1], "%ld", &stmtnum);
	if (i > 0)
	  {
	    fprintf (stderr, "statement %ld: ", stmtnum);
	    if ( stmtnum < 0 || stmtnum > csl->nstmts)
	      {
		fprintf (stderr, "... does not exist!\n");
	      }
	    else
	      {
		for ( j = csl->mct[stmtnum]->start;
		      j < csl->mct[stmtnum+1]->start;
		      j++)
		  {
		    fprintf (stderr, "%c", csl->filetext[j]);
		  };
		
	      };
	  }
	else
	  {
	    fprintf (stderr, "What statement do you want to view?\n");
	  }
      };
      break;
    case 'j':
      {
	long nextstmt;
	long i;
	long vindex;
	i = sscanf (&inbuf[1], "%ld", &nextstmt);
	if (i == 0)
	  {
	    //    maybe the user put in a label?
	    long tstart;
	    long tlen;
	    crm_nextword (&inbuf[1], strlen (&inbuf[1]), 0,
			  &tstart, &tlen);
	    memmove (inbuf, &inbuf[1+tstart], tlen);
	    inbuf[tlen] = '\000';
	    vindex= crm_vht_lookup (vht, inbuf, tlen);
	    if (vht[vindex] == NULL)
	      {
		fprintf (stderr, "No label '%s' in this program.  ", inbuf);
		fprintf (stderr, "Staying at line %ld\n", csl->cstmt);
		nextstmt = csl->cstmt;
	      }
	    else
	      {
		nextstmt = vht[vindex]->linenumber;
	      };
	  };
	if (nextstmt <= 0) 
	  {
	    nextstmt = 0;
	  }
	if (nextstmt >= csl->nstmts)
	  {
	    nextstmt = csl-> nstmts;
	    fprintf (stderr, "last statement is %ld, assume you meant that.\n",
		     csl->nstmts);
	  };
	if (csl->cstmt != nextstmt)
	  fprintf (stderr, "Next statement is statement %ld\n", nextstmt);
	csl->cstmt = nextstmt;
      }
      return (1);
      break;
    case 'b':
      {                  //  is there a breakpoint to make?
	long breakstmt;
	long i;
	long vindex;
	breakstmt = -1;
	i = sscanf (&inbuf[1], "%ld", &breakstmt);
	if (i == 0)
	  {
	    //    maybe the user put in a label?
	    long tstart;
	    long tlen;
	    crm_nextword (&inbuf[1], strlen (&inbuf[1]), 0,
			  &tstart, &tlen);
	    memmove (inbuf, &inbuf[1+tstart], tlen);
	    inbuf[tlen] = '\000';
	    vindex= crm_vht_lookup (vht, inbuf, tlen);
	    fprintf (stderr, "vindex = %ld\n", vindex);
	    if (vht[vindex] == NULL)
	      {
		fprintf (stderr, "No label '%s' in this program.  ", inbuf);
		fprintf (stderr, "No breakpoint change made. \n");
	      }
	    else
	      {
		breakstmt = vht[vindex]->linenumber;
	      };
	  };
	if (breakstmt  <= -1) 
	  {
	    breakstmt = 0;
	  }
	if (breakstmt >= csl->nstmts)
	  {
	    breakstmt = csl-> nstmts;
	    fprintf (stderr, "last statement is %ld, assume you meant that.\n",
		     csl->nstmts);
	  };
	csl->mct[breakstmt]->stmt_break = 1 - csl->mct[breakstmt]->stmt_break;
	if (csl->mct[breakstmt]->stmt_break == 1)
	  {
	    fprintf (stderr, "Setting breakpoint at statement %ld\n", 
		     breakstmt);
	  }
	else
	  {
	    fprintf (stderr, "Clearing breakpoint at statement %ld\n", 
		     breakstmt);
	  };
      }
      return (1);
      break;
    case 'a':
      {                  //  do a debugger-level alteration
	//    maybe the user put in a label?
	long vstart, vlen;
	long vindex;
	long ostart, oend, olen;
	crm_nextword (&inbuf[1], strlen (&inbuf[1]), 0, 
		      &vstart, &vlen);
	memmove (inbuf, &inbuf[1+vstart], vlen);
	inbuf[vlen] = '\000';
	vindex= crm_vht_lookup (vht, inbuf, vlen);
	if (vht[vindex] == NULL)
	  {
	    fprintf (stderr, "No variable '%s' in this program.  ", inbuf);
	  }
	
	//     now grab what's left of the input as the value to set
	//
	ostart = vlen + 1;
	while (inbuf[ostart] != '/' && inbuf[ostart] != '\000') ostart++;
	ostart++;
	oend = ostart + 1;
	while (inbuf[oend] != '/' && inbuf[oend] != '\000') oend++;

	memmove (outbuf, 
		 &inbuf[ostart],
		 oend - ostart);

	outbuf [oend - ostart] = '\000';
	olen = crm_nexpandvar (outbuf, oend - ostart, data_window_size);
	crm_destructive_alter_nvariable (inbuf, vlen, outbuf, olen);
      };
      break;
    case 'f':
      {
	csl->cstmt = csl->mct[csl->cstmt]->fail_index - 1;
	fprintf (stderr, "Forward to }, next statement : %ld\n", csl->cstmt);
	csl->aliusstk [csl->mct[csl->cstmt]->nest_level] = -1;
      };
      return (1);
      break;
    case 'l':
      {
	csl->cstmt = csl->mct[csl->cstmt]->liaf_index;
	fprintf (stderr, "Backward to {, next statement : %ld\n", csl->cstmt);
      };
      return (1);
      break;
    case 'h':
      {
	fprintf (stderr, "a :var: /value/ - alter :var: to /value \n");
	fprintf (stderr, "b <n> - toggle breakpoint on line <n> \n");
	fprintf (stderr, "b <label> - toggle breakpoint on <label> \n");
	fprintf (stderr, "c - continue execution till breakpoint or end\n");
	fprintf (stderr, "c <n> - execute <number> more statements\n");
	fprintf (stderr, "e - expand an expression\n");
	fprintf (stderr, "f - fail forward to block-closing '}'\n");
	fprintf (stderr, "j <n> - jump to statement <number>\n");
	fprintf (stderr, "j <label> - jump to statement <label>\n");
	fprintf (stderr, "l - liaf backward to block-opening '{'\n");
	fprintf (stderr, "n - execute next statement (same as 'c 1')\n");
	fprintf (stderr, "q - quit the program and exit\n");
	fprintf (stderr, "t - toggle user-level tracing\n");
	fprintf (stderr, "T - toggle system-level tracing\n");
	fprintf (stderr, "v <n> - view source code statement <n>\n");
      };
      break;
    default:
      {
	fprintf (stderr, "Command unrecognized - type \"h\" for help. \n");
      };
    };
  goto debug_top;
}
